// ************************************************************************* //
/// GBB - Graphics Blue Box, by Alexis Angelidis
/// $Author: silex $
/// $Date: 2006/01/11 21:56:28 $
/// $Revision: 1.1 $
// ************************************************************************* //
#include <fstream>
#include <qfiledialog.h>
#include "qtAttributeEditor.h"
#include "qtCurveEditor.h"
#include "qtMainWidget.h"
#include "qtClockEditor.h"
#include "commandcustom.h"
#include "dagnode.h"
#include "datablock.h"
#include "plug.h"
#include "nodecurve.h"
#include "nodetransform.h"
#include "nodegizmo.h"
#include "nodesmokeparticles.h"
#include "matmnd.h"
#include "gbb.h"
#include "mystring.h"
 
// ************************************************************************* //
int	CmdSelect::selectIndex(0);
int	CmdSelect::id(56);
String	CmdSelect::name("select");
int	CmdTranslate::id(58);
String	CmdTranslate::name("translate");
int	CmdNew::id(59);
String	CmdNew::name("new");
int	CmdDelete::id(64);
String	CmdDelete::name("delete");
int	CmdDeselect::id(61);
String	CmdDeselect::name("deselect");
int	CmdSet::id(62);
String	CmdSet::name("set");
int	CmdConnect::id(63);
String	CmdConnect::name("connect");
int	CmdSave::id(65);
String	CmdSave::name("save");
int	CmdExport::id(83);
String	CmdExport::name("export");
int	CmdLoad::id(66);
String	CmdLoad::name("load");
int	CmdNewScene::id(67);
String	CmdNewScene::name("newscene");
int	CmdSetKey::id(68);
String	CmdSetKey::name("setkey");
int	CmdRemKey::id(69);
String	CmdRemKey::name("remkey");
int	CmdEdit::id(70);
String	CmdEdit::name("edit");
int	CmdParent::id(71);
String	CmdParent::name("parent");
int	CmdDeformer::id(72);
String	CmdDeformer::name("deformer");
int	CmdControlGenerator::id(73);
String	CmdControlGenerator::name("controlgenerator");
int	CmdBakeObstacle::id(74);
String	CmdBakeObstacle::name("bakeobstacle");
int	CmdBindVortexGenerator::id(75);
String	CmdBindVortexGenerator::name("bindvortexgenerator");
int	CmdPassItOn::id(76);
String	CmdPassItOn::name("passiton");
int	CmdConstraintCurves::id(77);
String	CmdConstraintCurves::name("constraintcurves");
int	CmdNewCoonsPatch::id(78);
String	CmdNewCoonsPatch::name("newcoonspatch");
int	CmdNewCamera::id(79);
String	CmdNewCamera::name("newcamera");
int	CmdAttachControl::id(80);
String	CmdAttachControl::name("attachcontrol");
int	CmdNewMesh::id(81);
String	CmdNewMesh::name("newmesh");
int	CmdEditCurve::id(82);
String	CmdEditCurve::name("editcurve");

// ************************************************************************* //
CmdDeselect::CmdDeselect()
{
}

// ************************************************************************* //
/// DESCRIPTION: Deselects all the objects.
// ************************************************************************* //
DagNode	*CmdDeselect::execute(const String &)//params
{
  CmdSelect::selectIndex = 0;
  DagNode::root().deselect();
  DagNode	*gizmo = DagNode::getNodeByPath(":Gizmo0");

  gizmo->writeAsBool("visible", false);
  DagNode::disconnect(*gizmo, "transform");
  return 0;
}

// ************************************************************************* //
CmdSelect::CmdSelect(): Command()
{
  addSignal("multiple");
  addInt(0, "level");
  addString("path");
}

// ************************************************************************* //
void	CmdSelect::connectGizmoTo(DagNode &dn)
{
  DagNode	*gizmo = DagNode::getNodeByPath(":Gizmo0");
  DagNode	*tr(dn.getNodeParentByType("Transform"));

  if (tr)
    {
      DagNode::connect(*gizmo, "transform", *tr, "signal");
      gizmo->writeAsBool("visible", true);
    }
}

// ************************************************************************* //
void	CmdSelect::disconnectGizmoFrom(DagNode &dn)
{
  DagNode	*gizmo = DagNode::getNodeByPath(":Gizmo0");
  DagNode	*tr(dn.getNodeParentByType("Transform"));

  if (tr)
    {
      DagNode::disconnect(*gizmo, "transform", *tr, "signal");
      //gizmo->writeAsBool("visible", true);
    }
}

// ************************************************************************* //
void	CmdSelect::disconnectGizmoFromAll()
{
  DagNode	*gizmo = DagNode::getNodeByPath(":Gizmo0");

  gizmo->writeAsBool("visible", false);
  DagNode::disconnect(*gizmo, "transform");
}

// ************************************************************************* //
/// DESCRIPTION: Selects the object.
/// Arguments:
///  "-none" or node path or path + data.
///  -level 0, 1, 2 level of selection.
///  "-0"    level 0, at Node
///  "-1"    level 1, at Plug (if selectable)
/// Path syntax, from root, is  :Node1:Node2:Node3:Node4[...]
/// Plug syntax, from root, is  :Node1:Node2:Node3:Node4.Data
/// If a piece of the gizmo is clicked, nothing must be done here.
// ************************************************************************* //
DagNode	*CmdSelect::execute(const String &params)
{
  String	errorMsg;
  Plug		*hnd = 0;
  DagNode	*dn;
  std::list<DagNode*> nodes;
  std::list<DagNode*>::iterator	nit;

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }
  if ((dn = DagNode::getNodeByPath(readAsString("path").chopBack('.'))) &&
      (dn->getTypeStr() == "Gizmo"))
    return 0;
  if (readAsInt("level") == 0) // select node
    {
      if ((dn = DagNode::getNodeByPath(readAsString("path").chopBack('.'))) &&
	  (hnd = dn->getPlug("sel")))
	{
	  if (readAsSignal("multiple"))
	    {
	      if (hnd->readAsInt() == 0)
		{
		  DagNode::getNodesSelected(nodes);
		  for (nit = nodes.begin(); nit != nodes.end(); ++nit)
		    (*nit)->writeAsInt("sel", (*nit)->readAsInt("sel") + 1);
		  hnd->writeAsInt(1);
		  ++selectIndex;
		  connectGizmoTo(*dn);
		}
	      else
		{
		  dn->deselect();
		  DagNode::getNodesSelected(nodes);
		  --CmdSelect::selectIndex;
		  disconnectGizmoFrom(*dn);
		}
	    }
	  else
	    {
	      DagNode::getNodesSelected(nodes);
	      disconnectGizmoFromAll();
	      if (hnd->readAsInt() == 0)//disconnect
		{
		  hnd->writeAsInt(1);
		  selectIndex = 1;
		  connectGizmoTo(*dn);
		}
	      for (nit = nodes.begin(); nit != nodes.end(); ++nit)
		(*nit)->deselect();
	    }
	}
      return 0;
    }
  else if (readAsInt("level") == 1) // select data
    {
      if ((dn = DagNode::getNodeByPath(readAsString("path").chopBack('.'))) &&
	  (hnd = dn->getPlug("sel")))
	{
	  const String	dataName(params.chopFront('.'));

	  if (dataName.empty())
	    {
	      std::cerr << "Error: data expected as parameter" << std::endl;
	      return 0;
	    }
	  if (hnd->readAsInt() == 0)
	    hnd->writeAsInt(++selectIndex);

	  TPlug	*data = static_cast<TPlug*>(dn->getPlug(dataName));

	  if (!data)
	    {
	      std::cerr << "Error: no such data as " << dataName << std::endl;
	      return 0;
	    }
	  data->select();
	}
      return 0;
    }
  return 0;
}

// ************************************************************************* //
CmdTranslate::CmdTranslate(): Command()
{
  addPnt3d("p0");
  addPnt3d("p1");
}

// ************************************************************************* //
/// DESCRIPTION: Applies translation to selected pieces of data of type 
///  Pnt3d.
/// TODO: translate a list of selected
// ************************************************************************* //
DagNode	*CmdTranslate::execute(const String &params)
{
  String	errorMsg;

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }

  std::list<Plug *>	hnd, hnda;
  bool	isAttributeSelected(false);
  std::list<DagNode *>		nodes;
  std::list<DagNode*>::iterator	nit;
  const Pnt3d	p0(readAsPnt3d("p0"));
  const Pnt3d	p1(readAsPnt3d("p1"));
  DagNode	*gizmo(DagNode::getNodeByPath(":Gizmo0"));
  
  if (gizmo && gizmo->readAsBool("visible"))
    {
      gizmo->writeAsPnt3d("p0", p0);
      gizmo->writeAsPnt3d("p1", p1);
      gizmo->readAsSignal("update");
    }
  else
    {
      DagNode::getNodesSelected(nodes);
      for (nit = nodes.begin(); nit != nodes.end(); ++nit)
	{
	  DataBlock::iterator	dit;

	  for (dit = (*nit)->getDataBlock().begin();
	       dit != (*nit)->getDataBlock().end(); ++dit)
	    {
	      std::list<Plug*>			hnd;
	      std::list<Plug *>::iterator	hndit;

	      (*dit)->getPlugByType(ID_PNT3D, hnd);
	      for (hndit = hnd.begin(); hndit != hnd.end(); ++hndit)
		if ((*hndit)->isSelected())
		  {
		    const Vec3d	t(p1 - p0);
		    DagNode	*n((*nit)->getNodeParentByType("Transform"));

		    if (n)
		      {
			const Mat4d	&m = n->readAsMat4d("matChild");

			(*hndit)->writeAsPnt3d((*hndit)->readAsPnt3d() + 
					       m.inverse() * t);
		      }
		    else
		      (*hndit)->writeAsPnt3d((*hndit)->readAsPnt3d() + t);
		    isAttributeSelected = true;
		  }
	    }
	  if (!isAttributeSelected)
	    if ((*nit)->getNodeParentByType("Transform"))
	      {
		const Vec3d	t(p1 - p0);
		DagNode		&n(*(*nit)->getNodeParentByType("Transform"));

		n.writeAsDouble("tx", n.readAsDouble("tx") + t.x);
		n.writeAsDouble("ty", n.readAsDouble("ty") + t.y);
		n.writeAsDouble("tz", n.readAsDouble("tz") + t.z);
	      }
	}
    }
  return 0;
}

// ************************************************************************* //
CmdNew::CmdNew(): Command()
{
  addString("type");
  addString(":", "path");
}

// ************************************************************************* //
/// DESCRIPTION:  new <Type> <Node parent>
///  if no parent node is specified, attaches the node to the scene.
// ************************************************************************* //
DagNode	*CmdNew::execute(const String &params)
{
  String	errorMsg;

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }

  if (!DagNode::queryCreator(DagNode::names2id()[readAsString("type")]))
    {
      std::cerr << "Error: invalid type" << std::endl;
      return 0;
    }

  DagNode *child =
    DagNode::queryCreator(DagNode::names2id()[readAsString("type")])();
  DagNode *parent = DagNode::getNodeByPath(readAsString("path"));

  if (!parent)
    {
      std::cerr << "Error: invalid path" << std::endl;
      return 0;
    }
  DagNode::parent(*parent, *child);
  child->createInstName();
  child->initialize();
  return child;
}

// ************************************************************************* //
CmdDelete::CmdDelete(): Command()
{
  addString("path");
}

// ************************************************************************* //
/// DESCRIPTION:  delete -path <path>
///  if no parent node is specified, attaches the node to the scene.
// ************************************************************************* //
DagNode	*CmdDelete::execute(const String &params)
{
  String	errorMsg;

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }

  DagNode *deletee = DagNode::getNodeByPath(readAsString("path"));
  
  if (deletee)
    delete deletee;
  else
    std::cerr << "Error: invalid path" << std::endl;
  return 0;
}

// ************************************************************************* //
CmdConnect::CmdConnect(): Command()
{
  addString("src");
  addString("dst");
}

// ************************************************************************* //
/// DESCRIPTION:  new <Type> <Node parent>
///  if no parent node is specified, attaches the node to the scene.
// ************************************************************************* //
DagNode	*CmdConnect::execute(const String &params)
{
  String	errorMsg;

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }
  DagNode	*src = DagNode::getNodeByPath
    (readAsString("src").chopBack('.'));
  DagNode	*dst = DagNode::getNodeByPath
    (readAsString("dst").chopBack('.'));

  if (src && dst)
    DagNode::connect(*src, readAsString("src").chopFront('.'),
		     *dst, readAsString("dst").chopFront('.'));
  return 0;
}

// ************************************************************************* //
CmdSet::CmdSet(): Command()
{
  addString("path");
  addString("value");
}

// ************************************************************************* //
/// DESCRIPTION:  new <Type> <Node parent>
///  if no parent node is specified, attaches the node to the scene.
// ************************************************************************* //
DagNode	*CmdSet::execute(const String &params)
{
  String	errorMsg;

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }

  DagNode *node = DagNode::getNodeByPath(readAsString("path").chopBack('.'));

  if (!node)
    {
      std::cerr << "Error: node " << readAsString("path").chopBack('.')
		<< " does not exists" << std::endl;
      return 0;
    }

  Plug	*data = node->getPlug(readAsString("path").chopFront('.'));

  if (!data)
    {
      std::cerr << "Error: " << readAsString("path").chopFront('.')
		<< " unknown data attribute" << std::endl;
      return 0;
    }
  if (data->getTypeNbr() == ID_INT)
    data->writeAsInt(readAsString("value").asInt());
  else if (data->getTypeNbr() == ID_DOUBLE)
    data->writeAsDouble(readAsString("value").asDouble());
  else
    std::cerr << "Error: data type cannot be set." << std::endl;
  return 0;
}

// ************************************************************************* //
CmdSetKey::CmdSetKey(): Command()
{
  addString("path", 0);
}

// ************************************************************************* //
void	CmdSetKey::setKeyToPlug(Plug &d) const
{
  DagNode	*clock = DagNode::getNodeByPath(":Clock0");
  DagNode	&nodeOfData(d.getNode());
  DagNode	*cNode;

  const double vx(clock->readAsDouble("fd"));
  const double vy(*static_cast<double*>(*d.rawdata()));

  if (!(cNode = d.getInputNode()))
    {
      cNode = Command::run(String("new -type Curve -path ") +
			   nodeOfData.path());

      DagNode::connect(*clock, "fd", *cNode, "x");
      DagNode::connect(*cNode->getPlug("y"), d);
    }
  static_cast<NCurve*>(cNode)->setkey(Pnt2d(vx, vy));
}

// ************************************************************************* //
/// DESCRIPTION:  new <Type> <Node parent>
///  if no parent node is specified, attaches the node to the scene.
// ************************************************************************* //
DagNode	*CmdSetKey::execute(const String &params)
{
  String	errorMsg;

  if (!browse(params, errorMsg))// No parameter found. Use selected object
    {
      std::list<DagNode*> nodes;
      std::list<DagNode*>::iterator nit;

      DagNode::getNodesSelected(nodes);
      for (nit = nodes.begin(); nit != nodes.end(); ++nit)
	{
	  DagNode	&n(**nit);
	  bool	isDataSelected(false);

	  std::list<Plug *>	hnd;
	  DataBlock::iterator	dit;
	  std::list<Plug *>::iterator	hndit, hndjt;

	  for (dit = n.getDataBlock().begin();
	       dit != n.getDataBlock().end(); ++dit)
	    {
	      std::list<Plug *>		hnd;

	      (*dit)->getPlugByType(ID_PNT3D, hnd);
	      for (hndit = hnd.begin(); hndit != hnd.end(); ++hndit)
		if ((*hndit)->isSelected())
		  {
		    std::list<Plug *>	subhnd;

		    (*hndit)->getPlugByType(ID_DOUBLE, subhnd);
		    for (hndjt = subhnd.begin(); hndjt != subhnd.end();
			 ++hndjt)
		      {
			setKeyToPlug(**hndjt);
			isDataSelected = true;
		      }
		  }
	    }
	  if (!isDataSelected)
	    if (n.getNodeParent())
	      if (n.getNodeParent()->getTypeNbr() == NTransform::typeNbr)
		{
		  setKeyToPlug(*n.getNodeParent()->getPlug("tx"));
		  setKeyToPlug(*n.getNodeParent()->getPlug("ty"));
		  setKeyToPlug(*n.getNodeParent()->getPlug("tz"));
		}
	}
    }
  else
    {
      Plug *plug = DagNode::getPlugByPath(readAsString("path"));

      if (plug == 0)
	{
	  std::cerr << "Error: " << readAsString("path")
		    << " is not a plug." << std::endl;
	  return 0;
	}
      else
	{
	  std::cout << "Setting key to " << readAsString("path") << std::endl;
	  if (plug->getTypeStr() == PlugRgb3d::typeStr)
	    {
	      Plug *plugr = DagNode::getPlugByPath(readAsString("path") +".r");
	      Plug *plugg = DagNode::getPlugByPath(readAsString("path") +".g");
	      Plug *plugb = DagNode::getPlugByPath(readAsString("path") +".b");

	      setKeyToPlug(*plugr);
	      setKeyToPlug(*plugg);
	      setKeyToPlug(*plugb);
	    }
	  else
	    setKeyToPlug(*plug);
	}
    }
  return 0;
}

// ************************************************************************* //
CmdRemKey::CmdRemKey(): Command()
{
  addString("path", 0);
}

// ************************************************************************* //
void	CmdRemKey::remKeyToPlug(Plug &d) const
{
  DagNode	*cNode;
  DagNode	*clock = DagNode::getNodeByPath(":Clock0");
  const int	f(clock->readAsInt("f"));

  if ((cNode = d.getInputNode()))
    static_cast<NCurve*>(cNode)->remkey(f);
}

// ************************************************************************* //
/// DESCRIPTION:  new <Type> <Node parent>
///  if no parent node is specified, attaches the node to the scene.
// ************************************************************************* //
DagNode	*CmdRemKey::execute(const String &params)
{
  String	errorMsg;

  if (!browse(params, errorMsg))// No parameter found. Use selected object
    {
      std::list<DagNode*> nodes;
      std::list<DagNode*>::iterator nit;

      DagNode::getNodesSelected(nodes);
      for (nit = nodes.begin(); nit != nodes.end(); ++nit)
	{
	  DagNode	&n(**nit);
	  bool	isDataSelected(false);

	  std::list<Plug *>	hnd;
	  DataBlock::iterator	dit;
	  std::list<Plug *>::iterator	hndit, hndjt;

	  for (dit = n.getDataBlock().begin();
	       dit != n.getDataBlock().end(); ++dit)
	    {
	      std::list<Plug *>		hnd;

	      (*dit)->getPlugByType(ID_PNT3D, hnd);
	      for (hndit = hnd.begin(); hndit != hnd.end(); ++hndit)
		if ((*hndit)->isSelected())
		  {
		    std::list<Plug *>	subhnd;

		    (*hndit)->getPlugByType(ID_DOUBLE, subhnd);
		    for (hndjt = subhnd.begin(); hndjt != subhnd.end();
			 ++hndjt)
		      {
			remKeyToPlug(**hndjt);
			isDataSelected = true;
		      }
		  }
	    }
	  if (!isDataSelected)
	    if (n.getNodeParent())
	      if (n.getNodeParent()->getTypeNbr() == NTransform::typeNbr)
		{
		  remKeyToPlug(*n.getNodeParent()->getPlug("tx"));
		  remKeyToPlug(*n.getNodeParent()->getPlug("ty"));
		  remKeyToPlug(*n.getNodeParent()->getPlug("tz"));
		}
	}
    }
  else
    {
      Plug *plug = DagNode::getPlugByPath(readAsString("path"));

      if (plug == 0)
	{
	  std::cerr << "Error: " << readAsString("path")
		    << " is not a plug." << std::endl;
	  return 0;
	}
      else
	{
	  std::cout << "Deleting key to " << readAsString("path") << std::endl;
	  if (plug->getTypeStr() == PlugRgb3d::typeStr)
	    {
	      Plug *plugr = DagNode::getPlugByPath(readAsString("path") +".r");
	      Plug *plugg = DagNode::getPlugByPath(readAsString("path") +".g");
	      Plug *plugb = DagNode::getPlugByPath(readAsString("path") +".b");

	      remKeyToPlug(*plugr);
	      remKeyToPlug(*plugg);
	      remKeyToPlug(*plugb);
	    }
	  else
	    remKeyToPlug(*plug);
	}
    }
  return 0;
}

// ************************************************************************* //
CmdEdit::CmdEdit(): Command()
{
  addString("path");
}

// ************************************************************************* //
/// DESCRIPTION:  new <Type> <Node parent>
///  if no parent node is specified, attaches the node to the scene.
// ************************************************************************* //
DagNode	*CmdEdit::execute(const String &params)
{
  String	errorMsg;

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }
  DagNode	*dn;
  
  if ((dn = DagNode::getNodeByPath(readAsString("path").chopBack('.'))))
    {
      AttributeEditor *newf = new AttributeEditor(*dn);
      newf->show();
    }
  else
    std::cerr << "Error: " << readAsString("path")
	      << " does not exist." << std::endl;
  return 0;
}

// ************************************************************************* //
CmdEditCurve::CmdEditCurve(): Command()
{
  addString("path");
}

// ************************************************************************* //
/// DESCRIPTION:  new <Type> <Node parent>
///  if no parent node is specified, attaches the node to the scene.
// ************************************************************************* //
DagNode	*CmdEditCurve::execute(const String &params)
{
  String	errorMsg;

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }
  DagNode	*dn;
  
  if ((dn = DagNode::getNodeByPath(readAsString("path").chopBack('.'))))
    {
      CurveEditor *newf = new CurveEditor(*dn);
      newf->show();
      // TODO: get the right one!!!
      //
    }
  else
    std::cerr << "Error: " << readAsString("path")
	      << " does not exist." << std::endl;
  return 0;
}

// ************************************************************************* //
CmdSave::CmdSave()
{
  addString("file");
}

// ************************************************************************* //
/// DESCRIPTION: Selects the object.
/// Arguments:
///  "-none" or node path or path + data.
///  -level 0, 1, 2 level of selection.
///  "-0"    level 0, at Node
///  "-1"    level 1, at Plug (if selectable)
/// Path syntax, from root, is  :Node1:Node2:Node3:Node4[...]
/// Plug syntax, from root, is  :Node1:Node2:Node3:Node4.Data
// ************************************************************************* //
DagNode	*CmdSave::execute(const String &params)
{
  String	errorMsg;

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }

  std::ofstream	ofs(readAsString("file").data());
  
  ofs << GBBVERSION << std::endl;
  DagNode::root().save(ofs);
  MainWindow::mainwindow()->setWindowTitle(QString(": ") + 
				       readAsString("file").data());
  return 0;
}

// ************************************************************************* //
CmdExport::CmdExport()
{
  addString("file");
}

// ************************************************************************* //
/// DESCRIPTION: Selects the object.
/// Arguments:
///  "-none" or node path or path + data.
///  -level 0, 1, 2 level of selection.
///  "-0"    level 0, at Node
///  "-1"    level 1, at Plug (if selectable)
/// Path syntax, from root, is  :Node1:Node2:Node3:Node4[...]
/// Plug syntax, from root, is  :Node1:Node2:Node3:Node4.Data
// ************************************************************************* //
DagNode	*CmdExport::execute(const String &params)
{
  String	errorMsg;

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }

  std::ofstream	ofs(readAsString("file").data());
  
  DagNode	*dn = DagNode::getNodeByPath(":SmokeParticles0");

  if (!dn)
    std::cerr << "Error: cannot find node SmokeParticles0." << std::endl;
  else
    {
      std::list<SmokeParticle> *sp = 
	static_cast<std::list<SmokeParticle> *>(dn->readAsCustom("sp"));
      std::list<SmokeParticle>::iterator spit;

      if (dn == 0)
	std::cerr << "Error: there are no particles is this node."
		  << std::endl;
      else
	{
	  std::cout << "Exporting particles in file "
		    << readAsString("file") << " ..." << std::flush;
	  for (spit = sp->begin(); spit != sp->end(); ++spit)
	    {
	      Vec3d	axes[3], evec[3];
	      double	eval[3] = { 0.0, 0.0, 0.0 };
	      Mat4d	m, r;

	      if (spit->cov.eigenDecomposition(eval, evec) == 3)
		{
		  for (int i(0); i < 3; ++i)
		    axes[i] = evec[i].unit() * sqrt(3.0 * fabs(eval[i]));
		  if (0.0 <= (evec[0] ^ evec[1]) * evec[2])
		    m = Mat4d(axes[0], axes[1], axes[2], Pnt3d(0.0)),
		      r = Mat4d(axes[0].unit(), axes[1].unit(), axes[2].unit(),
				Pnt3d(0.0));
		  else
		    m = Mat4d(axes[1], axes[0], axes[2], Pnt3d(0.0)),
		      r = Mat4d(axes[1].unit(), axes[0].unit(), axes[2].unit(),
				Pnt3d(0.0));
		}

	      //ofs << "sphere -ch 0;" << std::endl;
	      ofs << "makeblob;" << std::endl;
	      ofs << "scale -r "
		  << axes[0].norm() << " " 
		  << axes[1].norm() << " " 
		  << axes[2].norm() << ";" << std::endl;

	      const double	rz = atan2(-r(1, 0), r(0, 0));
	      const double	ry = atan2(r(2, 0) , r(0, 0) / cos(rz));
	      const double	rx = atan2(-r(2, 1), r(2, 2));
	    
	      ofs << "rotate -r "
		  << RAD2DEG(rx) << " "
		  << RAD2DEG(ry) << " "
		  << RAD2DEG(rz) << " "
		  << ";" << std::endl;

	      ofs << "move -r " << spit->p << ";" << std::endl;

	      ofs << "setAttr .envelope " << spit->density() / 5.0 << ";"
		  << std::endl;
      //ofs << "addAttr -ln density;" << std::endl;
      //ofs << "setAttr .density " << spit->density() / 5.0 << std::endl;

	      ofs << std::endl;
	    }
	  std::cout << "done." << std::endl;
	}
    }
  return 0;
}

// ************************************************************************* //
CmdNewScene::CmdNewScene()
{
}

// ************************************************************************* //
// DESCRIPTION: Resets the scene.
// ************************************************************************* //
DagNode	*CmdNewScene::execute(const String &)//params
{
  DagNode::root().clear();
  DagNode::root().initialize();
  DagNode::root().createInstName();

  return 0;
}

// ************************************************************************* //
CmdLoad::CmdLoad()
{
  addString("file");
}

// ************************************************************************* //
/// DESCRIPTION: Selects the object.
/// Arguments:
///  "-none" or node path or path + data.
///  -level 0, 1, 2 level of selection.
///  "-0"    level 0, at Node
///  "-1"    level 1, at Plug (if selectable)
/// Path syntax, from root, is  :Node1:Node2:Node3:Node4[...]
/// Plug syntax, from root, is  :Node1:Node2:Node3:Node4.Data
/// v2real	- map vadress to real adress.
/// ref2v	- Vadresses refs.
// ************************************************************************* //
DagNode	*CmdLoad::execute(const String &params)
{
  String	errorMsg;

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }
  std::ifstream	ifs(readAsString("file").data());
  String	rootType;
  unsigned	adress;
  std::map<int, void *>	v2real;
  std::map<int, std::list<void **> >	ref2v;
  std::map<int, std::list<void **> >::iterator	vit;
  std::list<void **>::iterator	rit;
  std::list<Plug*>	notSaved;
  std::list<Plug*>::iterator	nsit;
  String	version;

  ifs >> version;
  if (version != GBBVERSION)
    {
      std::cerr << "Error: cannot read version " << version
		<< " format." << std::endl;
      return 0;
    }

  ifs >> rootType;
  ifs >> adress;
  v2real[adress] = &DagNode::root();
  DagNode::root().load(ifs, v2real, ref2v, notSaved);
  MainWindow::mainwindow()->setWindowTitle(QString(": ") +
					   readAsString("file").data());
  for (vit = ref2v.begin(); vit != ref2v.end(); ++vit)
    for (rit = vit->second.begin(); rit != vit->second.end(); ++rit)
      {
	**rit = v2real[vit->first];
	if (v2real[vit->first] == 0)
	  std::cout << "WARNING! Assigning a null pointer to vAdress "
		    << vit->first << std::endl;
      }
  for (nsit = notSaved.begin(); nsit != notSaved.end(); ++nsit)
    (*nsit)->propagateDirtyBit();
  // TODO: update clock editor
  MainWindow::mainwindow()->clockEditor()->update();
  
  return 0;
}

// ************************************************************************* //
CmdParent::CmdParent()
{
  addString("child", 1);
  addString("parent", 2);
}

// ************************************************************************* //
/// DESCRIPTION:
// ************************************************************************* //
DagNode	*CmdParent::execute(const String &params)
{
  String	errorMsg;
  DagNode	*parent(0), *child(0);

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }
  parent = DagNode::getNodeByPath(readAsString("parent"));
  child = DagNode::getNodeByPath(readAsString("child"));
  if (!parent)
    {
      std::cerr << "Error: " << readAsString("parent") 
		<< " is not a valid path." << std::endl;
      return 0;
    }
  if (!child)
    {
      std::cerr << "Error: " << readAsString("child") 
		<< " is not a valid path." << std::endl;
      return 0;
    }
  if (parent->getPlug("matChild") && child->getPlug("matParent"))
    {
      //const Mat4d	&mp =  parent->readAsMat4d("matChild");
      //const Mat4d	&mc =  child->readAsMat4d("mat");
      
      DagNode::connect(*parent->getPlug("matChild"),
		       *child->getPlug("matParent"));
      //child->writeAsMat4d("mat", mp.inverse() * mc);
    }
  if (parent->getPlug("renderIn") && child->getPlug("renderIn"))
    {
      DagNode::connect(*parent->getPlug("renderIn"),
		       *child->getPlug("renderIn"));
    }
  DagNode::parent(*parent, *child);
  return 0;
}

// ************************************************************************* //
CmdDeformer::CmdDeformer()
{
  addString("shape");
}

// ************************************************************************* //
/// DESCRIPTION: 
///   If node doesn't have a deformer node, creates one.
///   Creates a swirling sweeper
// ************************************************************************* //
DagNode	*CmdDeformer::execute(const String &params)
{
  std::list<DagNode *>	nodes;
  std::list<DagNode *>::iterator	nit;
  String	errorMsg;

  if (!browse(params, errorMsg))
    {
      DagNode::getNodesSelected(nodes);

      if (nodes.empty())
	{
	  std::cerr << "Error: " << errorMsg << std::endl;
	  return 0;
	}
    }
  else
    nodes.push_back(DagNode::getNodeByPath(readAsString("shape")));

  DagNode	*swirlingsweeper = 0;

  if (!nodes.empty())
    swirlingsweeper = Command::run(String("new -type SwirlingSweeper"));
  for (nit = nodes.begin(); nit != nodes.end(); ++nit)
    {
      DagNode	*parent((*nit)->getNodeParent());
      DagNode	*def(0);

      if (parent->getTypeStr() == "Deformer")
	def = parent;
      else
	{
	  def = Command::run(String("new -type Deformer"));
	  DagNode::parent(*parent, *def);
	  DagNode::parent(*def, **nit);
	  DagNode::connect(*parent, "renderIn", *def, "renderIn");
	  DagNode::connect(*parent, "matChild", *def, "matParent");
	}
      DagNode::connect(*def, "renderIn", **nit, "renderIn");
      //DagNode::connect(*def, "matChild", **nit, "matParent");
      DagNode::connect(*def, "shape", **nit, "deformer");
      DagNode::connect(*swirlingsweeper, "deformer", *def, "ss");
    }
  return 0;
}

// ************************************************************************* //
CmdControlGenerator::CmdControlGenerator()
{
  addString("shape", 1);
}

// ************************************************************************* //
/// DESCRIPTION: 
///   If node doesn't have a deformer node, creates one.
///   Creates a swirling sweeper
// ************************************************************************* //
DagNode	*CmdControlGenerator::execute(const String &params)
{
  std::list<DagNode *>	nodes;
  String	errorMsg;
  DagNode	*currentgenerator(0);

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }
  currentgenerator = DagNode::getNodeByPath(readAsString("shape"));
  if (currentgenerator->getTypeStr() != "CurrentGenerator")
    {
      std::cerr << "Error: please specify a CurrentGenerator." << std::endl;
      return 0;
    }

  DagNode	*control = Command::run(String("new -type Control"));
  DagNode	*clock = DagNode::getNodeByPath(":Clock0");

  DagNode::connect(*control, "dst", *currentgenerator, "dst");
  DagNode::connect(*currentgenerator, "src", *control, "src");
  DagNode::connect(*clock, "f", *control, "f");
  DagNode::connect(*clock, "dt", *control, "dt");
  return 0;
}

// ************************************************************************* //
CmdBakeObstacle::CmdBakeObstacle()
{
  addString("obs", 1);
}

// ************************************************************************* //
/// DESCRIPTION: Saves the obstacle matrix in a file.
// ************************************************************************* //
DagNode	*CmdBakeObstacle::execute(const String &params)
{
  String	errorMsg;
  DagNode	*obstacle(0);

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }

  obstacle = DagNode::getNodeByPath(readAsString("obs"));
  if (obstacle->getTypeStr() != "Obstacle")
    {
      std::cerr << "Error: please specify an Obstacle." << std::endl;
      return 0;
    }

  QString s = QFileDialog::getSaveFileName(MainWindow::mainwindow(),
					   "save file dialog", ".",
					   "matrix (*.mat)");

  if (s.isEmpty())
    {
      std::cerr << "Error: no file specified." << std::endl;
      return 0;
    }
  if (!s.contains(".mat"))
    s = s + ".mat";

  std::ofstream	ofs(s.toAscii().data());

  if (!ofs)
    {
      std::cerr << "Error: please specify a file." << std::endl;
      return 0;
    }
  //TODO: progress bar!
  static_cast<MatMNd *>(obstacle->readAsCustom("v2vComputed"))->saveBits(ofs);
  obstacle->writeAsString("matrixfile", s.toAscii().data());
  return 0;
}

// ************************************************************************* //
CmdBindVortexGenerator::CmdBindVortexGenerator()
{
  addString("obs1", 1);
  addString("obs2", 2);
}

// ************************************************************************* //
/// DESCRIPTION: Binds a generator to the control.
// ************************************************************************* //
DagNode	*CmdBindVortexGenerator::execute(const String &params)
{
  String	errorMsg;
  DagNode	*vortexgenerator(0);
  DagNode	*currentcontrol(0);
  DagNode	*b3d(0);
  DagNode	*tr(0);

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }
  vortexgenerator = DagNode::getNodeByPath(readAsString("obs1"));
  b3d = DagNode::getNodeByPath(readAsString("obs2"));
  if (vortexgenerator->getTypeStr() != "CurrentGenerator" || 
      b3d->getTypeStr() != "Bezier3d" && 
      b3d->getNodeParent()->getTypeStr() != "CurrentControl")
    {
      std::cerr << "Error: wrong type parameter." << std::endl;
      return 0;
    }
  tr = vortexgenerator->getNodeParent();
  currentcontrol = b3d->getNodeParent();
  Command::run(String("parent -child ") +  tr->path() +
	       String(" -parent ") + currentcontrol->path());
  return 0;
}

// ************************************************************************* //
CmdPassItOn::CmdPassItOn()
{
  addString("checkpoint", 1);
  addString("controlCurve", 2);
}

// ************************************************************************* //
/// DESCRIPTION: Binds a generator to the control.
// ************************************************************************* //
DagNode	*CmdPassItOn::execute(const String &params)
{
  String	errorMsg;

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }

  DagNode *checkpoint = DagNode::getNodeByPath(readAsString("checkpoint"));
  DagNode *controlCurve = DagNode::getNodeByPath(readAsString("controlCurve"));

  if (checkpoint->getTypeStr() != "RingCheckpoint" || 
      controlCurve->getTypeStr() != "Bezier3d" && 
      controlCurve->getNodeParent()->getTypeStr() != "CurrentControl")
    {
      std::cerr << "Error: wrong type parameter." << std::endl;
      return 0;
    }
  DagNode::connect(*controlCurve->getNodeParent(), "signal",
		   *checkpoint, "passItOn");
  DagNode::connect(*checkpoint, "center", *controlCurve, "front");
  return 0;
}

// ************************************************************************* //
CmdConstraintCurves::CmdConstraintCurves()
{
  addString("curve1", 1);
  addString("curve2", 2);

  addSignal("back0");
  addSignal("back1");
}

// ************************************************************************* //
/// DESCRIPTION: Saves the obstacle matrix in a file.
// ************************************************************************* //
DagNode	*CmdConstraintCurves::execute(const String &params)
{
  String	errorMsg;

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }

  DagNode	*c0 = DagNode::getNodeByPath(readAsString("curve1"));
  DagNode	*c1 = DagNode::getNodeByPath(readAsString("curve2"));
  DagNode	*ctrn = Command::run("new -type CurveConstraint");

  if (readAsSignal("back0"))
    ctrn->writeAsBool("front0", false);
  if (readAsSignal("back1"))
    ctrn->writeAsBool("front1", false);
  
  DagNode::connect(*c0, "signal", *ctrn, "curve0");
  DagNode::connect(*c1, "signal", *ctrn, "curve1");
  // 
  return 0;
}

// ************************************************************************* //
CmdNewCoonsPatch::CmdNewCoonsPatch()
{
  addString("curve0", 1);
  addString("curve1", 2);
  addString("curve2", 3);
  addString("curve3", 4);
}

// ************************************************************************* //
/// DESCRIPTION: Saves the obstacle matrix in a file.
// ************************************************************************* //
DagNode	*CmdNewCoonsPatch::execute(const String &params)
{
  String	errorMsg;

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }

  DagNode	*ctrn = Command::run("new -type CoonsPatch");

  DagNode	*c0 = DagNode::getNodeByPath(readAsString("curve0"));
  DagNode	*c1 = DagNode::getNodeByPath(readAsString("curve1"));
  DagNode	*c2 = DagNode::getNodeByPath(readAsString("curve2"));
  DagNode	*c3 = DagNode::getNodeByPath(readAsString("curve3"));

  DagNode::connect(*c0, "signal", *ctrn, "curve0");
  DagNode::connect(*c1, "signal", *ctrn, "curve1");
  DagNode::connect(*c2, "signal", *ctrn, "curve2");
  DagNode::connect(*c3, "signal", *ctrn, "curve3");

  return 0;
}

// ************************************************************************* //
CmdNewCamera::CmdNewCamera()
{
}

// ************************************************************************* //
/// DESCRIPTION: Creates a camera, a locator, and a constain.
// ************************************************************************* //
DagNode	*CmdNewCamera::execute(const String &params)
{
  String	errorMsg;

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }
  
  // Creates a camera, a locator, and a constain.
  DagNode	*camera = Command::run("new -type Camera");
  DagNode	*locator = Command::run("new -type Locator");
  DagNode	*aim = Command::run("new -type ConstraintAim");
  
  DagNode	*cameraTr(camera->getNodeParentByType("Transform"));
  //DagNode	*locatorTr(locator->getNodeParentByType("Transform"));

  // Plug everything
  cameraTr->writeAsDouble("ty", 5.0);
  //locatorTr->writeAsDouble("ty", -5.0);

  DagNode::connect(*aim, "signal", *camera, "constraintAim");
  DagNode::connect(*cameraTr, "t", *aim, "pos");// camera eye!
  DagNode::connect(*locator, "pos", *aim, "target");
  DagNode::connect(*aim, "rx", *cameraTr, "rx");
  DagNode::connect(*aim, "ry", *cameraTr, "ry");
  DagNode::connect(*aim, "rz", *cameraTr, "rz");
  return 0;
}

// ************************************************************************* //
CmdAttachControl::CmdAttachControl()
{
  addString("control1", 1);
  addString("control2", 2);
}

// ************************************************************************* //
DagNode	*CmdAttachControl::execute(const String &params)
{
  String	errorMsg;

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }

  DagNode	*c1 = DagNode::getNodeByPath(readAsString("control1"));
  DagNode	*c2 = DagNode::getNodeByPath(readAsString("control2"));
  DagNode	*cc1 = c1->getNodeParent();
  DagNode	*cc2 = c2->getNodeParent();

  if (cc1 && cc1->getTypeStr() != "CurrentControl" && 
      cc2 && cc2->getTypeStr() != "CurrentControl")
    {
      std::cerr << "Error: wrong type parameter." << std::endl;
      return 0;
    }

  DagNode::connect(*cc1, "next", *cc2, "prev");
  DagNode::connect(*c1, "back", *c2, "front");
  DagNode::connect(*c1, "extrapolate", *c2, "following");

  return 0;
}

// ************************************************************************* //
CmdNewMesh::CmdNewMesh()
{
  addString("file", 1);
}

// ************************************************************************* //
DagNode	*CmdNewMesh::execute(const String &params)
{
  String	errorMsg;

  if (!browse(params, errorMsg))
    {
      std::cerr << "Error: " << errorMsg << std::endl;
      return 0;
    }

  DagNode *mesh = Command::run("new -type Mesh");

  if (mesh)
    {
      String	str = readAsString("file");

      mesh->writeAsString("meshFile", readAsString("file"));
      return mesh;
    }
  return 0;
}
