// ************************************************************************* //
/// GBB - Graphics Blue Box, by Alexis Angelidis
/// $Author: silex $
/// $Date: 2006/01/11 21:56:28 $
/// $Revision: 1.1 $
// ************************************************************************* //
#ifndef __COMMANDCUSTOM_HH__
#define __COMMANDCUSTOM_HH__

// ************************************************************************* //
#include "command.h"

// ************************************************************************* //
class CmdDeselect;
class Plug;

// ************************************************************************* //
/// DESCRIPTION: Enables/disables bit "sel" of object given in parameter (-0), 
///  or Enables/disables bit select of data (-1)
///  or deselects all the shapes.
// ************************************************************************* //
class CmdSelect: public Command
{
  friend class CmdDeselect;
private:
  static int	selectIndex;
public:
  static int	id;
  static String	name;
public:
  CmdSelect();
  virtual ~CmdSelect() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdSelect; }
  void	connectGizmoTo(DagNode &dn);
  void	disconnectGizmoFrom(DagNode &dn);
  void	disconnectGizmoFromAll();
};

// ************************************************************************* //
/// DESCRIPTION: Enables/disables bit "sel" of object given in parameter (-0), 
///  or Enables/disables bit select of data (-1)
///  or deselects all the shapes.
// ************************************************************************* //
class CmdDeselect: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdDeselect();
  virtual ~CmdDeselect() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdDeselect; }
};

// ************************************************************************* //
/// DESCRIPTION: Enables/disables bit "sel" of object given in parameter (-0), 
///  or Enables/disables bit select of data (-1)
///  or deselects all the shapes.
// ************************************************************************* //
class CmdSave: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdSave();
  virtual ~CmdSave() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdSave; }
};

// ************************************************************************* //
/// DESCRIPTION: Enables/disables bit "sel" of object given in parameter (-0), 
///  or Enables/disables bit select of data (-1)
///  or deselects all the shapes.
// ************************************************************************* //
class CmdExport: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdExport();
  virtual ~CmdExport() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdExport; }
};

// ************************************************************************* //
/// DESCRIPTION: Enables/disables bit "sel" of object given in parameter (-0), 
///  or Enables/disables bit select of data (-1)
///  or deselects all the shapes.
// ************************************************************************* //
class CmdLoad: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdLoad();
  virtual ~CmdLoad() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdLoad; }
};

// ************************************************************************* //
/// DESCRIPTION: Enables/disables bit "sel" of object given in parameter (-0), 
///  or Enables/disables bit select of data (-1)
///  or deselects all the shapes.
// ************************************************************************* //
class CmdNewScene: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdNewScene();
  virtual ~CmdNewScene() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdNewScene; }
};

// ************************************************************************* //
/// DESCRIPTION: Applies translation to every selected thing.
///  does two modes: -0 -1.
// ************************************************************************* //
class CmdTranslate: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdTranslate();
  virtual ~CmdTranslate() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdTranslate; }
};

// ************************************************************************* //
/// DESCRIPTION: Creates a new object of given type.
/// RETURN VALUE: a new instance of that type.
// ************************************************************************* //
class CmdNew: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdNew();
  virtual ~CmdNew() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdNew; }
};

// ************************************************************************* //
/// DESCRIPTION: Deletes the given instance.
/// RETURN VALUE: 0
// ************************************************************************* //
class CmdDelete: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdDelete();
  virtual ~CmdDelete() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdDelete; }
};

// ************************************************************************* //
/// DESCRIPTION: Connects two data attributes.
// ************************************************************************* //
class CmdConnect: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdConnect();
  virtual ~CmdConnect() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdConnect; }
};

// ************************************************************************* //
/// DESCRIPTION: Sets a data attribute
// ************************************************************************* //
class CmdSet: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdSet();
  virtual ~CmdSet() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdSet; }
};

// ************************************************************************* //
/// DESCRIPTION: Deletes the given instance.
/// RETURN VALUE: 0
// ************************************************************************* //
class CmdSetKey: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdSetKey();
  virtual ~CmdSetKey() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdSetKey; }
private:
  void	setKeyToPlug(Plug &d) const;
};

// ************************************************************************* //
/// DESCRIPTION: Deletes the given instance.
/// RETURN VALUE: 0
// ************************************************************************* //
class CmdRemKey: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdRemKey();
  virtual ~CmdRemKey() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdRemKey; }
private:
  void	remKeyToPlug(Plug &d) const;
};

// ************************************************************************* //
/// DESCRIPTION: Deletes the given instance.
/// RETURN VALUE: 0
// ************************************************************************* //
class CmdEdit: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdEdit();
  virtual ~CmdEdit() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdEdit; }
};

// ************************************************************************* //
/// DESCRIPTION: Deletes the given instance.
/// RETURN VALUE: 0
// ************************************************************************* //
class CmdEditCurve: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdEditCurve();
  virtual ~CmdEditCurve() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdEditCurve; }
};

// ************************************************************************* //
/// DESCRIPTION: Deletes the given instance.
/// RETURN VALUE: 0
// ************************************************************************* //
class CmdParent: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdParent();
  virtual ~CmdParent() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdParent; }
};

// ************************************************************************* //
/// DESCRIPTION: Deletes the given instance.
/// RETURN VALUE: 0
// ************************************************************************* //
class CmdDeformer: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdDeformer();
  virtual ~CmdDeformer() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdDeformer; }
};

// ************************************************************************* //
/// DESCRIPTION: Deletes the given instance.
/// RETURN VALUE: 0
// ************************************************************************* //
class CmdControlGenerator: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdControlGenerator();
  virtual ~CmdControlGenerator() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdControlGenerator; }
};

// ************************************************************************* //
/// DESCRIPTION: Deletes the given instance.
/// RETURN VALUE: 0
// ************************************************************************* //
class CmdBakeObstacle: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdBakeObstacle();
  virtual ~CmdBakeObstacle() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdBakeObstacle; }
};

// ************************************************************************* //
/// DESCRIPTION: Deletes the given instance.
/// RETURN VALUE: 0
// ************************************************************************* //
class CmdBindVortexGenerator: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdBindVortexGenerator();
  virtual ~CmdBindVortexGenerator() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdBindVortexGenerator; }
};

// ************************************************************************* //
/// DESCRIPTION: Deletes the given instance.
/// RETURN VALUE: 0
// ************************************************************************* //
class CmdPassItOn: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdPassItOn();
  virtual ~CmdPassItOn() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdPassItOn; }
};

// ************************************************************************* //
/// DESCRIPTION: Deletes the given instance.
/// RETURN VALUE: 0
// ************************************************************************* //
class CmdConstraintCurves: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdConstraintCurves();
  virtual ~CmdConstraintCurves() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdConstraintCurves; }
};

// ************************************************************************* //
/// DESCRIPTION: Deletes the given instance.
/// RETURN VALUE: 0
// ************************************************************************* //
class CmdNewCoonsPatch: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdNewCoonsPatch();
  virtual ~CmdNewCoonsPatch() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdNewCoonsPatch; }
};

// ************************************************************************* //
/// DESCRIPTION: Deletes the given instance.
/// RETURN VALUE: 0
// ************************************************************************* //
class CmdNewCamera: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdNewCamera();
  virtual ~CmdNewCamera() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdNewCamera; }
};

// ************************************************************************* //
/// DESCRIPTION: Deletes the given instance.
/// RETURN VALUE: 0
// ************************************************************************* //
class CmdAttachControl: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdAttachControl();
  virtual ~CmdAttachControl() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdAttachControl; }
};

// ************************************************************************* //
/// DESCRIPTION: Deletes the given instance.
/// RETURN VALUE: 0
// ************************************************************************* //
class CmdNewMesh: public Command
{
public:
  static int	id;
  static String	name;
public:
  CmdNewMesh();
  virtual ~CmdNewMesh() {}
  virtual int		getId() const { return id; }
  virtual const String	&getName() const { return name; }
  virtual DagNode	*execute(const String &params);
  static Command	*creator() { return new CmdNewMesh; }
};

// ************************************************************************* //
#endif//__COMMANDCUSTOM_HH__
