// ************************************************************************* //
/// GBB - Graphics Blue Box, by Alexis Angelidis
/// $Author: silex $
/// $Date: 2006/01/11 21:56:29 $
/// $Revision: 1.1 $
// ************************************************************************* //
#ifndef	__DATABLOCK_HH__
#define	__DATABLOCK_HH__

// ************************************************************************* //
#include <list>
#include <vector>
#include <map>
#include "mystring.h"
#include "gfxtypes.h"
#include "twistlog.h"

// ************************************************************************* //
class DataBlock;
class DagNode;
class QWidget;
class Plug;

// ************************************************************************* //
/// DESCRIPTION: List of attributes of a class.
///   Indirect access to data.
/// _m_validBits - sets them all to false when node is destroyed.
/// _m_node	 - node to whom the datablock belongs to.
// ************************************************************************* //
class DataBlock
{
  friend class Plug;
protected:
  std::map<String, Plug *>	_m_quickMap;
  std::list<Plug *>		_m_plugs;
  std::list<QWidget *>		_m_validBits;
  DagNode	&_m_node;
  int		_m_idCounter;
public:
  typedef std::list<Plug *>::iterator	iterator;
public:
  DataBlock(DagNode &node);
  virtual ~DataBlock();

  int		size() const { return _m_quickMap.size(); }
  void		affect(const char *input, const char *output);
  void		affect(Plug &dh1, Plug &dh2);
  static void	connect(Plug &dh1, Plug &dh2);
  static void	disconnect(Plug &dh1, Plug &dh2);
  static void	disconnect(Plug &dh);
  static void	disconnectAll(DataBlock &db);
  static void	removeAll(DataBlock &db);
  DagNode	&node() { return _m_node; }
  Plug		*getPlug(const String &name);
  const Plug	*getPlug(const String &name) const;
  Plug		*getPlug(const int id);
  void		getPlugByType(const int id, std::list<Plug *> &hnd);
  void		deselect();
  void		addValidBit(QWidget *b);
  void		remValidBit(QWidget *b);
  
  // Add methods *********************************************************** //
  void	addCustom(Plug &d, const char *shortName, const bool isVisible = true,
		  const bool isSaved = true);
  void	addSignal(const char *shortName, const bool isVisible = true,
		  const bool isSaved = true);
  void	addBool(const bool value, const char *shortName,
		const bool isVisible = true, const bool isSaved = true);
  void	addInt(const int value, const int minValue, const int maxValue,
	       const char *shortName, const bool isVisible = true,
	       const bool isSaved = true);
  void	addDouble(const double defaultValue, const char *shortName,
		  const bool isVisible = true, const bool isSaved = true);
  void	addDouble(const double defaultValue, const double minValue,
		  const double maxValue, const char *shortName,
		  const bool isVisible = true, const bool isSaved = true);
  void	addString(const String &defaultValue, const char *shortName,
		  const bool isVisible = true, const bool isSaved = true);
  void	addPnt2d(const Pnt2d &defaultValue, const char *shortName,
		 const bool isVisible = true, const bool isSaved = true);
  void	addMat2i(const Mat2i &defaultValue, const char *shortName,
		 const bool isVisible = true, const bool isSaved = true);
  void	addPnt3d(const Pnt3d &defaultValue, const double minValue,
		 const double maxValue, const char *shortName,
		 const bool isVisible = true, const bool isSaved = true);
  void	addPnt3d(const Pnt3d &defaultValue, const char *shortName,
		 const bool isVisible = true, const bool isSaved = true);
  void	addVec3d(const Vec3d &defaultValue, const double minValue,
		 const double maxValue, const char *shortName,
		 const bool isVisible = true, const bool isSaved = true);
  void	addVec3d(const Vec3d &defaultValue, const char *shortName,
		 const bool isVisible = true, const bool isSaved = true);
  void	addRgb3d(const Rgb3d &defaultValue, const double minValue,
		 const double maxValue, const char *shortName,
		 const bool isVisible = true, const bool isSaved = true);
  void	addRgb3d(const Rgb3d &defaultValue, const char *shortName,
		 const bool isVisible = true, const bool isSaved = true);
  void	addMat3d(const Mat3d &defaultValue, const char *shortName,
		 const bool isVisible = true, const bool isSaved = true);
  void	addTwistLog(const TwistLog &defaultValue, const char *shortName,
		    const bool isVisible = true, const bool isSaved = true);
  void	addMat4d(const Mat4d &defaultValue, const char *shortName,
		 const bool isVisible = true, const bool isSaved = true);
  void	addIntArray(const int n, const int defaultValue,
		    const char *shortName, const bool isVisible = true,
		    const bool isSaved = true);
  void	addDoubleArray(const int n, const double defaultValue,
		       const char *shortName, const bool isVisible = true,
		       const bool isSaved = true);
  void	addPnt2dArray(const int n, const Pnt2d &defaultValue,
		      const char *shortName, const bool isVisible = true,
		      const bool isSaved = true);
  void	addPnt3dArray(const int n, const Pnt3d &defaultValue,
		      const char *shortName, const bool isVisible = true,
		      const bool isSaved = true);
  void	addVec3dArray(const int n, const Vec3d &defaultValue,
		      const char *shortName, const bool isVisible = true,
		      const bool isSaved = true);
  void	addTwistLogArray(const int n, const TwistLog &defaultValue,
			 const char *shortName, const bool isVisible = true,
			 const bool isSaved = true);
  void	addPnt2dList(const char *shortName, const bool isVisible = true,
		     const bool isSaved = true);

  // Gets ****************************************************************** //
  double			getAsDouble(const char *name);

  // Reads ***************************************************************** //
  void				*readAsCustom(const char *name);
  void				readAsSignal(const char *name);
  bool				readAsBool(const char *name);
  int				readAsInt(const char *name);
  double			readAsDouble(const char *name);
  const String			&readAsString(const char *name);
  const Pnt2d			&readAsPnt2d(const char *name);
  const Mat2i			&readAsMat2i(const char *name);
  const Pnt3d			&readAsPnt3d(const char *name);
  const Vec3d			&readAsVec3d(const char *name);
  const Rgb3d			&readAsRgb3d(const char *name);
  const Mat3d			&readAsMat3d(const char *name);
  const TwistLog		&readAsTwistLog(const char *name);
  const Mat4d			&readAsMat4d(const char *name);
  const std::vector<int>	&readAsIntArray(const char *name);
  const std::vector<double>	&readAsDoubleArray(const char *name);
  const std::vector<Pnt2d>	&readAsPnt2dArray(const char *name);
  const Pnt2d			&readAsPnt2dArray(const char*name,const int i);
  const std::vector<Pnt3d>	&readAsPnt3dArray(const char *name);
  const Pnt3d			&readAsPnt3dArray(const char*name,const int i);
  const std::vector<Vec3d>	&readAsVec3dArray(const char *name);
  const Vec3d			&readAsVec3dArray(const char*name,const int i);
  const std::vector<TwistLog>	&readAsTwistLogArray(const char *name);
  const TwistLog		&readAsTwistLogArray(const char *name,
						     const int i);
  const std::list<Pnt2d>	&readAsPnt2dList(const char *name);

  // Writes **************************************************************** //
  void	writeAsCustom(const char *name, const void *value);
  void	writeAsSignal(const char *name);
  void	writeAsBool(const char *name, const bool value);
  void	writeAsInt(const char *name, const int value);
  void	writeAsDouble(const char *name, const double value);
  void	writeAsString(const char *name, const String &value);
  void	writeAsPnt2d(const char *name, const Pnt2d &value);
  void	writeAsMat2i(const char *name, const Mat2i &value);
  void	writeAsPnt3d(const char *name, const Pnt3d &value);
  void	writeAsVec3d(const char *name, const Vec3d &value);
  void	writeAsRgb3d(const char *name, const Rgb3d &value);
  void	writeAsMat3d(const char *name, const Mat3d &value);
  void	writeAsTwistLog(const char *name, const TwistLog &value);
  void	writeAsMat4d(const char *name, const Mat4d &value);
  void	writeAsIntArray(const char *name, const std::vector<int> &value);
  void	writeAsDoubleArray(const char *name, const std::vector<double> &value);
  void	writeAsPnt2dArray(const char *name, const std::vector<Pnt2d> &value);
  void	writeAsPnt2dArray(const char *name, const int i, const Pnt2d &);
  void	writeAsPnt3dArray(const char *name, const std::vector<Pnt3d> &value);
  void	writeAsPnt3dArray(const char *name, const int i, const Pnt3d &);
  void	writeAsVec3dArray(const char *name, const std::vector<Vec3d> &value);
  void	writeAsVec3dArray(const char *name, const int i, const Vec3d &);
  void	writeAsTwistLogArray(const char *name,
			     const std::vector<TwistLog> &value);
  void	writeAsTwistLogArray(const char *name, const int i, const TwistLog &);
  void	writeAsPnt2dList(const char *name, const std::list<Pnt2d> &value);

  // Iterate *************************************************************** //
  iterator	begin() { return _m_plugs.begin(); }
  iterator	end() { return _m_plugs.end(); }

  // Save/Load ************************************************************* //
  void	save(std::ostream &os);
  void	load(std::istream &is, std::map<int, void *> &v2real,
	     std::map<int, std::list<void **> > &ref2v,
	     std::list<Plug*> &notSaved);
  void	clear();
};

// ************************************************************************* //
#endif//__DATABLOCK_HH__
