// ************************************************************************* //
/// GBB - Graphics Blue Box, by Alexis Angelidis
/// $Author: silex $
/// $Date: 2006/01/11 21:56:29 $
/// $Revision: 1.1 $
// ************************************************************************* //
//#include <GL/glew.h>
#include "qtMainWidget.h"
#include "commandcustom.h"
#include "nodescene.h"
#include "nodesky.h"
#include "nodeimagelayer.h"
#include "nodevortexset.h"
#include "nodecontrol.h"
#include "nodevortexring.h"
#include "nodecurrentcontrol.h"
#include "nodecurrentgenerator.h"
#include "nodeclock.h"
#include "nodesmokegrid.h"
#include "nodesmokeparticles.h"
#include "nodesmokegpuparticles.h"
#include "nodesmokegenerator.h"
#include "nodesmokegpugenerator.h"
#include "nodecurve.h"
#include "nodecamera.h"
#include "nodegizmo.h"
#include "nodetransform.h"
#include "nodedeformer.h"
#include "nodenoise.h"
#include "nodeturbulence.h"
#include "noderingcheckpoint.h"
#include "nodeobstacle.h"
#include "nodebezier3d.h"
#include "nodevelocityplot.h"
#include "nodeextrudedshape.h"
#include "nodemesh.h"
#include "nodeconstraintaim.h"
#include "nodelocator.h"
#include "nodeplane.h"
#include "plugcustom.h"
#include "plugoctree.h"
#include "plugmesh.h"

// ************************************************************************* //
/// DESCRIPTION: Register the GBB commands, nodes and plugs, then launches 
///   the application.
// ************************************************************************* //
int	main(int	ac,
	     char	**av)
{
  // Commands ************************************************************** //
  Command::registerCommand(CmdDeselect::id, CmdDeselect::name,
			   CmdDeselect::creator);
  Command::registerCommand(CmdSelect::id, CmdSelect::name, CmdSelect::creator);
  Command::registerCommand(CmdNew::id, CmdNew::name, CmdNew::creator);
  Command::registerCommand(CmdDelete::id, CmdDelete::name, CmdDelete::creator);
  Command::registerCommand(CmdTranslate::id, CmdTranslate::name,
			   CmdTranslate::creator);
  Command::registerCommand(CmdSet::id, CmdSet::name, CmdSet::creator);
  Command::registerCommand(CmdConnect::id, CmdConnect::name,
			   CmdConnect::creator);
  Command::registerCommand(CmdSave::id, CmdSave::name, CmdSave::creator);
  Command::registerCommand(CmdExport::id, CmdExport::name, CmdExport::creator);
  Command::registerCommand(CmdLoad::id, CmdLoad::name, CmdLoad::creator);
  Command::registerCommand(CmdNewScene::id, CmdNewScene::name,
			   CmdNewScene::creator);
  Command::registerCommand(CmdSetKey::id, CmdSetKey::name, CmdSetKey::creator);
  Command::registerCommand(CmdRemKey::id, CmdRemKey::name, CmdRemKey::creator);
  Command::registerCommand(CmdEdit::id, CmdEdit::name, CmdEdit::creator);
  Command::registerCommand(CmdEditCurve::id, CmdEditCurve::name,
			   CmdEditCurve::creator);
  Command::registerCommand(CmdParent::id, CmdParent::name, CmdParent::creator);
  Command::registerCommand(CmdDeformer::id, CmdDeformer::name,
			   CmdDeformer::creator);
  Command::registerCommand(CmdControlGenerator::id, CmdControlGenerator::name,
			   CmdControlGenerator::creator);
  Command::registerCommand(CmdBakeObstacle::id, CmdBakeObstacle::name,
			   CmdBakeObstacle::creator);
  Command::registerCommand(CmdBindVortexGenerator::id,
			   CmdBindVortexGenerator::name,
			   CmdBindVortexGenerator::creator);
  Command::registerCommand(CmdPassItOn::id, CmdPassItOn::name,
			   CmdPassItOn::creator);
  Command::registerCommand(CmdNewCamera::id, CmdNewCamera::name,
			   CmdNewCamera::creator);
  Command::registerCommand(CmdAttachControl::id, CmdAttachControl::name,
			   CmdAttachControl::creator);
  Command::registerCommand(CmdNewMesh::id, CmdNewMesh::name,
			   CmdNewMesh::creator);

  // PATCH: testing.
  Command::registerCommand(CmdConstraintCurves::id, CmdConstraintCurves::name,
			   CmdConstraintCurves::creator);
  Command::registerCommand(CmdNewCoonsPatch::id, CmdNewCoonsPatch::name,
			   CmdNewCoonsPatch::creator);

  // Nodes ***************************************************************** //
  DagNode::registerNode(NScene::typeNbr, NScene::typeStr, NScene::creator);
  DagNode::registerNode(NSky::typeNbr, NSky::typeStr, NSky::creator);
  DagNode::registerNode(NImageLayer::typeNbr, NImageLayer::typeStr,
			NImageLayer::creator);
  DagNode::registerNode(NCamera::typeNbr, NCamera::typeStr, NCamera::creator);
  DagNode::registerNode(NGizmo::typeNbr, NGizmo::typeStr, NGizmo::creator);

  DagNode::registerNode(NVortexRing::typeNbr, NVortexRing::typeStr,
			NVortexRing::creator);
  DagNode::registerNode(NWind::typeNbr, NWind::typeStr, NWind::creator);
  DagNode::registerNode(NVortexSet::typeNbr, NVortexSet::typeStr,
			NVortexSet::creator);
  DagNode::registerNode(NCurrentGenerator::typeNbr, NCurrentGenerator::typeStr,
			NCurrentGenerator::creator);
  DagNode::registerNode(NNoiseGenerator::typeNbr, NNoiseGenerator::typeStr,
			NNoiseGenerator::creator);
  DagNode::registerNode(NClock::typeNbr, NClock::typeStr, NClock::creator);
  DagNode::registerNode(NSmokeGrid::typeNbr, NSmokeGrid::typeStr,
			NSmokeGrid::creator);
  DagNode::registerNode(NSmokeParticles::typeNbr, NSmokeParticles::typeStr,
			NSmokeParticles::creator);
  DagNode::registerNode(NSmokeGPUParticles::typeNbr,
			NSmokeGPUParticles::typeStr,
			NSmokeGPUParticles::creator);
  DagNode::registerNode(NSmokeGenerator::typeNbr, NSmokeGenerator::typeStr,
			NSmokeGenerator::creator);
  DagNode::registerNode(NSmokeGPUGenerator::typeNbr,
			NSmokeGPUGenerator::typeStr,
			NSmokeGPUGenerator::creator);
  DagNode::registerNode(NCurve::typeNbr, NCurve::typeStr, NCurve::creator);
  DagNode::registerNode(NTransform::typeNbr, NTransform::typeStr,
			NTransform::creator);
  DagNode::registerNode(NDeformer::typeNbr, NDeformer::typeStr,
			NDeformer::creator);
  DagNode::registerNode(NSwirlingSweeper::typeNbr, NSwirlingSweeper::typeStr,
			NSwirlingSweeper::creator);
  DagNode::registerNode(NControl::typeNbr, NControl::typeStr,
			NControl::creator);
  DagNode::registerNode(NObstacle::typeNbr, NObstacle::typeStr,
			NObstacle::creator);
  DagNode::registerNode(NCurrentControl::typeNbr, NCurrentControl::typeStr,
			NCurrentControl::creator);
  DagNode::registerNode(NBezier3d::typeNbr, NBezier3d::typeStr,
			NBezier3d::creator);
  DagNode::registerNode(NVelocityPlot::typeNbr, NVelocityPlot::typeStr,
			NVelocityPlot::creator);
  DagNode::registerNode(NMesh::typeNbr, NMesh::typeStr, NMesh::creator);
  DagNode::registerNode(NConstraintAim::typeNbr, NConstraintAim::typeStr,
			NConstraintAim::creator);
  DagNode::registerNode(NLocator::typeNbr, NLocator::typeStr,
			NLocator::creator);
  DagNode::registerNode(NPlane::typeNbr, NPlane::typeStr, NPlane::creator);
  DagNode::registerNode(NTurbulence::typeNbr, NTurbulence::typeStr,
			NTurbulence::creator);
  DagNode::registerNode(NRingCheckpoint::typeNbr, NRingCheckpoint::typeStr,
			NRingCheckpoint::creator);

  // PATCH: testing.
  DagNode::registerNode(NExtrudedShape::typeNbr, NExtrudedShape::typeStr,
			NExtrudedShape::creator);
  DagNode::registerNode(NCurveConstraint::typeNbr, NCurveConstraint::typeStr,
			NCurveConstraint::creator);
  DagNode::registerNode(NCoonsPatch::typeNbr, NCoonsPatch::typeStr,
			NCoonsPatch::creator);

  // Data ****************************************************************** //
  Plug::registerPlug(PlugSmokeParticles::typeStr, PlugSmokeParticles::creator);
  Plug::registerPlug(PlugSmokeGPUParticles::typeStr,
		     PlugSmokeGPUParticles::creator);
  Plug::registerPlug(PlugNoiseVortices::typeStr, PlugNoiseVortices::creator);
  Plug::registerPlug(PlugSky::typeStr, PlugSky::creator);
  Plug::registerPlug(PlugFrustum::typeStr, PlugFrustum::creator);
  Plug::registerPlug(PlugBBox3d::typeStr, PlugBBox3d::creator);
  Plug::registerPlug(PlugOctree::typeStr, PlugOctree::creator);
  Plug::registerPlug(PlugMatMNd::typeStr, PlugMatMNd::creator);
  Plug::registerPlug(PlugMesh::typeStr, PlugMesh::creator);
  Plug::registerPlug(PlugNodeList::typeStr, PlugNodeList::creator);
  Plug::registerPlug(PlugOctreeVortex::typeStr, PlugOctreeVortex::creator);
  Plug::registerPlug(PlugObstacleVortex::typeStr, PlugObstacleVortex::creator);
  Plug::registerPlug(PlugOctreeNoise::typeStr, PlugOctreeNoise::creator);
  Plug::registerPlug(PlugTurbulence::typeStr, PlugTurbulence::creator);

  QApplication	qapp(ac, av);

  QWidget	*w = new FrontPanel();

  w->show();
  w->raise();

  MainWindow	*mainWindow = new MainWindow();

  mainWindow->setGeometry(0, 0, 640, 550);
  mainWindow->move(10, 10);
  qapp.setActiveWindow(mainWindow);
  mainWindow->show();
  w->raise();
  return qapp.exec();
}
