// ************************************************************************* //
/// GBB - Graphics Blue Box, by Alexis Angelidis
/// $Author: silex $
/// $Date: 2006/01/11 21:56:29 $
/// $Revision: 1.1 $
// ************************************************************************* //
#include "nodeconstraintaim.h"
#include "datablock.h"
#include "plug.h"

// ************************************************************************* //
int	NConstraintAim::typeNbr(78);
String	NConstraintAim::typeStr("ConstraintAim");

// ************************************************************************* //
/// DESCRIPTION: Connected to a transform node, computes its rotation 
///  to aim at "aim".
///          ^
///          |
///          +-> target
///        _/
///      _/
///     /
///    + pos (camera)
// ************************************************************************* //
void	NConstraintAim::initialize()
{
  DataBlock	&db(getDataBlock());

  db.addSignal("signal", false);
  db.addVec3d(Vec3d(0.0, 0.0, 1.0), "up");
  db.addPnt3d(0.0, "target", false);
  db.addPnt3d(0.0, "pos", false);

  db.addVec3d(0.0, "x", false);// local frame
  db.addVec3d(0.0, "y", false);// local frame
  db.addVec3d(0.0, "z", false);// local frame

  db.addDouble(0.0, MINDOUBLE, MAXDOUBLE, "rx", false);
  db.addDouble(0.0, MINDOUBLE, MAXDOUBLE, "ry", false);
  db.addDouble(0.0, MINDOUBLE, MAXDOUBLE, "rz", false);

  db.addMat4d(Mat4d::identity(), "mat", false);

  db.affect("pos", "z");
  db.affect("target", "z");
  db.affect("up", "y");
  db.affect("z", "y");
  db.affect("y", "x");
  db.affect("z", "x");

  db.affect("x", "mat");
  db.affect("y", "mat");
  db.affect("z", "mat");

  db.affect("mat", "rx");
  db.affect("mat", "ry");
  db.affect("rz", "ry");
  db.affect("mat", "rz");

  //db.affect("target", "rx");
  //db.affect("target", "ry");
  //db.affect("target", "rz");
  //db.affect("pos", "rx");
  //db.affect("pos", "ry");
  //db.affect("pos", "rz");
}

// ************************************************************************* //
void	NConstraintAim::buildDagNodes()
{
  // nothing!
}

// ************************************************************************* //
void	NConstraintAim::compute(Plug &plugOut)
{
  if (plugOut.shortName() == "z")
    plugOut.writeAsVec3d((readAsPnt3d("pos") - readAsPnt3d("target")).unit());
  else if (plugOut.shortName() == "y")
    {
      const Vec3d &up = readAsVec3d("up");
      const Vec3d &z = readAsVec3d("z");

      plugOut.writeAsVec3d((up - z * (up * z)).unit());
    }
  else if (plugOut.shortName() == "x")
    plugOut.writeAsVec3d(readAsVec3d("y") ^ readAsVec3d("z"));
  else if (plugOut.shortName() == "mat")
    plugOut.writeAsMat4d(Mat4d(readAsVec3d("x"),
			       readAsVec3d("y"),
			       readAsVec3d("z"),
			       0.0));
  else if (plugOut.shortName() == "rz")
    {
      const Mat4d	&a = readAsMat4d("mat");

      plugOut.writeAsDouble(RAD2DEG(atan2(-a(1, 0), a(0, 0))));
    }
  else if (plugOut.shortName() == "ry")
    {
      const Mat4d	&a = readAsMat4d("mat");

      plugOut.writeAsDouble(RAD2DEG(atan2(a(2, 0), a(0, 0) / 
					  cos(DEG2RAD(readAsDouble("rz"))))));
    }
  else if (plugOut.shortName() == "rx")
    {
      const Mat4d	&a = readAsMat4d("mat");

      plugOut.writeAsDouble(RAD2DEG(atan2(-a(2, 1), a(2, 2))));
    }
}
