// ************************************************************************* //
/// GBB - Graphics Blue Box, by Alexis Angelidis
/// $Author: silex $
/// $Date: 2006/01/11 21:56:29 $
/// $Revision: 1.1 $
// ************************************************************************* //
#include "plug.h"
#include "datablock.h"
#include "nodecurve.h"

// ************************************************************************* //
int	NCurve::typeNbr(51);
String	NCurve::typeStr("Curve");

// ************************************************************************* //
void	NCurve::initialize()
{
  DataBlock	&db(getDataBlock());

  db.addInt(0, 0, MAXINT, "sel", false);
  db.addPnt2dList("ci");
  db.addDouble(0.0, MINDOUBLE, MAXDOUBLE, "x");
  db.addDouble(0.0, MINDOUBLE, MAXDOUBLE, "y");

  db.affect("ci", "y");
  db.affect("x", "y");
}

// ************************************************************************* //
int	solve(const double	a,
	      const double	b,
	      const double	c,
	      double		u[2],
	      const double	epsilon = 0.0)
{
  if (0.0 == a && 0.0 == b)
    return 0;
  if (0.0 == a)
    {
      u[0] = -c / b;
      return 1;
    }

  const double	delta(b * b - 4.0 * a * c);
  
  if (delta < -epsilon)
    return 0;
  if (fabs(delta) <= epsilon)
    {
      u[0] = -b / (2.0 * a);
      return 1;
    }

  const double	sqrtdelta(sqrt(fabs(delta)));
  const double	i2a(0.5 / a);

  if (0.0 < a)
    {
      u[0] = -(b + sqrtdelta) * i2a;
      u[1] = (sqrtdelta - b) * i2a;
      return 2;
    }
  u[0] = (sqrtdelta - b) * i2a;
  u[1] = -(b + sqrtdelta) * i2a;
  return 2;
}

// ************************************************************************* //
/// DESCRIPTION:
// ************************************************************************* //
void	NCurve::compute(Plug &hndOut)
{
  if (hndOut.shortName() == "y")
    {
      double	x = readAsDouble("x");
      TPlug	&ci = *static_cast<TPlug*>(getPlug("ci"));
      TPlug::iterator	cit;

      if (!ci.empty())
	{
	  if (x <= ci.front()->readAsPnt2d().x || ci.size() == 1)
	    hndOut.writeAsDouble(ci.front()->readAsPnt2d().y);
	  else if (ci.back()->readAsPnt2d().x <= x)
	    hndOut.writeAsDouble(ci.back()->readAsPnt2d().y);
	  else
	    {
	      for (cit = ci.begin(); cit != ci.end(); )
		{
		  const Pnt2d	&p1((*cit)->readAsPnt2d());
		  const Pnt2d	&p2((*(++cit))->readAsPnt2d());

		  if (p1.x <= x && x <= p2.x)
		    break;
		}
	      TPlug::iterator	cit3(cit), cit2(cit), cit1(--cit), cit0(cit);

	      if (cit0 != ci.begin())
		--cit0;
	      ++cit3;
	      if (cit3 == ci.end())
		cit3 = cit2;

	      Pnt2d		p0((*cit0)->readAsPnt2d());
	      const Pnt2d	&p1((*cit1)->readAsPnt2d());
	      const Pnt2d	&p2((*cit2)->readAsPnt2d());
	      Pnt2d		p3((*cit3)->readAsPnt2d());
	      const double	u((x - p1.x) / (p2.x - p1.x));
	      const bool	clamp1 = (p1.y - p0.y) * (p2.y - p1.y) <= 0.0;
	      const bool	clamp2 = (p2.y - p1.y) * (p3.y - p2.y) <= 0.0;

	      if (clamp1)
		p0 = p2;
	      if (clamp2)
		p3 = p1;
	      if (!clamp1 || !clamp2)// need adjustement if untouched
		{
		  const double	d1 = clamp1? 0.0: ((p2.y - p0.y) /
						   (p2.x - p0.x));
		  const double	d2 = clamp2? 0.0: ((p3.y - p1.y) /
						   (p3.x - p1.x));
		  const Vec2d	p12 = p2 - p1;
		  const double	b1 = d1 * p12.x;
		  const double	b2 = 3.0 * p12.y - (2.0 * d1 + d2) * p12.x;
		  const double	b3 = -2.0 * p12.y + (d1 + d2) * p12.x;
		  double	u[2];

		  if ((0 < solve(3.0 * b3, 2.0 * b2, b1, u)) &&
		      ((0.0 < u[0] && u[0] < 1.0) ||
		       (0.0 < u[1] && u[1] < 1.0)))
		    {
		      const double	d12 = p12.y / p12.x;
		      const double	tmp = 3.0 * d12 - d1 - d2;
		      const double	i1 = d1 * (2 * tmp + d2);
		      const double	j1 = d2 * (2 * tmp + d1);
		      const double	g0 = sq(tmp) - d1 * d2;
		      double	r[2], beta(0.0);
		      int	n;

		      if (clamp1)
			{
			  const double j0 = g0, j2 = sq(d2);

			  if (0 < (n = solve(j2, j1, j0, r, 1.0e-3)))
			    {
			      if (0 < r[0] && r[0] < 1.0)
				beta = r[0];
			      if (n == 2 && 0 < r[1] && r[1] < 1 &&
				  r[1] < r[0])
				beta = r[1];
			      p3.y = p3.y + (p1.y - p3.y) * beta;
			    }
			}
		      else if (clamp2)//!clamp1
			{
			  const double i0 = g0, i2 = sq(d1);

			  if (0 < (n = solve(i2, i1, i0, r, 1.0e-3)))
			    {
			      if (0 < r[0] && r[0] < 1.0)
				beta = r[0];
			      if (n == 2 && 0 < r[1] && r[1] < 1.0 &&
				  r[1] < r[0])
				beta = r[1];
			      p0.y = p0.y + (p2.y - p0.y) * beta;
			    }
			}
		      else//!clamp1 && !clamp2
			{
			  const double	g1 = i1 + j1;
			  const double	g2 = sq(d1 + d2) - d1 * d2;

			  if (0 < (n = solve(g2, g1, g0, r, 1.0e-3)))
			    {
			      if (0.0 < r[0] && r[0] < 1.0)
				beta = r[0];
			      if (n == 2 && 0.0 < r[1] && r[1] < 1.0 &&
				  r[1] < r[0])
				beta = r[1];

			      p0.y = p0.y + (p2.y - p0.y) * beta;
			      p3.y = p3.y + (p1.y - p3.y) * beta;
			    }
			}
		    }
		}
	      double d1 = (p2 == p0)? 0.0: ((p2.y - p0.y) / (p2.x - p0.x));
	      double d2 = (p3 == p1)? 0.0: ((p3.y - p1.y) / (p3.x - p1.x));
	      const Vec2d	p12(p2 - p1);
	      const double	b0 = p1.y;
	      const double	b1 = d1 * p12.x;
	      const double	b2 = 3 * p12.y - (2 * d1 + d2) * p12.x;
	      const double	b3 = -2 * p12.y + (d1 + d2) * p12.x;

	      hndOut.writeAsDouble(b0 + b1 * u + b2 * u * u + b3 * u * u * u);
	    }
	}
    }
}

// ************************************************************************* //
void	NCurve::setkey(const Pnt2d &p)
{
  TPlug	&ci = *static_cast<TPlug*>(getPlug("ci"));
  TPlug::iterator	cit;

  if (ci.empty() || p.x <= ci.front()->readAsPnt2d().x)
    {
      ci.push_front(&p);
      return;
    }
  if (ci.back()->readAsPnt2d().x <= p.x)
    {
      ci.push_back(&p);
      return;
    }
  for (cit = ci.begin(); cit != ci.end(); )
    {
      const Pnt2d	&p1((*cit)->readAsPnt2d());
      const Pnt2d	&p2((++cit)->readAsPnt2d());

      if (p1.x <= p.x && p.x <= p2.x)
	break;
    }
  ci.insert(cit, &p);
}

// ************************************************************************* //
void	NCurve::remkey(const double f)
{
  TPlug	&ci = *static_cast<TPlug*>(getPlug("ci"));
  TPlug::iterator	cit;

  for (cit = ci.begin(); cit != ci.end(); ++cit)
    if (cit->readAsPnt2d().x == f)
      {
	ci.erase(cit);
	break;
      }
}
