// ************************************************************************* //
/// GBB - Graphics Blue Box, by Alexis Angelidis
/// $Author: silex $
/// $Date: 2006/01/11 21:56:29 $
/// $Revision: 1.1 $
// ************************************************************************* //
#include <GL/glu.h>
#include "nodedeformer.h"
#include "plug.h"
#include "datablock.h"
#include "command.h"
#include "swirlingSweeper.h"

// ************************************************************************* //
int	NDeformer::typeNbr(56);
String	NDeformer::typeStr("Deformer");
int	NSwirlingSweeper::typeNbr(57);
String	NSwirlingSweeper::typeStr("SwirlingSweeper");

// ************************************************************************* //
void	NSwirlingSweeper::initialize()
{
  DataBlock	&db(getDataBlock());

  db.addInt(0, 0, MAXINT, "sel", false);
  db.addInt(0, 0, MAXINT, "renderIn", false);
  db.addSignal("renderOut", false);
  db.addMat4d(Mat4d::identity(), "matParent", false);//position
  db.addMat4d(Mat4d::identity(), "iMatParent", false);//inverse of position

  db.addPnt3d(Pnt3d(0.0), "src");
  db.addPnt3d(Pnt3d(0.0, 0.0, 2.0), "dst");
  db.addDouble(2.0, 0.0, MAXDOUBLE, "radius");
  db.addPnt3d(Pnt3d(0.0), "x", false);
  db.addPnt3d(Pnt3d(0.0), "y", false);
  db.addSignal("deformer", false);

  db.affect("matParent", "iMatParent");
  db.affect("renderIn", "renderOut");
  db.affect("sel", "renderOut");
  db.affect("matParent", "renderOut");

  db.affect("src", "renderOut");
  db.affect("dst", "renderOut");
  db.affect("radius", "renderOut");

  db.affect("x", "y");
  db.affect("src", "y");
  db.affect("dst", "y");
  db.affect("radius", "y");
  db.affect("matParent", "y");

  db.affect("src", "deformer");
  db.affect("dst", "deformer");
  db.affect("radius", "deformer");
  db.affect("matParent", "deformer");
  buildDagNodes();
}

// ************************************************************************* //
void	NSwirlingSweeper::buildDagNodes()
{
  DagNode	*oldParent(_m_parent);
  DagNode	*tr = Command::run(String("new -type Transform -path :"));

  if (oldParent)
    DagNode::parent(*oldParent, *tr);
  DagNode::parent(*tr, *this);
  DagNode::connect(*oldParent, "renderIn", *tr, "renderIn");
  DagNode::connect(*tr, "renderIn", *this, "renderIn");
  DagNode::connect(*tr, "matChild", *this, "matParent");
}

// ************************************************************************* //
void	NSwirlingSweeper::compute(Plug &hndOut)
{
  if (hndOut.shortName() == "renderOut")
    glRender(hndOut);
  else if (hndOut.shortName() == "iMatParent")
    hndOut.writeAsMat4d(readAsMat4d("matParent").inverse());
  else if (hndOut.shortName() == "y")
    {
      const Pnt3d	&src(readAsPnt3d("src"));
      const Pnt3d	&dst(readAsPnt3d("dst"));
      const double	radius(readAsDouble("radius"));

      SwirlingSweeper	ss(src, dst, radius);
      Pnt3d	x(readAsMat4d("iMatParent") * readAsPnt3d("x"));
      //TODO: cache swirlingsweeper!!!

      ss.sweep(x);
      hndOut.writeAsPnt3d(readAsMat4d("matParent") * x);
    }
  else if (hndOut.shortName() == "deformer")
    {
      readAsPnt3d("src");
      readAsPnt3d("dst");
      readAsDouble("radius");
      readAsMat4d("matParent");
      hndOut.writeAsSignal();
    }
}

// ************************************************************************* //
void	NSwirlingSweeper::glRenderDeco()
{
  const double	radius(readAsDouble("radius"));

  GLint	param;
  glGetIntegerv(GL_RENDER_MODE, &param);

  if (readAsInt("sel") && param != GL_RENDER)
    return;

  const Pnt3d	&src(readAsPnt3d("src"));
  const Pnt3d	&dst(readAsPnt3d("dst"));
  const Vec3d	z((dst - src).unit());
  Vec3d		x, y;

  if (readAsInt("sel"))
    glColor3d(1.0, 0.0, 0.0);
  else
    glColor3d(0.0, 0.0, 1.0);

  if (fabs(z.x) < fabs(z.y) && fabs(z.x) < fabs(z.z))
    y = Vec3d(1.0, 0.0, 0.0) - z * z.x;
  else if (fabs(z.y) < fabs(z.z))
    y = Vec3d(0.0, 1.0, 0.0) - z * z.y;
  else
    y = Vec3d(0.0, 0.0, 1.0) - z * z.z;
  y.eqUnit();
  x = y ^ z;

  glBegin(GL_LINE_LOOP);
  for (int i(0), n(16); i < n; ++i)
    {
      const double theta(2.0 * i * M_PI / n);

      (src + x * (cos(theta) * radius) + y * (sin(theta) * radius)).glVertex();
    }
  glEnd();//GL_LINE_LOOP

  glBegin(GL_LINES);
  readAsPnt3d("src").glVertex();
  readAsPnt3d("dst").glVertex();
  glEnd();//GL_LINES
}

// ************************************************************************* //
void	NSwirlingSweeper::glRender(Plug &hndOut)
{
  Plug	*hndSrc(getPlug("src"));
  Plug	*hndDst(getPlug("dst"));
  const Pnt3d	&src(readAsPnt3d("src"));
  const Pnt3d	&dst(readAsPnt3d("dst"));
  const double	radius(readAsDouble("radius"));

  static GLUquadric	*quad = gluNewQuadric();
  const Mat4d	&pos(readAsMat4d("matParent"));
  const int	sel(readAsInt("sel"));
  const Vec3d	z((dst - src).unit());
  Vec3d		x, y, color;

  readAsSignal("renderIn");
  hndOut.writeAsSignal();

  pushName();
  if (fabs(z.x) < fabs(z.y) && fabs(z.x) < fabs(z.z))
    y = Vec3d(1.0, 0.0, 0.0) - z * z.x;
  else if (fabs(z.y) < fabs(z.z))
    y = Vec3d(0.0, 1.0, 0.0) - z * z.y;
  else
    y = Vec3d(0.0, 0.0, 1.0) - z * z.z;
  y.eqUnit();
  x = y ^ z;
  if (sel == 1)
    color = Vec3d(1.0, 0.0, 0.5);
  else if (sel)
    color = Vec3d(1.0, 0.0, 0.0);
  else
    color = Vec3d(0.0, 0.0, 1.0);
  color.glColor();

  hndSrc->pushName();
  glPushAttrib(GL_CURRENT_BIT);
  if (hndSrc->isSelected())
    glColor3d(1.0, 1.0, 0.0);
  else
    color.glColor();
  glPushMatrix();
  Mat4d(x, y, z, src).glMultMatrix();
  glTranslated(0.0, 0.0, -0.1 * radius);
  glPushMatrix();
  glRotated(180.0, 1.0, 0.0, 0.0);
  gluDisk(quad, 0, 0.1 * radius, 16, 1);
  glPopMatrix();
  glPushMatrix();
  glTranslated(0.0, 0.0, 0.1 * radius);
  gluDisk(quad, 0, 0.1 * radius, 16, 1);
  glPopMatrix();
  gluCylinder(quad, 0.1 * radius, 0.1 * radius, 0.1 * radius, 16, 1);
  glPopMatrix();
  glPopAttrib();//GL_CURRENT_BIT
  hndSrc->popName();

  hndDst->pushName();
  glPushAttrib(GL_CURRENT_BIT);
  if (hndDst->isSelected())
    glColor3d(1.0, 1.0, 0.0);
  else
    color.glColor();
  glPushMatrix();
  Mat4d(x, y, z, dst).glMultMatrix();
  gluCylinder(quad, 0.1 * radius, 0.0, 0.25 * radius, 16, 1);
  glPushMatrix();
  glRotated(180.0, 1.0, 0.0, 0.0);
  gluDisk(quad, 0, 0.1 * radius, 16, 1);
  glPopMatrix();
  glPopMatrix();
  glPopAttrib();//GL_CURRENT_BIT
  hndDst->popName();

  glPushAttrib(GL_LIGHTING_BIT);
  glDisable(GL_LIGHTING);
  glRenderDeco();
  glPopAttrib();//GL_LIGHTING_BIT

  popName();
}

// ************************************************************************* //
/// DESCRIPTION: The deformer is added before the deformable shape.
///  Non-linear transformation.
///   Transform                              Transform
///   `-Deformer <--- SwirlingSweepers --->  `-Deformer
///     `-Shape0 `-Shape1                      `-Shape0 `-Shape1
// ************************************************************************* //
void	NDeformer::initialize()
{
  DataBlock	&db(getDataBlock());

  db.addInt(0, 0, MAXINT, "renderIn", false);
  db.addSignal("renderOut", false);
  db.addMat4d(Mat4d::identity(), "matParent", false);
  db.addMat4d(Mat4d::identity(), "matChild", false);

  db.addPnt3d(Pnt3d(0.0), "x", false);
  db.addPnt3d(Pnt3d(0.0), "y", false);
  db.addSignal("ss", false);
  db.addSignal("shape", false);

  db.affect("renderIn", "renderOut");
  db.affect("matParent", "matChild");
  db.affect("x", "y");
  db.affect("ss", "shape");
  buildDagNodes();
}

// ************************************************************************* //
void	NDeformer::buildDagNodes()
{
}

// ************************************************************************* //
void	NDeformer::glRender(Plug &hndOut)
{
  std::list<DagNode*>::iterator	nit;

  readAsSignal("renderIn");
  hndOut.writeAsSignal();

  pushName();
  glPushMatrix();
  readAsMat4d("matParent").glMultMatrix();

  for (nit = _m_children.begin(); nit != _m_children.end(); ++nit)
    if ((*nit)->getPlug("renderOut"))
      (*nit)->readAsSignal("renderOut");

  glPopMatrix();
  popName();
}

// ************************************************************************* //
void	NDeformer::compute(Plug &hndOut)
{
  if (hndOut.shortName() == "renderOut")
    glRender(hndOut);
  else if (hndOut.shortName() == "matChild")
    hndOut.writeAsMat4d(readAsMat4d("matParent"));
  else if (hndOut.shortName() == "y")
    {
      // TODO: deform.
      DagNode	*ss(getPlug("ss")->getInputNode());

      if (ss)
	{
	  ss->writeAsPnt3d("x", readAsPnt3d("x"));
	  hndOut.writeAsPnt3d(ss->readAsPnt3d("y"));
	}
      else
	hndOut.writeAsPnt3d(readAsPnt3d("x"));
    }
  else if (hndOut.shortName() == "shape")
    {
      readAsSignal("ss");
      hndOut.writeAsSignal();
    }
}
