// ************************************************************************* //
/// GBB - Graphics Blue Box, by Alexis Angelidis
/// $Author: silex $
/// $Date: 2006/01/11 21:56:29 $
/// $Revision: 1.1 $
// ************************************************************************* //
#include <fstream>
#include "nodemesh.h"
#include "datablock.h"
#include "plug.h"
#include "plugmesh.h"
#include "command.h"

// ************************************************************************* //
int	NMesh::typeNbr(70);
String	NMesh::typeStr("Mesh");

// ************************************************************************* //
void	NMesh::initialize()
{
  NShape::initialize();

  DataBlock	&db(getDataBlock());

  db.addMat4d(Mat4d::identity(), "matParent", false);//position
  db.addCustom(*new PlugMesh(db), "mesh", false, false);
  db.addString("./cigarette.obs", "meshFile");
  db.addSignal("textureChange", false);
  db.addString("./cigaretteTexMedium.jpg", "texFile");
  db.addBool(true, "isFlatShaded");
  db.addBool(true, "rotate");
  db.addBool(false, "mirroru");
  db.addBool(false, "mirrorv");
  db.addRgb3d(1.0, "color");

  db.affect("meshFile", "mesh");
  db.affect("texFile", "textureChange");
  db.affect("rotate", "textureChange");
  db.affect("mirroru", "textureChange");
  db.affect("mirrorv", "textureChange");
  db.affect("textureChange", "mesh");
  db.affect("mesh", "renderOut");
  buildDagNodes();
}

// ************************************************************************* //
void	NMesh::buildDagNodes()
{
  DagNode	*oldParent(_m_parent);
  DagNode	*tr = Command::run(String("new -type Transform -path :"));

  if (oldParent)
    DagNode::parent(*oldParent, *tr);
  DagNode::parent(*tr, *this);
  DagNode::connect(*oldParent, "renderIn", *tr, "renderIn");
  DagNode::connect(*tr, "renderIn", *this, "renderIn");
  DagNode::connect(*tr, "matChild", *this, "matParent");
}

// ************************************************************************* //
void	NMesh::compute(Plug &plugOut)
{
  if (plugOut.shortName() == "renderOut")
    {
      const int		sel(readAsInt("sel"));
      const bool	isFlatShaded(readAsBool("isFlatShaded"));
      const Rgb3d	&col = readAsRgb3d("color");
      //const Rgb3d	col(1.0, 1.0, 1.0);

      if (sel == 1)
	glColor3d(1.0 * col.r, 0.0, 0.5 * col.g);
      else if (sel)
	glColor3d(1.0 * col.r, 0.5 * col.g, 0.0);
      else
	col.glColor();

      pushName();
      glPushMatrix();
      readAsMat4d("matParent").glMultMatrix();
      
      static_cast<Mesh *>(readAsCustom("mesh"))->glRender(isFlatShaded);

      glPopMatrix();
      popName();
      readAsSignal("renderIn");
      plugOut.writeAsSignal();
    }
  else if (plugOut.shortName() == "textureChange")
    {
      Mesh *mesh = static_cast<Mesh *>(readAsCustom("mesh"));
      const String	&texFile = readAsString("texFile");
      
      QImage	qimage(texFile.data());

      mesh->setTexture(qimage);

      if (readAsBool("rotate"))
	mesh->cwRotateTexture();
      if (readAsBool("mirroru"))
	mesh->mirroru();
      if (readAsBool("mirrorv"))
	mesh->mirrorv();

      plugOut.writeAsSignal();
    }
  else if (plugOut.shortName() == "mesh")
    {
      std::ifstream	ifs(readAsString("meshFile").data());
      Mesh *mesh = static_cast<Mesh *>(readAsCustom("mesh"));

      if (ifs)
	ifs >> *mesh;
      readAsSignal("textureChange");
      plugOut.writeAsSignal();
    }
}

