// ************************************************************************* //
/// GBB - Graphics Blue Box, by Alexis Angelidis
/// $Author: silex $
/// $Date: 2006/01/11 21:56:29 $
/// $Revision: 1.1 $
// ************************************************************************* //
#ifndef __NODENOISE_HH__
#define __NODENOISE_HH__

// ************************************************************************* //
#include <vector>
#include "nodeshape.h"
#include "plug.h"
#include "dataoctree.h"

// ************************************************************************* //
#ifndef EPSILON_JAC
#define EPSILON_JAC	1.0e-8
#endif//EPSILON_JAC

// ************************************************************************* //
struct NoiseVortex
{
  Pnt3d		c;// Position
  Vec3d		n;// Axis of rotation
  double	r;// Size of influence
  int		type;// Kind. Currenlty one (two other unnecessary)
  double	t;// Age of vortex
public:
  NoiseVortex() {}
  NoiseVortex(const Pnt3d &c_, const Vec3d &n_, const double r_,
	      const int type_): c(c_), n(n_), r(r_), type(type_), t(0.0) {}
  friend std::ostream	&operator<<(std::ostream &os, const NoiseVortex &sp)
  { return os << sp.c << ' ' << sp.n << ' ' << sp.r; }
  friend std::istream	&operator>>(std::istream &is, NoiseVortex &sp)
  { return is >> sp.c >> sp.n >> sp.r; }
  void	glRender() const;
  TwistLog	tlog(const Pnt3d &p) const;
  double	presence(const double ttb, const double ttl,
			 const double ttd) const;
};

// ************************************************************************* //
class PlugOctreeNoise: public Plug
{
protected:
  Octree<std::vector<NoiseVortex*> >	*_m_data;
public:
  static int	typeNbr;
  static String	typeStr;
public:
  PlugOctreeNoise(DataBlock &db): Plug(db), _m_data(0) { newData(); }
  virtual ~PlugOctreeNoise() { freeData(); }
  static Plug	*creator(DataBlock &db) { return new PlugOctreeNoise(db); }
  virtual int	getTypeNbr() const { return typeNbr; }
  virtual const String	&getTypeStr() const { return typeStr; }
  virtual void	freeData() { if (_m_data) delete _m_data; _m_data = 0; }
  virtual void	newData()
  { if (!_m_data) _m_data = new Octree<std::vector<NoiseVortex*> >; }
  virtual void	saveData(std::ostream &) {}
  virtual void	loadData(std::istream &) {}
  virtual void	copy(const void *v)
  { *_m_data = *static_cast<const Octree<std::vector<NoiseVortex*> >*>(v); }
  virtual void	**rawdata() { return (void**)(&_m_data); }
};

// ************************************************************************* //
/// DESCRIPTION: Extends a data node. This is stored in the scene graph.
// ************************************************************************* //
class PlugNoiseVortices: public Plug
{
protected:
  std::list<NoiseVortex>	*_m_data;
public:
  static int	typeNbr;
  static String	typeStr;
public:
  PlugNoiseVortices(DataBlock &db): Plug(db), _m_data(0) { newData(); }
  virtual ~PlugNoiseVortices() { freeData(); }
  static Plug	*creator(DataBlock &db) { return new PlugNoiseVortices(db); }
  virtual int	getTypeNbr() const { return typeNbr; }
  virtual const String	&getTypeStr() const { return typeStr; }
  virtual void	freeData() { if (_m_data) delete _m_data; _m_data = 0; }
  virtual void	newData()
  { if (!_m_data) _m_data = new std::list<NoiseVortex>; }
  virtual void	saveData(std::ostream &os);
  virtual void	loadData(std::istream &is);
  virtual void	copy(const void *v)
  { *_m_data = *static_cast<const std::list<NoiseVortex>*>(v); }
  virtual void	**rawdata() { return (void**)(&_m_data); }
};

// ************************************************************************* //
class NNoiseGenerator: public NShape
{
public:
  static int	typeNbr;
  static String	typeStr;
public:
  NNoiseGenerator(DagNode *parent = 0): NShape(parent) {}
  virtual void		initialize();
  virtual int		getTypeNbr() const { return typeNbr; }
  virtual const String	&getTypeStr() const { return typeStr; }
  static DagNode	*creator() { return new NNoiseGenerator(); }
  void			buildDagNodes();
  virtual void		compute(Plug &plugOut);
  void			computeRlog(Plug &plugOut);
  void			glRender(Plug &plugOut);
  void			update(Plug &plugOut);
};

// ************************************************************************* //
#endif//__NODENOISE_HH__

