// ************************************************************************* //
/// AUTHOR: Alexis Angelidis.
/// STARTED: Sat Nov  5 13:16:11 EST 2005
// ************************************************************************* //
#include <QImage>
#include <GL/glu.h>
#include "nodeplane.h"
#include "datablock.h"
#include "plug.h"
#include "command.h"

// ************************************************************************* //
int	NPlane::typeNbr(80);
String	NPlane::typeStr("Plane");

// ************************************************************************* //
void	NPlane::initialize()
{
  DataBlock	&db(getDataBlock());

  NShape::initialize();
  db.addMat4d(Mat4d::identity(), "matParent", false);//position
  db.addMat4d(Mat4d::identity(), "imatParent", false);//position inverse
  db.addDouble(2.0, 0.0, MAXDOUBLE, "size");
  db.addString("./plane.jpg", "texFilename");
  db.addInt(0, 0, MAXINT, "texName", false, false);

  db.affect("matParent", "imatParent");
  db.affect("texFilename", "texName");
  db.affect("texName", "renderOut");

  buildDagNodes();
}

// ************************************************************************* //
void	NPlane::buildDagNodes()
{
  DagNode	*oldParent(_m_parent);
  DagNode	*tr = Command::run(String("new -type Transform -path :"));

  if (oldParent)
    DagNode::parent(*oldParent, *tr);
  DagNode::parent(*tr, *this);
  DagNode::connect(*oldParent, "renderIn", *tr, "renderIn");
  DagNode::connect(*tr, "renderIn", *this, "renderIn");
  DagNode::connect(*tr, "matChild", *this, "matParent");
}

// ************************************************************************* //
void	NPlane::compute(Plug &plugOut)
{
  if (plugOut.shortName() == "texName")
    {
      const String	&texFilename = readAsString("texFilename");
      QImage		qimage(texFilename.data());
      GLuint		texName(plugOut.readAsInt());

      if (texName)
	glDeleteTextures(1, &texName);
      if (!qimage.isNull())
	{
	  glGenTextures(1, &texName);
	  glBindTexture(GL_TEXTURE_2D, texName);
	  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
			  GL_LINEAR_MIPMAP_LINEAR);
#ifndef _WIN32
	  gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, qimage.width(),
			    qimage.height(), GL_BGRA, 
			    GL_UNSIGNED_INT_8_8_8_8_REV, qimage.bits());
#else//_WIN32
	  gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, qimage.width(),
			    qimage.height(), GL_RGBA, 
			    GL_UNSIGNED_INT, qimage.bits());
#endif//_WIN32
	}
      plugOut.writeAsInt(texName);
    }
  if (plugOut.shortName() == "renderOut")
    {
      const double	size(readAsDouble("size"));
      const int		sel(readAsInt("sel"));

      readAsSignal("renderIn");
      plugOut.writeAsSignal();
      pushName();
      glPushMatrix();
      readAsMat4d("matParent").glMultMatrix();
      glScaled(size, size, size);

      if (sel == 1)
	glColor4d(1.0, 0.0, 0.5, 1.0);
      else if (sel)
	glColor4d(1.0, 0.5, 0.0, 1.0);
      else
	glColor4d(1.0, 1.0, 1.0, 1.0);

      glPushAttrib(GL_LIGHTING_BIT);
      glDisable(GL_LIGHTING);
      glBegin(GL_LINES);
      glVertex3d(0.0, 0.0, 1.0);
      glVertex3d(0.0, 0.0, -1.0);
      glVertex3d(0.0, 1.0, 0.0);
      glVertex3d(0.0, -1.0, 0.0);
      glVertex3d(1.0, 0.0, 0.0);
      glVertex3d(-1.0, 0.0, 0.0);
      glEnd();//GL_LINES
      glPopAttrib();//GL_LIGHTING_BIT

      Frustum	fs;
      Pnt3d	hor[6];
      int	n;

      fs.getWorldFrustum();
      //
      n = fs.intersect(Plane::z(), hor);

      glEnable(GL_TEXTURE_2D);
      glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

      glBindTexture(GL_TEXTURE_2D, readAsInt("texName"));

      glNormal3d(0.0, 0.0, 1.0);
      glBegin(GL_TRIANGLE_FAN);
      for (int i(0); i < n; ++i)
	{
	  glTexCoord2d(hor[n - 1 - i].x / size, hor[n - 1 - i].y / size);
	  hor[n - 1 - i].glVertex();
	}
      glEnd();//GL_POLYGON

      glDisable(GL_TEXTURE_2D);

      glPopMatrix();

      popName();
    }
}
