// ************************************************************************* //
/// GBB - Graphics Blue Box, by Alexis Angelidis
/// $Author $
/// $Date $
/// $Revision $
// ************************************************************************* //
#ifndef __NODETURBULENCE_HH__
#define __NODETURBULENCE_HH__

// ************************************************************************* //
#include "nodeshape.h"
#include "plug.h"

// ************************************************************************* //
struct TVortex
{
  Pnt3d		p;// Position within cube
  Vec3d		w;// Strength
public:
  TVortex() {}
  TVortex(const Pnt3d &p_, const Vec3d &w_):
    p(p_), w(w_) {}
};

// ************************************************************************* //
class DataTurbulence
{
protected:
  std::list<TVortex>	_m_data;//!!!
  double		_m_strength;
  double		_m_r2;
public:
  DataTurbulence(): _m_strength(1.0) { init(64); }
  void	init(const int n);
  void	update(const double dt);
  void	glRender() const;
  TwistLog	rlogTorus(const Pnt3d &p) const;
  TwistLog	rlog(const Pnt3d &p, const int repeat) const;
  void		setStrength(const double strength) { _m_strength = strength; }
  void		setRadius(const double r) { _m_r2 = r * r; }
};

// ************************************************************************* //
/// DESCRIPTION: Wraps DataTurbulence.
// ************************************************************************* //
class PlugTurbulence: public Plug
{
protected:
  DataTurbulence	*_m_data;
public:
  static int	typeNbr;
  static String	typeStr;
public:
  PlugTurbulence(DataBlock &db): Plug(db), _m_data(0) { newData(); }
  virtual ~PlugTurbulence() { freeData(); }
  static Plug	*creator(DataBlock &db) { return new PlugTurbulence(db); }
  virtual int	getTypeNbr() const { return typeNbr; }
  virtual const String	&getTypeStr() const { return typeStr; }
  virtual void	freeData() { if (_m_data) delete _m_data; _m_data = 0; }
  virtual void	newData() { if (!_m_data) _m_data = new DataTurbulence; }
  virtual void	saveData(std::ostream &) {}
  virtual void	loadData(std::istream &) {}
  virtual void	copy(const void *v)
  { *_m_data = *static_cast<const DataTurbulence*>(v); }
  virtual void	**rawdata() { return (void**)(&_m_data); }
};

// ************************************************************************* //
class NTurbulence: public NShape
{
public:
  static int	typeNbr;
  static String	typeStr;
public:
  NTurbulence(DagNode *parent = 0): NShape(parent) {}
  virtual void		initialize();
  virtual int		getTypeNbr() const { return typeNbr; }
  virtual const String	&getTypeStr() const { return typeStr; }
  static DagNode	*creator() { return new NTurbulence(); }
  void			buildDagNodes();
  virtual void		compute(Plug &plugOut);
  void			glRender(Plug &plugOut);
  void			update(Plug &plugOut);
};

// ************************************************************************* //
#endif//__NODETURBULENCE_HH__
