// ************************************************************************* //
/// AUTHOR: Alexis Angelidis.
/// STARTED: Wed Sep 14 16:21:01 NZST 2005
// ************************************************************************* //
#include <GL/glu.h>
#include "nodevelocityplot.h"
#include "datablock.h"
#include "command.h"
#include "plug.h"

// ************************************************************************* //
int	NVelocityPlot::typeNbr(66);
String	NVelocityPlot::typeStr("VelocityPlot");

// ************************************************************************* //
void	NVelocityPlot::initialize()
{
  NShape::initialize();

  DataBlock	&db(getDataBlock());

  db.addInt(0, MININT, MAXINT, "f", false);
  db.addDouble(0, 0.0, MAXDOUBLE, "dt", false);
  db.addSignal("update", false);
  db.addMat4d(Mat4d::identity(), "matParent", false);//position
  db.addMat4d(Mat4d::identity(), "imatParent", false);//position
  db.addInt(16, 2, MAXINT, "n");
  db.addBool(true, "cached");

  db.affect("matParent", "imatParent");
  db.affect("f", "update");
  buildDagNodes();
}

// ************************************************************************* //
void	NVelocityPlot::buildDagNodes()
{
  DagNode	*oldParent(_m_parent);
  DagNode	*tr = Command::run(String("new -type Transform -path :"));
  DagNode	*clock = DagNode::getNodeByPath(":Clock0");

  if (oldParent)
    DagNode::parent(*oldParent, *tr);
  DagNode::parent(*tr, *this);
  DagNode::connect(*oldParent, "renderIn", *tr, "renderIn");
  DagNode::connect(*tr, "renderIn", *this, "renderIn");
  DagNode::connect(*tr, "matChild", *this, "matParent");
  DagNode::connect(*clock, "f", *this, "f");
  DagNode::connect(*clock, "dt", *this, "dt");
}

// ************************************************************************* //
void	NVelocityPlot::compute(Plug &plugOut)
{
  // TODO: 
  if (plugOut.shortName() == "imatParent")
    plugOut.writeAsMat4d(readAsMat4d("matParent").inverse());
  else if (plugOut.shortName() == "update")
    {
      const int	f = readAsInt("f");
      //TODO:
      plugOut.writeAsSignal();
    }
  else if (plugOut.shortName() == "renderOut")
    {
      //static GLUquadric	*quad = gluNewQuadric();
      const int		sel(readAsInt("sel"));
      const Mat4d	&matParent(readAsMat4d("matParent"));
      const Mat4d	&imatParent(readAsMat4d("imatParent"));
      const double	dt = readAsDouble("dt");

      readAsSignal("renderIn");
      pushName();

      glPushAttrib(GL_LIGHTING_BIT);
      glDisable(GL_LIGHTING);

      if (sel == 1)
	glColor3d(1.0, 0.0, 0.5);
      else if (sel)
	glColor3d(1.0, 0.5, 0.0);
      else
	glColor3d(0.0, 0.0, 1.0);

      DagNode	*rs = DagNode::getNodeByPath(":VortexSet0");
      const bool	cached(readAsBool("cached"));
      Plug	*plugp(cached? rs->getPlug("ptotal"): rs->getPlug("p"));
      Plug	*plugTlog(cached? rs->getPlug("rlogTotal"):
			  rs->getPlug("rlog"));

      /*Plug	*plugpo(rs->getPlug("poctree"));
      Plug	*plugTlogo(rs->getPlug("rlogOctree"));
      */

      glPushMatrix();
      imatParent.glMultMatrix();

      glBegin(GL_LINE_LOOP);
      (matParent * Pnt3d(0.0, 0.0, 0.0)).glVertex();
      (matParent * Pnt3d(1.0, 0.0, 0.0)).glVertex();
      (matParent * Pnt3d(1.0, 1.0, 0.0)).glVertex();
      (matParent * Pnt3d(0.0, 1.0, 0.0)).glVertex();
      glEnd();//GL_LINE_LOOP

      glBegin(GL_LINES);
      for (unsigned i(0), n(readAsInt("n")); i < n; ++i)
	for (unsigned j(0); j < n; ++j)
	  {
	    const Pnt3d	p = matParent * Pnt3d(i / (n - 1.0), 
					      j / (n - 1.0), 
					      0.0);
	    plugp->writeAsPnt3d(p);
	    //plugpo->writeAsPnt3d(p);
	    //if (1.0e-8 < plugTlogo->readAsTwistLog().norm2())
	      {
		p.glVertex();
		(p + (plugTlog->readAsTwistLog() * dt) * p).glVertex();
	      }
	  }
      glEnd();//GL_LINES

      glPopMatrix();
      glPopAttrib();//GL_LIGHTING_BIT

      popName();
      plugOut.writeAsSignal();
    }
}
