// ************************************************************************* //
/// GBB - Graphics Blue Box, by Alexis Angelidis
/// $Author: silex $
/// $Date: 2006/01/11 21:56:30 $
/// $Revision: 1.1 $
// ************************************************************************* //
#include "nodevortexset.h"
#include "datablock.h"
#include "nodesmokeparticles.h"
#include "nodesmokegpuparticles.h"
#include "nodeobstacle.h"
#include "plugcustom.h"
#include "plugoctree.h"
#include "command.h"
#include "plugmesh.h"//DEBUG: TOREM
#include "nodenoise.h"

// ************************************************************************* //
int	NVortexSet::typeNbr(46);
String	NVortexSet::typeStr("VortexSet");

// ************************************************************************* //
int	PlugNodeList::typeNbr(73);
String	PlugNodeList::typeStr("NodeList");

// ************************************************************************* //
/// DESCRIPTION: One factory per scene!
/// TODO: Buildin the rings in the set (no need for rings to be in the graph)
///     same thing for smoke particles!!!
///  rlog	- the twist at p
///  jac	- the jacobian at p
///  rlogOctree	- rlog cached in octree
///  jacOctree	- jac cached in octree
///  rlogTotal	- rlogOctree with collisions
///  jacTotal	- jacOctree with collisions
///  rlogNoisy  - rlogOctree with collisions and noise
///  jacNoisy	- jacOctree with collisions and noise
// ************************************************************************* //
void	NVortexSet::initialize()
{
  DataBlock	&db(getDataBlock());

  db.addInt(0, 0, MAXINT, "renderIn", false);
  db.addSignal("renderOut", false);
  db.addInt(0, MININT, MAXINT, "f", false);//curFrame
  db.addDouble(0, 0.0, MAXDOUBLE, "dt", false);
  db.addInt(0, MININT, MAXINT, "fPrev", false);//framePrevious
  db.addSignal("update", false);
  db.addPnt3d(Pnt3d(0.0), "p", false);
  db.addPnt3d(Pnt3d(0.0), "poctree", false);
  db.addPnt3d(Pnt3d(0.0), "ptotal", false);
  db.addPnt3d(Pnt3d(0.0), "pnoisy", false);
  db.addTwistLog(TwistLog(0.0), "rlog", false);
  db.addTwistLog(TwistLog(0.0), "rlogOctree", false);
  db.addTwistLog(TwistLog(0.0), "rlogTotal", false);
  db.addTwistLog(TwistLog(0.0), "rlogNoisy", false);
  db.addMat3d(Mat3d::identity(), "jac", false);
  db.addMat3d(Mat3d::identity(), "jacOctree", false);
  db.addMat3d(Mat3d::identity(), "jacTotal", false);
  db.addMat3d(Mat3d::identity(), "jacNoisy", false);
  db.addSignal("signal", false);
  db.addCustom(*new PlugSmokeParticles(db), "sp", false);
  db.addCustom(*new PlugSmokeGPUParticles(db), "spGPU", false);
  db.addCustom(*new PlugBBox3d(db), "bboxRings", false);
  db.addCustom(*new PlugBBox3d(db), "bboxSmoke", false);
  db.addCustom(*new PlugBBox3d(db), "bboxSmokeGPU", false);
  db.addCustom(*new PlugBBox3d(db), "bboxNoise", false);
  db.addCustom(*new PlugBBox3d(db), "bboxOctree", false);
  db.addCustom(*new PlugOctree(db), "octree", false);
  db.addCustom(*new PlugNodeList(db), "nodeList", false, false);
  db.addBool(false, "renderOctree");
  db.addDouble(0.25, 0.0001, MAXDOUBLE, "octreeResolution");
  db.addVec3d(Vec3d(0.0, 0.0, -9.8), MINDOUBLE, MAXDOUBLE, "g");

  db.affect("f", "update");
  db.affect("renderIn", "renderOut");
  db.affect("f", "nodeList");

  //db.affect("bboxRings", "renderOut");
  db.affect("f", "bboxRings");
  db.affect("f", "bboxSmoke");
  db.affect("f", "bboxSmokeGPU");
  db.affect("f", "bboxNoise");
  db.affect("bboxRings", "bboxOctree");
  db.affect("bboxSmoke", "bboxOctree");
  db.affect("bboxSmokeGPU", "bboxOctree");
  db.affect("bboxNoise", "bboxOctree");
  db.affect("bboxOctree", "octree");
  //db.affect("octree", "renderOut");

  db.affect("p", "rlog");
  db.affect("p", "jac");

  db.affect("octree", "rlogOctree");
  db.affect("poctree", "rlogOctree");
  db.affect("octree", "jacOctree");
  db.affect("poctree", "jacOctree");

  db.affect("ptotal", "rlogTotal");
  db.affect("ptotal", "jacTotal");

  db.affect("pnoisy", "rlogNoisy");
  db.affect("pnoisy", "jacNoisy");

  buildDagNodes();
}

// ************************************************************************* //
void	NVortexSet::buildDagNodes()
{
  DagNode	*clock = DagNode::getNodeByPath(":Clock0");

  DagNode::connect(*clock, "f", *this, "f");
  DagNode::connect(*clock, "dt", *this, "dt");
  DagNode::connect(*_m_parent, "renderIn", *this, "renderIn");
}

// ************************************************************************* //
void	NVortexSet::glRender(Plug &plug)
{
  readAsSignal("renderIn");

  pushName();
  if (readAsBool("renderOctree"))
    {
      const Octree<TwistLog> &octree = 
	*static_cast<const Octree<TwistLog>*>(readAsCustom("octree"));
      octree.glRender();
    }

  std::list<DagNode*>::iterator	nit;

  for (nit = _m_children.begin(); nit != _m_children.end(); ++nit)
    (*nit)->readAsSignal("renderOut");

  popName();
  plug.writeAsSignal();
}

// ************************************************************************* //
void	NVortexSet::update(Plug &plugOut)
{
  const int	fPrev = readAsInt("fPrev");
  const int	f = readAsInt("f");

  writeAsInt("fPrev", f);

  std::list<DagNode*>::iterator	nit;
  std::list<DagNode *>	vortexRings;

  if (f == 0 && f < fPrev)
    {
      getChildrenByType("VortexRing", vortexRings);
      for (nit = vortexRings.begin(); nit != vortexRings.end(); ++nit)
	Command::run(String("delete -path ") + (*nit)->path());
    }
  else
    {
      getChildrenByType("VortexRing", vortexRings);
      for (nit = vortexRings.begin(); nit != vortexRings.end(); ++nit)
	(*nit)->readAsSignal("update");

      for (nit = vortexRings.begin(); nit != vortexRings.end(); )
	if ((*nit)->readAsDouble("ttd") < (*nit)->readAsDouble("t"))
	  {
	    Command::run(String("delete -path ") + (*nit)->path());
	    vortexRings.erase(nit++);
	  }
	else
	  ++nit;

      std::list<DagNode *>	nodes;

      getChildrenByType("VortexRing", nodes);
      getNodesByType("CurrentControl", nodes);
      DagNode::root().getNodesByType("Wind", nodes);//TEST!

      writeAsCustom("nodeList", &nodes);
      //for (nit = vortexRings.begin(); nit != vortexRings.end(); ++nit)
      //(*nit)->readAsSignal("update");
    }

  std::list<DagNode *>	currentControls;

  getNodesByType("CurrentControl", currentControls);
  for (nit = currentControls.begin(); nit != currentControls.end(); ++nit)
    (*nit)->readAsSignal("update");

  std::list<DagNode *>	nodeCheckpoints;

  getNodesByType("RingCheckpoint", nodeCheckpoints);
  for (nit = nodeCheckpoints.begin(); nit != nodeCheckpoints.end(); ++nit)
    (*nit)->readAsSignal("update");

  plugOut.writeAsSignal();
}

// ************************************************************************* //
/// DESCRIPTION: compute the log of twist at p. in "meter" per second.
///  multiply result with dt.
///  Uses precomputed list of required nodes.
/// NOTE: To get fast high quality noise, noise should be added to 
///  rlogtotal.
// ************************************************************************* //
void	NVortexSet::computeRlog(Plug &plug)
{
  const Pnt3d	&p = readAsPnt3d("p");
  TwistLog	tlog(0.0);

  std::list<DagNode*>::iterator	nit;
  std::list<DagNode*>	&nodes = 
    *static_cast<std::list<DagNode*> *>(readAsCustom("nodeList"));

  for (nit = nodes.begin(); nit != nodes.end(); ++nit)
    {
      (*nit)->writeAsPnt3d("p", p);
      tlog += (*nit)->readAsTwistLog("rlog");
    }

  std::list<DagNode *>		  noises;

  DagNode::root().getNodesByType("Noise", noises);
  for (nit = noises.begin(); nit != noises.end(); ++nit)
    {
      (*nit)->writeAsPnt3d("p", p);
      tlog += (*nit)->readAsTwistLog("rlog");
    }
  plug.writeAsTwistLog(tlog);
}

// ************************************************************************* //
/// DESCRIPTION: compute the log of twist at p using the octree.
/// NOTE: Points outside of the octree may be queried because of forward finite
///   difference evaluation. Those are forced inside.
// ************************************************************************* //
void	NVortexSet::computeRlogOctree(Plug &plug)
{
  Pnt3d	p = readAsPnt3d("poctree");
  Octree<TwistLog> &octree = 
    *static_cast<Octree<TwistLog>*>(readAsCustom("octree"));
  const OLeaf<TwistLog>	*rleaf[8];

  octree.query8Neighbors(p, rleaf);

  for (unsigned i(0); i < 8; ++i)
     if (!rleaf[i])
      {
	plug.writeAsTwistLog(TwistLog(0.0));
	//std::cerr << "Error: no octree node at " << p << "." << std::endl;
	return;
      }

  double x(0.0), y(0.0), z(0.0); // in [0,1]

  if (rleaf[1]->p.x != rleaf[0]->p.x)
    x = (p.x - rleaf[0]->p.x) / (rleaf[1]->p.x - rleaf[0]->p.x);
  if (rleaf[2]->p.y != rleaf[0]->p.y)
    y = (p.y - rleaf[0]->p.y) / (rleaf[2]->p.y - rleaf[0]->p.y);
  if (rleaf[4]->p.z != rleaf[0]->p.z)
    z = (p.z - rleaf[0]->p.z) / (rleaf[4]->p.z - rleaf[0]->p.z);
  
  const TwistLog	v00x = rleaf[0]->elm * (1 - x) + rleaf[1]->elm * x;
  const TwistLog	v01x = rleaf[2]->elm * (1 - x) + rleaf[3]->elm * x;
  const TwistLog	v10x = rleaf[4]->elm * (1 - x) + rleaf[5]->elm * x;
  const TwistLog	v11x = rleaf[6]->elm * (1 - x) + rleaf[7]->elm * x;

  const TwistLog	v0yx = v00x * (1.0 - y) + v01x * y;
  const TwistLog	v1yx = v10x * (1.0 - y) + v11x * y;

  const TwistLog	vzyx = v0yx * (1.0 - z) + v1yx * z;

  plug.writeAsTwistLog(vzyx);
}

// ************************************************************************* //
/// DESCRIPTION: in m per s
/// NOTE: Currently handles only one obstacle.
/// The field of the obstacle is not added to the octree for accuracy reasons.
// ************************************************************************* //
void	NVortexSet::computeRlogTotal(Plug &plugOut)
{
  std::list<DagNode *>	nodes;

  const Pnt3d	&p = readAsPnt3d("ptotal");
  DagNode::root().getNodesByType("Obstacle", nodes);
  DagNode	*obs = nodes.empty()? 0: nodes.front();

  if (obs)
    {
      Mesh *mesh = static_cast<Mesh *>(obs->readAsCustom("mesh"));
      const Mat4d	&imatParent(obs->readAsMat4d("imatParent"));
      const double	safeZone(obs->readAsDouble("safeZone"));
      const double	smoothZone(obs->readAsDouble("smoothZone"));
      const double	f(mesh->field(imatParent * p, safeZone, smoothZone));

      if (f == 1.0)
	plugOut.writeAsTwistLog(0.0);
      else if (f == 0.0)
	{
	  writeAsPnt3d("poctree", p);
	  obs->writeAsPnt3d("p", p);
	  plugOut.writeAsTwistLog(readAsTwistLog("rlogOctree") +
				  obs->readAsTwistLog("rlog"));
	}
      else
	{
	  writeAsPnt3d("poctree", p);
	  obs->writeAsPnt3d("p", p);
	  plugOut.writeAsTwistLog((readAsTwistLog("rlogOctree") +
				   obs->readAsTwistLog("rlog")) * (1.0 - f));
	}
    }
  else
    {
      writeAsPnt3d("poctree", p);
      plugOut.writeAsTwistLog(readAsTwistLog("rlogOctree"));
    }
}

// ************************************************************************* //
/// DESCRIPTION: Adds a layer of noise over the total field of twists.
/// TODO: optimize with spatial clusters.
// ************************************************************************* //
void	NVortexSet::computeRlogNoisy(Plug &plugOut)
{
  const Pnt3d	&p = readAsPnt3d("pnoisy");
  TwistLog	tlog(0.0);

  // Use everything else (e.g. obstacles). ********************************* //
  writeAsPnt3d("ptotal", p);
  tlog += readAsTwistLog("rlogTotal");

  // Add the noise touch *************************************************** //
  /*
  std::list<DagNode *>		  nodes;
  std::list<DagNode *>::iterator  nit;

  DagNode::root().getNodesByType("Noise", nodes);
  for (nit = nodes.begin(); nit != nodes.end(); ++nit)
    {
      (*nit)->writeAsPnt3d("p", p);
      tlog += (*nit)->readAsTwistLog("rlog");
    }
  */

  // TODO: Add turbulences:
  std::list<DagNode *>		  nodes;
  std::list<DagNode *>::iterator  nit;

  DagNode::root().getNodesByType("Turbulence", nodes);
  for (nit = nodes.begin(); nit != nodes.end(); ++nit)
    {
      (*nit)->writeAsPnt3d("p", p);
      tlog += (*nit)->readAsTwistLog("rlog");
    }

  plugOut.writeAsTwistLog(tlog);
}
 
// ************************************************************************* //
void	NVortexSet::computeBBoxSmoke(Plug &plugOut)
{
  readAsInt("f");
  BBox3d	bboxSmoke(0.0);

  std::list<SmokeParticle>::const_iterator spit;
  const std::list<SmokeParticle> *sp =
    static_cast<std::list<SmokeParticle> *>(readAsCustom("sp"));

  if (!sp->empty())
    {
      bboxSmoke = sp->front().p;
      for (spit = sp->begin(); spit != sp->end(); ++spit)
	bboxSmoke.fill(spit->p);
    }
  plugOut.writeAsCustom(&bboxSmoke);
}

// ************************************************************************* //
void	NVortexSet::computeBBoxSmokeGPU(Plug &plugOut)
{
  readAsInt("f");
  BBox3d	bboxSmokeGPU(0.0);

  std::list<SmokeGPUParticle>::const_iterator spit;
  const std::list<SmokeGPUParticle> *sp =
    static_cast<std::list<SmokeGPUParticle> *>(readAsCustom("spGPU"));

  if (!sp->empty())
    {
      bboxSmokeGPU = sp->front().p;
      for (spit = sp->begin(); spit != sp->end(); ++spit)
	bboxSmokeGPU.fill(spit->p);
    }
  plugOut.writeAsCustom(&bboxSmokeGPU);
}

// ************************************************************************* //
void	NVortexSet::computeBBoxNoise(Plug &plugOut)
{
  readAsInt("f");
  BBox3d	bboxNoise(0.0);

  std::list<DagNode *>		noiseGens;
  std::list<DagNode *>::iterator	nit;

  DagNode::root().getNodesByType("Noise", noiseGens);//DEBUG

  if (!noiseGens.empty())
    {
      std::list<NoiseVortex>	*nv = 
	static_cast<std::list<NoiseVortex> *>
	(noiseGens.front()->readAsCustom("nv"));

      if (!nv->empty())
	bboxNoise = nv->front().c;
      for (nit = noiseGens.begin(); nit != noiseGens.end(); ++nit)
	{
	  std::list<NoiseVortex>	*nv = 
	    static_cast<std::list<NoiseVortex> *>((*nit)->readAsCustom("nv"));
	  std::list<NoiseVortex>::iterator	nvit;

	  for (nvit = nv->begin(); nvit != nv->end(); ++nvit)
	    bboxNoise.fill(nvit->c);
	}
    }
  plugOut.writeAsCustom(&bboxNoise);
}

// ************************************************************************* //
void	NVortexSet::computeBBoxRings(Plug &plugOut)
{
  readAsInt("f");
  BBox3d	bboxRings(0.0);
  bool	isInit(false);
  std::list<DagNode*>::iterator	nit;
  std::list<DagNode *>	nodes;

  getNodesByType("VortexRing", nodes);
  for (nit = nodes.begin(); nit != nodes.end(); ++nit)
    {
      const std::vector<Pnt3d> &samples
	((*nit)->readAsPnt3dArray("motionSamples"));

      if (!isInit && (isInit = true))
	bboxRings = samples.front();
      for (unsigned i(0); i < samples.size(); ++i)
	bboxRings.fill(samples[i]);
    }
  plugOut.writeAsCustom(&bboxRings);
}

// ************************************************************************* //
void	NVortexSet::computeBBoxOctree(Plug &plugOut)
{
  readAsInt("f");

  const BBox3d &bs(*static_cast<const BBox3d*>(readAsCustom("bboxSmoke")));
  const BBox3d &bg(*static_cast<const BBox3d*>(readAsCustom("bboxSmokeGPU")));
  const BBox3d &br(*static_cast<const BBox3d*>(readAsCustom("bboxRings")));
  const BBox3d &bn(*static_cast<const BBox3d*>(readAsCustom("bboxNoise")));

  BBox3d ret(bs + bg + br + bn);

  std::list<DagNode *>	obss;//DEBUG

  DagNode::root().getNodesByType("Obstacle", obss);//DEBUG

  DagNode	*obs = obss.empty()? 0: obss.front();

  if (obs)
    ret = ret + *static_cast<const BBox3d*>(obs->readAsCustom("bbox"));
  plugOut.writeAsCustom(&ret);
}

// ************************************************************************* //
/// DESCRIPTION: Fills a neighborhood around p in the octree. Also adds
///  epsilons to p, for evaluating the jacobian.
///  If you don't need the jacobian at p, then don't use this function.
// ************************************************************************* //
void	fillNeighborhoodJac(Octree<TwistLog>	&octree, 
			    const Pnt3d		&p,
			    Plug		&plugp,
			    Plug		&plugRlog)
{
  OLeaf<TwistLog>	*wleaf[8];
  const Pnt3d		ptab[4] = { p, Pnt3d(p.x + EPSILON_JAC, p.y, p.z),
				    Pnt3d(p.x, p.y + EPSILON_JAC, p.z),
				    Pnt3d(p.x, p.y, p.z + EPSILON_JAC) };
  for (int k(0); k < 4; ++k)
    {
      octree.create8Neighbors(ptab[k], wleaf);
      for (int i(0); i < 8; ++i)
	if (wleaf[i])
	  {
	    plugp.writeAsPnt3d(wleaf[i]->p);
	    wleaf[i]->elm = plugRlog.readAsTwistLog();
	  }
    }
}

// ************************************************************************* //
/// DESCRIPTION: Fills a neighborhood around p in the octree. Also adds
///  epsilons to p, for evaluating the jacobian.
///  If you don't need the jacobian at p, then don't use this function.
// ************************************************************************* //
void	fillNeighborhood(Octree<TwistLog>	&octree, 
			 const Pnt3d		&p,
			 Plug			&plugp,
			 Plug			&plugRlog)
{
  OLeaf<TwistLog>	*wleaf[8];

  octree.create8Neighbors(p, wleaf);
  for (int i(0); i < 8; ++i)
    if (wleaf[i])
      {
	plugp.writeAsPnt3d(wleaf[i]->p);
	wleaf[i]->elm = plugRlog.readAsTwistLog();
      }
}

// ************************************************************************* //
/// DESCRIPTION: Puts in the octree the places where the twist will be 
///  evaluated.
// ************************************************************************* //
void	NVortexSet::computeOctree(Plug &plug)
{
  Plug &plugRlog(*getPlug("rlog"));
  Plug &plugp(*getPlug("p"));

  const BBox3d	&bs(*static_cast<const BBox3d*>(readAsCustom("bboxOctree")));
  Octree<TwistLog>	&octree = 
    *static_cast<Octree<TwistLog>*>(readAsCustom("octree"));

  // TODO: compute the number of required bits.
  octree.resizeCube(bs, readAsDouble("octreeResolution"));

  // Fill octree with particle positions *********************************** //
  std::list<SmokeParticle>::const_iterator spit;
  const std::list<SmokeParticle> *sp = 
    static_cast<std::list<SmokeParticle> *>(readAsCustom("sp"));

  for (spit = sp->begin(); spit != sp->end(); ++spit)
    fillNeighborhoodJac(octree, spit->p, plugp, plugRlog);

  // Fill octree with GPU particle positions ******************************* //
  std::list<SmokeGPUParticle>::const_iterator spitGPU;
  const std::list<SmokeGPUParticle> *spGPU = 
    static_cast<std::list<SmokeGPUParticle> *>(readAsCustom("spGPU"));
  
  for (spitGPU = spGPU->begin(); spitGPU != spGPU->end(); ++spitGPU)
    fillNeighborhoodJac(octree, spitGPU->p, plugp, plugRlog);

  // Fill octree with ring positions *************************************** //
  std::list<DagNode*>::iterator	nit;
  std::list<DagNode *>	nodes;

  getNodesByType("VortexRing", nodes);
  for (nit = nodes.begin(); nit != nodes.end(); ++nit)
    {
      const std::vector<Pnt3d>	&samples
	((*nit)->readAsPnt3dArray("motionSamples"));
      std::vector<Pnt3d>::const_iterator	sit;

      for (sit = samples.begin(); sit != samples.end(); ++sit)
	fillNeighborhood(octree, *sit, plugp, plugRlog);
    }

  // Fill octree with obstacle positions *********************************** //
  std::list<DagNode *>	obss;//DEBUG

  DagNode::root().getNodesByType("Obstacle", obss);//DEBUG

  DagNode	*obs = obss.empty()? 0: obss.front();

  if (obs)
    {
      const std::vector<Pnt3d> &ssamples = 
	obs->readAsPnt3dArray("ssamplesWorld");
      for (unsigned i(0); i < ssamples.size(); ++i)
	fillNeighborhood(octree, ssamples[i], plugp, plugRlog);

      // Fill octree with obstacle vortex positions ************************ //
      std::list<OVortex>	&ov = 
	static_cast<DataObstacleVortex *>(obs->readAsCustom("ovortex"))->
	_m_ovortices;
      std::list<OVortex>::iterator	oit;
      
      for (oit = ov.begin(); oit != ov.end(); ++oit)
	fillNeighborhood(octree, oit->c, plugp, plugRlog);
    }

  // Cache twist at noise vortices positions ******************************* //
  {
    std::list<DagNode *>		noiseGens;
    std::list<DagNode *>::iterator	nit;

    DagNode::root().getNodesByType("Noise", noiseGens);//DEBUG
    for (nit = noiseGens.begin(); nit != noiseGens.end(); ++nit)
      {
	std::list<NoiseVortex>	*nv = 
	  static_cast<std::list<NoiseVortex> *>((*nit)->readAsCustom("nv"));
	std::list<NoiseVortex>::iterator	nvit;

	for (nvit = nv->begin(); nvit != nv->end(); ++nvit)
	  fillNeighborhoodJac(octree, nvit->c, plugp, plugRlog);
      }
  }

  plug.writeAsSignal();
}

// ************************************************************************* //
/// DESCRIPTION: Computes the jacobian matrix of plugRlog(plugp).
// ************************************************************************* //
void	NVortexSet::computeJac(Plug	&plugOut,
			       Plug	&plugp,
			       Plug	&plugRlog)
{
  const double	dt(readAsDouble("dt"));
  Pnt3d	p = plugp.readAsPnt3d(), px(p), py(p), pz(p);

  plugp.writeAsPnt3d(p);
  (plugRlog.readAsTwistLog() * dt).exp(p);
  px.x += EPSILON_JAC;
  plugp.writeAsPnt3d(px);
  (plugRlog.readAsTwistLog() * dt).exp(px);
  py.y += EPSILON_JAC;
  plugp.writeAsPnt3d(py);
  (plugRlog.readAsTwistLog() * dt).exp(py);
  pz.z += EPSILON_JAC;
  plugp.writeAsPnt3d(pz);
  (plugRlog.readAsTwistLog() * dt).exp(pz);
  plugOut.writeAsMat3d(Mat3d(px - p, py - p, pz - p) * 1.0e8);
}

// ************************************************************************* //
void	NVortexSet::compute(Plug &plugOut)
{
  if (plugOut.shortName() == "rlogOctree")
    computeRlogOctree(plugOut);
  else if (plugOut.shortName() == "jacOctree")
    computeJac(plugOut, *getPlug("poctree"), *getPlug("rlogOctree"));
  else if (plugOut.shortName() == "rlogNoisy")
    computeRlogNoisy(plugOut);
  else if (plugOut.shortName() == "jacNoisy")
    computeJac(plugOut, *getPlug("pnoisy"), *getPlug("rlogNoisy"));
  else if (plugOut.shortName() == "rlog")
    computeRlog(plugOut);
  else if (plugOut.shortName() == "jac")
    computeJac(plugOut, *getPlug("p"), *getPlug("rlog"));
  else if (plugOut.shortName() == "rlogTotal")
    computeRlogTotal(plugOut);
  else if (plugOut.shortName() == "jacTotal")
    computeJac(plugOut, *getPlug("ptotal"), *getPlug("rlogTotal"));
  else if (plugOut.shortName() == "renderOut")
    glRender(plugOut);
  else if (plugOut.shortName() == "update")
    update(plugOut);
  else if (plugOut.shortName() == "octree")
    computeOctree(plugOut);
  else if (plugOut.shortName() == "bboxRings")
    computeBBoxRings(plugOut);
  else if (plugOut.shortName() == "bboxSmoke")
    computeBBoxSmoke(plugOut);
  else if (plugOut.shortName() == "bboxSmokeGPU")
    computeBBoxSmokeGPU(plugOut);
  else if (plugOut.shortName() == "bboxNoise")
    computeBBoxNoise(plugOut);
  else if (plugOut.shortName() == "bboxOctree")
    computeBBoxOctree(plugOut);
  else if (plugOut.shortName() == "nodeList")
    {
      std::list<DagNode *>	nodes;

      readAsInt("f");
      getChildrenByType("VortexRing", nodes);
      getNodesByType("CurrentControl", nodes);
      DagNode::root().getNodesByType("Wind", nodes);//TEST!
      plugOut.writeAsCustom(&nodes);
    }
}
