// ************************************************************************* //
/// AUTHOR Alexis Angelidis.
/// STARTED Thu Apr  8 10:39:33 NZST 2004
// ************************************************************************* //
#ifndef __PBUFFER_HH__
#define __PBUFFER_HH__

// ************************************************************************* //
#include <GL/glu.h>
#include "gfxTypes.hh"

// ************************************************************************* //
class PBuffer
{
  GLXContext	_m_curCtx;
  GLXDrawable	_m_curDraw, _m_curReadDraw;
  GLXPbuffer	_m_pBufDraw;
  GLXContext	_m_pBufCtx;
  Display	*_m_dpy;
  int		_m_screen;
  const int	_m_w, _m_h;
public:
  PBuffer(Display *dpy, const int screen,
	  const int w, const int h, 
	  const int cBits, const int dBits);
  bool		enable();
  bool		disable();
  static bool	implemented();
  bool		isCurrent() const;
};

// ************************************************************************* //
/// DESCRIPTION:
///  use glXGetCurrentDisplay() to get current display
///  use XDefaultScreen(Display *) to get default screen
///  The pbuffer shares lists with current context.
// ************************************************************************* //
inline PBuffer::PBuffer(Display		*dpy,
			const int	screen,
			const int	w,
			const int	h, 
			const int	cBits,
			const int	dBits):
  _m_dpy(dpy), _m_screen(screen), _m_w(w), _m_h(h)
{
  int	nConfigs(0);
  int	attr[64], i(0);

  attr[i++] = GLX_RED_SIZE, attr[i++] = cBits;
  attr[i++] = GLX_GREEN_SIZE, attr[i++] = cBits;
  attr[i++] = GLX_BLUE_SIZE, attr[i++] = cBits;
  attr[i++] = GLX_ALPHA_SIZE, attr[i++] = cBits;
  attr[i++] = GLX_DEPTH_SIZE, attr[i++] = dBits;
  attr[i++] = GLX_DRAWABLE_TYPE, attr[i++] = GLX_PBUFFER_BIT;
  attr[i++] = GLX_RENDER_TYPE, attr[i++] = GLX_RGBA_BIT;
  attr[i++] = GLX_DOUBLEBUFFER, attr[i++] = false;
  attr[i] = 0;

  GLXFBConfig *fbc = glXChooseFBConfig(_m_dpy, _m_screen, attr, &nConfigs);

  if (!nConfigs)
    std::cerr << "Failed to find any FB congif." << std::endl;
  _m_curCtx = glXGetCurrentContext();
  _m_curDraw = glXGetCurrentDrawable();
  _m_curReadDraw = glXGetCurrentReadDrawable();
  i = 0;
  attr[i++] = GLX_PBUFFER_WIDTH; attr[i++] = _m_w;
  attr[i++] = GLX_PBUFFER_HEIGHT; attr[i++] = _m_h;
  attr[i++] = GLX_PRESERVED_CONTENTS; attr[i++] = true;
  attr[i] = 0;

  _m_pBufDraw = glXCreatePbuffer(_m_dpy, *fbc, attr);
  if (!_m_pBufDraw)
    std::cerr << "Failed to create PBuffer." << std::endl;
  _m_pBufCtx = 
    glXCreateNewContext(_m_dpy, *fbc, GLX_RGBA_TYPE, _m_curCtx, true);
  if (!_m_pBufCtx)
    std::cerr << "Failed to create PBuffer context." << std::endl;
  XFree(fbc);
}

// ************************************************************************* //
/// DESCRIPTION: Enable the pBuffer.
// ************************************************************************* //
inline bool	PBuffer::enable()
{
  return glXMakeContextCurrent(_m_dpy, _m_pBufDraw, _m_pBufDraw, _m_pBufCtx);
}

// ************************************************************************* //
/// DESCRIPTION: Disable the pBuffer.
// ************************************************************************* //
inline bool	PBuffer::disable()
{
  return glXMakeContextCurrent(_m_dpy, _m_curDraw, _m_curReadDraw, _m_curCtx);
}

// ************************************************************************* //
/// DESCRIPTION: Returns true if pbuffer works for this implementation
///  of openGL.
// ************************************************************************* //
inline bool	PBuffer::implemented()
{
  Display	*dpy(glXGetCurrentDisplay());
  int		screen(XDefaultScreen(dpy));
  const char	*extStr(glXQueryExtensionsString(dpy, screen));
  int		min(0), maj(0);

  glXQueryVersion(dpy, &maj, &min);
  return maj == 1 && 3 == min && extStr && strstr(extStr, "GLX_SGIX_pbuffer");
}

// ************************************************************************* //
/// DESCRIPTION: Returns true if the pbuffer is the current context
// ************************************************************************* //
inline bool	PBuffer::isCurrent() const
{
  return _m_pBufCtx == glXGetCurrentContext() &&
    _m_pBufDraw == glXGetCurrentDrawable() &&
    _m_pBufDraw == glXGetCurrentReadDrawable();
}

// ************************************************************************* //
#endif//__PBUFFER_HH__
