// ************************************************************************* //
/// AUTHOR: Alexis Angelidis
/// STARTED: Fri Aug 26 11:11:19 NZST 2005
// ************************************************************************* //
#ifndef __PLUGCUSTOM_HH__
#define __PLUGCUSTOM_HH__

// ************************************************************************* //
#include "gfxtypes.h"
#include "mystring.h"
#include "plug.h"

// ************************************************************************* //
/// DESCRIPTION: Extends a data node. This is stored in the scene graph.
// ************************************************************************* //
class PlugBBox3d: public Plug
{
protected:
  BBox3d	*_m_data;
public:
  static int	typeNbr;
  static String	typeStr;
public:
  PlugBBox3d(DataBlock &db): Plug(db), _m_data(0) { newData(); }
  virtual ~PlugBBox3d() { freeData(); }
  static Plug	*creator(DataBlock &db) { return new PlugBBox3d(db); }
  virtual int	getTypeNbr() const { return typeNbr; }
  virtual const String	&getTypeStr() const { return typeStr; }
  virtual void	freeData() { if (_m_data) delete _m_data; _m_data = 0; }
  virtual void	newData() { if (!_m_data) _m_data = new BBox3d; }
  virtual void	saveData(std::ostream &os) { os << *_m_data; }
  virtual void	loadData(std::istream &is) { is >> *_m_data; }
  virtual void	copy(const void *v)
  { *_m_data = *static_cast<const BBox3d*>(v); }
  virtual void	**rawdata() { return (void**)(&_m_data); }
};

// ************************************************************************* //
template <class T, int N, int M, int NM = N * M>
class Table
{
  T	_m_m[NM];
public:
  Table() {}
  T		&operator()(int i, int j) { return _m_m[i + M * j]; }
  const T	&operator()(int i, int j) const { return _m_m[i + M * j]; }
  friend std::ostream &operator<<(std::ostream &os, const Table<T, N, M> &t)
  { for (int i(0); i < NM; ++i) os << t._m_m[i] << ' '; return os; }
  friend std::istream &operator>>(std::istream &is, Table<T, N, M> &t)
  { for (int i(0); i < NM; ++i) is >> t._m_m[i]; return is; }
};

// ************************************************************************* //
template <class T>
class DTable;

template <class T>
std::ostream &operator<<(std::ostream &os, const DTable<T> &t);

template <class T>
std::istream &operator>>(std::istream &is, DTable<T> &t);

// ************************************************************************* //
/// DESCRIPTION: Dynamic-size version of Table.
// ************************************************************************* //
template <class T>
class DTable
{
  int	_m_r;
  int	_m_c;
  T	*_m_m;
public:
  DTable(): _m_r(0), _m_c(0), _m_m(0) {}
  DTable(const int r, const int c): _m_r(r), _m_c(c), _m_m(new T[r * c]) {}
  DTable(const DTable &t);
  virtual ~DTable() { if (_m_m) delete[] _m_m; }
  T		&operator()(int i, int j) { return _m_m[i + _m_r * j]; }
  const T	&operator()(int i, int j) const { return _m_m[i + _m_r * j]; }
  void		resize(const int r, const int c);
  DTable<T>	&operator=(const DTable<T> &t);
  template <class T_>
  friend	std::ostream &operator<<(std::ostream &os, const DTable<T_>&t);
  template <class T_>
  friend	std::istream &operator>>(std::istream &is, DTable<T_> &t);
};

// ************************************************************************* //
template <class T>
inline std::ostream &operator<<(std::ostream &os, const DTable<T> &t)
{
  os << t._m_r << ' ' << t._m_c << ' ';
  for (int i(0); i < t._m_r * t._m_c; ++i)
    os << t._m_m[i] << ' ';
  return os;
}

// ************************************************************************* //
template <class T>
inline std::istream &operator>>(std::istream &is, DTable<T> &t)
{
  is >> t._m_r >> t._m_c;
  if (t._m_m)
    delete[] t._m_m;
  t._m_m = new T[t._m_r * t._m_c];
  for (int i(0); i < t._m_r * t._m_c; ++i)
    is >> t._m_m[i];
  return is;
}

// ************************************************************************* //
template <class T>
DTable<T>::DTable(const DTable &t): _m_r(t._m_r), _m_c(t._m_c), _m_m(0)
{
  if (t._m_m)
    {
      const int	size(t._m_r * t._m_c);

      _m_m = new T[size];
      for (int i(0); i < size; ++i)
	_m_m[i] = t._m_m[i];
    }
}

// ************************************************************************* //
/// DESCRIPTION: Memory is reallocated only if desired size if larger than
///  existing size.
// ************************************************************************* //
template <class T>
inline void	DTable<T>::resize(const int r,
				  const int c)
{
  if (r == _m_r && c == _m_c)
    return;
  if (r * c <= _m_r * _m_c)
    {
      _m_r = r;
      _m_c = c;
      return;
    }
  if (_m_m)
    delete[] _m_m;
  _m_r = r;
  _m_c = c;
  _m_m = new T[r * c];
}

// ************************************************************************* //
template <class T>
inline DTable<T>	&DTable<T>::operator=(const DTable<T> &t)
{
  const int	size(t._m_r * t._m_c);

  if (_m_m)
    {
      delete[] _m_m;
      _m_m = 0;
    }
  _m_r = t._m_r;
  _m_c = t._m_c;
  if (t._m_m)
    {
      _m_m = new T[size];
      for (int i(0); i < size; ++i)
	_m_m[i] = t._m_m[i];
    }
  return *this;
}

// ************************************************************************* //
/// DESCRIPTION: Extends a data node. This is stored in the scene graph.
// ************************************************************************* //
class PlugSky: public Plug
{
public:
  // *********************************************************************** //
  struct DataSky
  {
    Pnt3d	pNear;
    Pnt3d	pFar;
    Rgb3d	p1;
    DataSky() {}
    friend std::ostream &operator<<(std::ostream &os, const DataSky &es)
    { return os << es.pNear << ' ' << es.pFar << ' ' << es.p1; }
    friend std::istream &operator>>(std::istream &is, DataSky &es)
    { return is >> es.pNear >> es.pFar >> es.p1; }
  };
protected:
  DTable<DataSky>	*_m_data;
public:
  static int	typeNbr;
  static String	typeStr;
public:
  PlugSky(DataBlock &db): Plug(db), _m_data(0) { newData(); }
  virtual ~PlugSky() { freeData(); }
  static Plug	*creator(DataBlock &db) { return new PlugSky(db); }
  virtual int	getTypeNbr() const { return typeNbr; }
  virtual const String	&getTypeStr() const { return typeStr; }
  virtual void	freeData() { if (_m_data) delete _m_data; _m_data = 0; }
  virtual void	newData() { if (!_m_data) _m_data = new DTable<DataSky>; }
  virtual void	saveData(std::ostream &os) { os << *_m_data; }
  virtual void	loadData(std::istream &is) { is >> *_m_data; }
  virtual void	copy(const void *v)
  { *_m_data = *static_cast<const DTable<DataSky>*>(v); }
  virtual void	**rawdata() { return (void**)(&_m_data); }
};

// ************************************************************************* //
#endif//__PLUGCUSTOM_HH__
