// ************************************************************************* //
/// GBB - Graphics Blue Box C++ API - Graphic Layer
/// $Author: silex $
/// $Date: 2006/01/11 21:56:30 $
/// $Revision: 1.1 $
// ************************************************************************* //
#ifndef	__QTATTRIBUTEEDITOR_HH__
#define	__QTATTRIBUTEEDITOR_HH__

// ************************************************************************* //
#include <qmainwindow.h>
#include <qcheckbox.h>
#include <qlayout.h>
#include <iostream>
#include "qtSpinBoxPnt3d.h"
#include "qtSpinBoxVec3d.h"
#include "qtSpinBoxRgb3d.h"
#include "datablock.h"

// ************************************************************************* //
class DagNode;
class String;
class PlugInt;
class PlugDouble;
class PlugPnt3d;
class PlugVec3d;
class PlugRgb3d;

// ************************************************************************* //
/// DESCRIPTION: Call onto a class, to show its attributes.
///  Uses datablock Connect obj with as many spinboxes as required.
// ************************************************************************* //
class AttributeEditor: public QMainWindow
{
  Q_OBJECT
private:
  DagNode	*_m_node;
public:
  AttributeEditor(DagNode &node, QWidget *parent = 0);
  virtual ~AttributeEditor();
  virtual void keyPressEvent(QKeyEvent *e);
protected:
  virtual void	paintEvent (QPaintEvent *);
public slots:
  void	editCurves();
  void	editParent();
};

// ************************************************************************* //
/// DESCRIPTION: Call onto a class, to show its attributes.
///  Uses datablock Connect obj with as many spinboxes as required.
// ************************************************************************* //
class AttributeCentral: public QWidget
{
  Q_OBJECT
public:
  AttributeCentral(DagNode &node, const int size, QWidget *parent = 0);
};

// ************************************************************************* //
class InputEditorLauncher: public QObject
{
  Q_OBJECT
private:
  DagNode	&_m_node;
  Plug		&_m_data;
  QWidget	*_m_edited;
public:
  InputEditorLauncher(DagNode &node, Plug &data, QWidget *edited = 0,
		      QWidget *parent = 0);
public slots:
  void	editInput();
  void	editOutput();
signals:
  void	valueChanged(Rgb3d c);
};

// ************************************************************************* //
class AttributeName: public QWidget
{
  Q_OBJECT
  String	_m_name;
public:
  AttributeName(QWidget *parent = 0, const char *name = 0);
protected:
  virtual void	paintEvent(QPaintEvent *);
};

// ************************************************************************* //
class AttrBool: public QWidget
{
  Q_OBJECT
  Plug	*_m_data;
public:
  AttrBool(Plug &data, QWidget *parent = 0);
public slots:
  void	toggle(bool on);
};

// ************************************************************************* //
class AttrInt: public QWidget
{
  Q_OBJECT
  PlugInt	*_m_data;
public:
  AttrInt(PlugInt &data, QWidget *parent = 0);
public slots:
  void	setValue(int v);
};

// ************************************************************************* //
class AttrDouble: public QWidget
{
  Q_OBJECT
  PlugDouble	*_m_data;
public:
  AttrDouble(PlugDouble &data, QWidget *parent = 0);
public slots:
  void	setValue(double v);
};

// ************************************************************************* //
class AttrString: public QWidget
{
  Q_OBJECT
  Plug	*_m_data;
public:
  AttrString(Plug &data, QWidget *parent = 0);
public slots:
  void	setText(const QString &v);
};

// ************************************************************************* //
class AttrPnt3d: public QWidget
{
  Q_OBJECT
  Plug	*_m_data;
public:
  AttrPnt3d(PlugPnt3d &data, QWidget *parent = 0);
public slots:
  void	setValue(Pnt3d v);
};

// ************************************************************************* //
class AttrVec3d: public QWidget
{
  Q_OBJECT
  Plug	*_m_data;
public:
  AttrVec3d(PlugVec3d &data, QWidget *parent = 0);
public slots:
  void	setValue(Vec3d v);
};

// ************************************************************************* //
class AttrRgb3d: public QWidget
{
  Q_OBJECT
  Plug		*_m_data;
  SpinBoxRgb3d	*_m_sp;
public:
  AttrRgb3d(PlugRgb3d &data, QWidget *parent = 0);
  Rgb3d	getValue() const;
public slots:
  void	setValue(Rgb3d v);
};

// ************************************************************************* //
#endif//__QTATTRIBUTEEDITOR_HH__
