// ************************************************************************* //
/// GBB - Graphics Blue Box, by Alexis Angelidis
/// $Author: silex $
/// $Date: 2006/01/11 21:56:30 $
/// $Revision: 1.1 $
// ************************************************************************* //
#include "qtClockEditor.h"
#include "qtMainWidget.h"

// ************************************************************************* //
#include <qcursor.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <QImage>
#include "plug.h"
#include "datablock.h"
#include "dagnode.h"
#include "nodeclock.h"
#include "nodecurve.h"
#include "play.xpm"
#include "pause.xpm"
#include "stop.xpm"
#include "dataclock.h"

// ************************************************************************* //
FrameSlider::FrameSlider(ClockEditor *parent):
  QWidget(parent), _m_parent(parent)
{
  setCursor(QCursor(Qt::IBeamCursor));
  setBackgroundRole(QPalette::Background);
  //setBackgroundMode(Qt::NoBackground);
}

// ************************************************************************* //
void	FrameSlider::drawRuler(QPainter &p)
{
  const int	h(height()), w(width());
  const int	interval(_m_parent->endFrame() - _m_parent->startFrame());
  const int	tat(4 + ((w - 8) * (_m_parent->curFrame() -
				    _m_parent->startFrame())) / interval);

  bool	isOverKey(false), isOverRuler(false), isOverDash(false);
  int	maxDigit(0), ciFps(1), iFps;
  int	sqWidth(w / interval);

  if (sqWidth == 0)
    sqWidth = 1;

  // Draws Ruler Dashes. If they are too close, substitutes the lot with a 
  // single rectangle.
  p.setBrush(Qt::gray);
  if ((w - 8) / (2 * interval))
    for (int qt(_m_parent->startFrame());
	 qt <= _m_parent->endFrame(); ++qt)
      {
	const int t(4 + (w - 8) * (qt - _m_parent->startFrame()) /
		    interval);

	if (t == tat)
	  isOverDash = true;
	else if (4 <= t && t < w - 5)
	  p.drawRect(t, (3 * h) / 4 - 3, 1, h / 4 - 1);
      }
  else
    {
      p.setBrush(QColor((QColor(Qt::gray).red() + 189) / 2,
			(QColor(Qt::gray).red() + 189) / 2, 
			(QColor(Qt::gray).red() + 189) / 2));
      p.drawRect(0, (3 * h) / 4 - 3, w, h / 4 - 1);
    }

  //Finds the interval between rulers, in number of frames.
  for (int end(_m_parent->endFrame()); end != 0; end /= 10)
    ++maxDigit;
  ciFps = ((maxDigit * (h / 2 - 1 + 8)) * interval) / (w - 10);
  iFps = 1;
  for (int i(1), b(1); b < ciFps; ++i, b = !(i % 3)? 2 * b + b / 2: 2 * b)
    iFps = b;

  // Draw Ruler Numbers
  p.setBrush(Qt::black);
  p.setFont(QFont("AnyStyle", h / 2 - 7));
  for (int qt(_m_parent->startFrame() / iFps);
       qt <= _m_parent->endFrame() / iFps; ++qt)
    {
      const int	t(4 + (w - 8) * 
		  (iFps * qt - _m_parent->startFrame()) / interval);

      if (t == tat)
	isOverRuler = true;
      else if (4 <= t && t < w - 5)
	p.drawRect(t, h / 2, 2, h / 2 - 5);
      p.drawText(t, h / 2 - 1, QString::number(iFps * qt));
    }

  // Draw cursor
  p.setBrush(Qt::black);
  p.drawRect(tat, 3, sqWidth, h - 6);
  if (isOverRuler)
    p.setBrush(Qt::gray), p.drawRect(tat, h / 2, 2, h / 2 - 5);
  if (isOverDash)
    p.setBrush(Qt::gray), p.drawRect(tat, (3 * h) / 4 - 3, 1, h / 4 - 1);
  if (isOverKey)
    p.setBrush(QColor(87, 255, 255)), p.drawRect(tat, 3, 1, h - 6);
}

// ************************************************************************* //
/// DESCRIPTION: Draws rulers, current time and keys.
// ************************************************************************* //
void	FrameSlider::paintEvent(QPaintEvent *)
{
  const int	interval(_m_parent->endFrame() - _m_parent->startFrame());
  const int	h(height()), w(width());
  //QPixmap	pm(size());
  QPainter	p;

  p.begin(this);

  // Draw frame
  p.setPen(Qt::NoPen);
  p.setBrush(QColor(230, 230, 230));
  p.drawRect(0, 0, w, h);
  p.setBrush(QColor(189, 189, 189));
  p.drawRect(QRect(4, 4, w - 9, h - 9));

  drawRuler(p);

  // Draw keys // FIND SELECTED OBJECT, GET CHILDREN, DRAW CURVES KEY!!!
  p.setBrush(QColor(168, 0, 0));
  std::list<DagNode*>	nodeSel, nodeChild;
  std::list<DagNode*>::iterator	nit, njt;

  DagNode::getNodesSelected(nodeSel);
  for (nit = nodeSel.begin(); nit != nodeSel.end(); ++nit)
    if ((*nit)->getNodeParent() && 
	(*nit)->getNodeParent()->getTypeStr() == "Transform")
      nodeSel.push_front((*nit)->getNodeParent());
  for (nit = nodeSel.begin(); nit != nodeSel.end(); ++nit)
    {
      (*nit)->getNodesByDepth(1, nodeChild);
      for (njt = nodeChild.begin(); njt != nodeChild.end(); ++njt)
	if ((*njt)->getTypeNbr() == NCurve::typeNbr)
	  {
	    const std::list<Pnt2d>	&ci((*njt)->getPlug("ci")->
					    readAsPnt2dList());
	    std::list<Pnt2d>::const_iterator	cit;
	    for (cit = ci.begin(); cit != ci.end(); ++cit)
	      {
		const int	t(4 + ((w - 8) * 
				       (int(cit->x) - 
					_m_parent->startFrame())) / interval);

		p.drawRect(t, 4, 1, h - 9);//20 controls
	      }
	  }
    }

  // Draw frame over text
  p.setBrush(QColor(114, 98, 98));
  p.drawRect(2, 2, w - 5, 1), p.drawRect(2, 3, w - 6, 1);
  p.drawRect(2, 4, 1, h - 7), p.drawRect(3, 4, 1, h - 8);
  p.setBrush(QColor(237, 224, 224));
  p.drawRect(4, h - 5, w - 7, 1), p.drawRect(3, h - 4, w - 6, 1);
  p.drawRect(w - 5, 4, 1, h - 9), p.drawRect(w - 4, 3, 1, h - 8);
  p.setBrush(QColor(230, 230, 230));
  p.drawRect(0, 0, 2, h - 1);
  p.drawRect(w - 3, 0, 3, h - 1);
  p.end();
  //bitBlt(this, -1, -1, &pm, -1, -1);//!!!
}

// ************************************************************************* //
void	FrameSlider::mousePressEvent(QMouseEvent *e)
{
  if (e->x() < 4 || width() - 5 <= e->x())
    return;
  const double	interval(_m_parent->endFrame() - _m_parent->startFrame());
  const double	t(_m_parent->startFrame() +
		  double((e->x() - 4) * interval) / (width() - 10));

  _m_parent->setCurFrame(int(t));
  emit repaintglwidget();
  setFocus();
  update();
}

// ************************************************************************* //
void	FrameSlider::mouseMoveEvent(QMouseEvent *e)
{
  if (e->x() < 4 || width() - 5 <= e->x())
    return;
  const double	interval(_m_parent->endFrame() - _m_parent->startFrame());
  const double	t(_m_parent->startFrame() + 
		  double((e->x() - 4) * interval) / (width() - 10));

  _m_parent->setCurFrame(int(t));
  emit repaintglwidget();
  setFocus();
  update();
}

// ************************************************************************* //
ClockEditor::ClockEditor(QWidget *parent):
  QWidget(parent)
  //,
  //_m_clock(static_cast<NClock&>(*DagNode::getNodeByPath(":Clock0")))
{
  DagNode	*clock = DagNode::getNodeByPath(":Clock0");
  QHBoxLayout	*grid = new QHBoxLayout(this);
  QPushButton	*play = new QPushButton(this);
  QPushButton	*pause = new QPushButton(this);
  QPushButton	*stop = new QPushButton(this);
  QPixmap	playIcon((const char **)play_xpm);
  QPixmap	pauseIcon((const char **)pause_xpm);
  QPixmap	stopIcon((const char **)stop_xpm);

  _m_frameSlider = new FrameSlider(this);
  _m_startFrameSP = new QSpinBox(this);
  _m_endFrameSP = new QSpinBox(this);
  _m_startFrameSP->setRange(0, MAXINT);
  _m_endFrameSP->setRange(0, MAXINT);


  _m_startFrameSP->setValue(clock->readAsInt("sf"));
  _m_endFrameSP->setValue(clock->readAsInt("ef"));

  QWidget::connect(_m_startFrameSP, SIGNAL(valueChanged(int)),
		   this, SLOT(setValueStartFrame(int)));
  QWidget::connect(_m_endFrameSP, SIGNAL(valueChanged(int)),
		   this, SLOT(setValueEndFrame(int)));
  QWidget::connect(_m_frameSlider, SIGNAL(repaintglwidget()),
		   this, SLOT(passrepaintglwidget()));

  play->setIcon(playIcon);
  pause->setIcon(pauseIcon);
  stop->setIcon(stopIcon);
  play->setCursor(QCursor());
  pause->setCursor(QCursor());
  stop->setCursor(QCursor());
  QWidget::connect(play, SIGNAL(clicked()), this, SLOT(cmdPlay()));
  QWidget::connect(pause, SIGNAL(clicked()), this, SLOT(cmdPause()));
  QWidget::connect(stop, SIGNAL(clicked()), this, SLOT(cmdStop()));

  grid->addWidget(_m_startFrameSP);
  grid->addWidget(_m_frameSlider);

  grid->setStretchFactor(_m_frameSlider, 100);
  grid->addWidget(_m_endFrameSP);
  grid->addWidget(play);
  grid->addWidget(pause);
  grid->addWidget(stop);

  this->cmdStop();
}

// ************************************************************************* //
void	ClockEditor::setValueStartFrame(int value)
{
  DagNode	*clock = DagNode::getNodeByPath(":Clock0");

  clock->writeAsInt("sf", value);
  _m_frameSlider->repaint();
}

// ************************************************************************* //
void	ClockEditor::setValueEndFrame(int value)
{
  DagNode	*clock = DagNode::getNodeByPath(":Clock0");

  clock->writeAsInt("ef", value);
  _m_frameSlider->repaint();
}

// ************************************************************************* //
int	ClockEditor::startFrame() const
{
  return _m_startFrameSP->value();
}

// ************************************************************************* //
int	ClockEditor::endFrame() const
{
  return _m_endFrameSP->value();
}

// ************************************************************************* //
int	ClockEditor::curFrame() const
{
  //_m_clock()
  return static_cast<NClock&>(*DagNode::getNodeByPath(":Clock0")).
    readAsInt("f");
}

// ************************************************************************* //
void	ClockEditor::update() const
{
  DagNode	*clock = DagNode::getNodeByPath(":Clock0");

  _m_startFrameSP->setValue(clock->readAsInt("sf"));
  _m_endFrameSP->setValue(clock->readAsInt("ef"));
}

// ************************************************************************* //
/// TODO: Read bit to force update???
// ************************************************************************* //
void	ClockEditor::setCurFrame(const int f)
{
  static_cast<NClock&>(*DagNode::getNodeByPath(":Clock0")).
    writeAsInt("f", f);
  DagNode::root().readAsSignal("update");
}

// ************************************************************************* //
void	ClockEditor::cmdPlay()
{
  if (_m_isPlaying)
    {
      if (_m_isPaused)
	{
	  _m_isPaused = false;
	  _m_timerId = startTimer(int(1000 * (1.0 / 24.0)));
	}
      return;
    }
  //_m_at = _m_events.begin();
  setCurFrame(0);
  _m_isPlaying = true;
  _m_isPaused = false;
  _m_timerId = startTimer(int(1000 * (1.0 / 24.0)));
}

// ************************************************************************* //
void	ClockEditor::cmdPause()
{
  _m_isPaused = !_m_isPaused;
  if (_m_isPaused)
    killTimer(_m_timerId);
  else
    _m_timerId = startTimer(int(1000 * (1.0 / 24.0)));
}

// ************************************************************************* //
void	ClockEditor::cmdStop()
{
  //_m_at = _m_events.begin();
  setCurFrame(0);
  _m_isPlaying = _m_isPaused = false;
  _m_frameSlider->repaint();
  killTimer(_m_timerId);
  emit repaintglwidget();
}

// ************************************************************************* //
void	ClockEditor::keyPressEvent(QKeyEvent *e)
{
  if (e->key() == Qt::Key_Escape)
    cmdPause();
}

// ************************************************************************* //
/// DESCRIPTION: Sends signal to redraw the gl view.
// ************************************************************************* //
void	ClockEditor::timerEvent(QTimerEvent *)
{
  if (_m_isPaused)
    return;
  if (_m_isPlaying && !MainWindow::mainwindow()->isFrozen())
    {
      DagNode	*clockNode = DagNode::getNodeByPath(":Clock0");
      const int	n(8);
      static Clock	clock;
      static double	tics[n];
      static int	ntics(0), itic(0);
      double	ticave(0.0);

      clock.tic();
      //writeValue("curFrame", readAsInt("curFrame") + 1);
      setCurFrame(curFrame() + 1); // todo: pull frame!
      //update();
      repaint();
      _m_frameSlider->repaint();
      emit repaintglwidget();

      tics[itic % n] = clock.tic();
      for (int i(itic); i < itic + ntics; ++i)
	ticave += tics[i % n];
      ++itic;
      ntics = min(ntics + 1, n);
      ticave /= ntics;
      if (ticave != 0)
	clockNode->writeAsDouble("efps", 1.0 / ticave);
      else
	clockNode->writeAsDouble("efps", 24.0);
      if (_m_endFrameSP->value() < curFrame())
	cmdStop();
    }
}
