// ************************************************************************* //
/// GBB - Graphics Blue Box, by Alexis Angelidis
/// $Author: silex $
/// $Date: 2006/01/11 21:56:30 $
/// $Revision: 1.1 $
// ************************************************************************* //
#ifndef __QTCLOCKEDITOR_HH__
#define __QTCLOCKEDITOR_HH__

// ************************************************************************* //
#include <qwidget.h>
#include <qspinbox.h>

// ************************************************************************* //
class FrameSlider;

// ************************************************************************* //
/// DESCRIPTION: modifies the time.
// ************************************************************************* //
class ClockEditor: public QWidget
{
  Q_OBJECT
  friend class FrameSlider;
protected:
  FrameSlider	*_m_frameSlider;
  QSpinBox	*_m_startFrameSP;
  QSpinBox	*_m_endFrameSP;
  int	_m_timerId;
  bool	_m_isPlaying;
  bool	_m_isPaused;
public:
  ClockEditor(QWidget *parent);
  virtual ~ClockEditor() {}
  int	startFrame() const;
  int	endFrame() const;
  int	curFrame() const;
  void	update() const;
  void	setCurFrame(const int f);
  void	timerEvent(QTimerEvent *e);
  FrameSlider	&frameSlider() { return *_m_frameSlider; }
  virtual void	keyPressEvent(QKeyEvent *e);
public slots:
  void	cmdPlay();
  void	cmdPause();
  void	cmdStop();
  void	passrepaintglwidget() { emit repaintglwidget(); }
  void	setValueStartFrame(int value);
  void	setValueEndFrame(int value);
signals:
  void	repaintglwidget();
};

// ************************************************************************* //
class FrameSlider: public QWidget
{
  Q_OBJECT
protected:
  ClockEditor	*_m_parent;
public:
  FrameSlider(ClockEditor *parent);
  virtual ~FrameSlider() {}
  virtual void	paintEvent(QPaintEvent *e);
  void	drawRuler(QPainter &p);
  virtual QSizePolicy	sizePolicy() const
  { return QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding); }
  virtual void	mousePressEvent(QMouseEvent *e);
  virtual void	mouseMoveEvent(QMouseEvent *e);
signals:
  void	repaintglwidget();
};

// ************************************************************************* //
#endif//__QTCLOCKEDITOR_HH__
