// ************************************************************************* //
/// GBB - Graphics Blue Box, by Alexis Angelidis
/// $Author: silex $
/// $Date: 2006/01/11 21:56:30 $
/// $Revision: 1.1 $
// ************************************************************************* //
#ifndef	__QTCURVEEDITOR_HH__
#define	__QTCURVEEDITOR_HH__

// ************************************************************************* //
#include <qwidget.h>
#include <qmainwindow.h>
#include <QMouseEvent>
#include "gfxtypes.h"
#include "mystring.h"

// ************************************************************************* //
class DagNode;
class CEDataItem;
class NCurve;
class CECurveView;
class CEDataView;

// ************************************************************************* //
/// DESCRIPTION: Call onto a class, to show its attributes.
///  Uses datablock Connect obj with as many spinboxes as required.
// ************************************************************************* //
class CurveEditor: public QMainWindow
{
  DagNode	*_m_node;
protected:
  virtual void	paintEvent (QPaintEvent *);
public:
  CurveEditor(DagNode &node, QWidget *parent = 0);
  virtual ~CurveEditor();
  virtual void keyPressEvent(QKeyEvent *e);
};

// ************************************************************************* //
class CurveCentralWidget: public QWidget
{
  CECurveView	*_m_curveview;
  CEDataView	*_m_dataview;
public:
  CurveCentralWidget(DagNode &node, QWidget *parent = 0);
  CECurveView	&curveview() { return *_m_curveview; }
  CEDataView	&dataview() { return *_m_dataview; }
};

// ************************************************************************* //
class CECurveView: public QWidget
{
  Q_OBJECT
  DagNode	*_m_node;
  Pnt2d		_m_min;//lower left
  Pnt2d		_m_max;//upper right
  mutable QMouseEvent	_m_e0;
  CEDataItem	*_m_dataItem;
public:
  CECurveView(DagNode &node, QWidget *parent = 0);
  virtual void	paintEvent(QPaintEvent *e);
  virtual QSizePolicy	sizePolicy() const
  { return QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding); }
  virtual void	mousePressEvent(QMouseEvent *e);
  virtual void	mouseMoveEvent(QMouseEvent *e);
protected:
  virtual void	keyPressEvent(QKeyEvent *e);
  Pnt2i	domainToWindow(const Pnt2d &p) const;
  int	domainWidthToWindow(const double d) const;
  int	domainHeightToWindow(const double d) const;
  Pnt2d		windowToDomain(const Pnt2i &p) const;
  double	windowWidthToDomain(const int px) const;
  double 	windowHeightToDomain(const int py) const;
  void		drawGrid(QPainter &p);
  void		drawCurve(QPainter &p);
public slots:
  void	setCurve(CEDataItem *c);
  void	frameView();
};

// ************************************************************************* //
class CEDataView: public QWidget
{
  Q_OBJECT
  DagNode	*_m_node;
  std::list<CEDataItem *>	_m_dataItems;
public:
  CEDataView(DagNode &node, QWidget *parent = 0);
signals:
  void	curveChanged(CEDataItem *c);
};

// ************************************************************************* //
class CEDataItem: public QWidget
{
  Q_OBJECT
  NCurve	*_m_curve;
  bool		_m_isSelected;
  String	_m_name;
public:
  CEDataItem(NCurve &curve, QWidget *parent = 0, const char *name = 0);
  virtual void		mousePressEvent(QMouseEvent *e);
  virtual NCurve	&node();
  void	select(bool on);
protected:
  virtual void	paintEvent(QPaintEvent *e);
signals:
  void	curveChanged(CEDataItem *c);
};

// ************************************************************************* //
#endif//__QTCURVEEDITOR_HH__
