// ************************************************************************* //
/// GBB - Graphics Blue Box, by Alexis Angelidis
/// $Author: silex $
/// $Date: 2006/01/11 21:56:30 $
/// $Revision: 1.1 $
// ************************************************************************* //
#ifndef __QTGLWIDGET_HH__
#define __QTGLWIDGET_HH__

// ************************************************************************* //
#include <vector>
#include <list>
#include <string>
#include <QGLWidget>
#include <qimage.h>
#include <qevent.h> 
#include <qspinbox.h>
#include <qlineedit.h>
#include "glTrackball.h"
#include "gfxtypes.h"

// ************************************************************************* //
class NScene;
class NClock;
class SceneGLWidget;
class ClockEditor;

// ************************************************************************* //
class CentralWidget: public QWidget
{
  Q_OBJECT
protected:
  SceneGLWidget	*_m_glwidget;
  ClockEditor	*_m_clockEditor;
public:
  CentralWidget(QWidget *parent);
  virtual ~CentralWidget() {}
  SceneGLWidget	*glwidget() { return _m_glwidget; }
  ClockEditor	*clockEditor() { return _m_clockEditor; }
};

// ************************************************************************* //
class CommandEditor: public QLineEdit
{
  Q_OBJECT
  std::list<QString>	_m_history;
  std::list<QString>::iterator	_m_hit;
public:
  CommandEditor(QWidget *parent);
  virtual void keyPressEvent(QKeyEvent *e);
signals:
  void	repaintglwidget();
};

// ************************************************************************* //
class SceneGLWidget: public QGLWidget
{
  Q_OBJECT
protected:
  TrackBall	_m_trackball;
  View		_m_view;
  bool		_m_isInViewMode;
  mutable QMouseEvent	_m_e0;
  bool			_m_mouseHasDragged;
  GLuint		_m_z0;
  void			keyPressEvent(QKeyEvent *e);
  QGLContext		*_m_context;
public:
  SceneGLWidget(QWidget *parent = 0);
  virtual ~SceneGLWidget() {}
  void	initializeGL();
  void	resizeGL(int w, int h);
  void	paintGL();
  void	mouseMoveEvent(QMouseEvent *e);
  void	mousePressEvent(QMouseEvent *e);
  void	mouseReleaseEvent(QMouseEvent *e);
  QGLContext	*glcontext() { return _m_context; }
public slots:
  void	setGouraud();
  void	setWireframe();
  void	resetView();
  bool	saveView();
  bool	loadView();
signals:
  void	repaintclockeditor();
  void	repaintframeslider();
};

// ************************************************************************* //
inline void	SceneGLWidget::resizeGL(int	w,
					int	h)
{
  glViewport(0, 0, w, h);
  _m_view.glGet();
  _m_trackball.resize();
}

// ************************************************************************* //
#endif//__QTGLWIDGET_HH__
