// ************************************************************************* //
/// GBB - Graphics Blue Box, by Alexis Angelidis
/// $Author: silex $
/// $Date: 2006/01/11 21:56:30 $
/// $Revision: 1.1 $
// ************************************************************************* //
#ifndef __QTMAINWIDGET_HH__
#define __QTMAINWIDGET_HH__

// ************************************************************************* //
#include <QMainWindow> 
#include <QApplication> 
#include <QDesktopWidget>
#include <QMenu>
#include <QMenuBar>
#include <QGLWidget>
#include <QPainter>
#include "qtGlWidget.h"
#include "command.h"
#include "gbblogo.h"
#include "gbb.h"

// ************************************************************************* //
class RenderEditor;
class SceneGLWidget;
class ClockEditor;

// ************************************************************************* //
class MainWindow: public QMainWindow
{
  Q_OBJECT
  QMenu	*_m_shading;
  QMenu	*_m_new;
  QMenu	*_m_view;
  QMenu	*_m_edit;
  QAction *_m_easyGizmo;
  RenderEditor	*_m_rendereditor;
  bool		_m_isFrozen;
  SceneGLWidget	*_m_glwidget;
  ClockEditor	*_m_clockEditor;
public:
  MainWindow(QWidget *parent = 0);
  virtual ~MainWindow() {}
  static MainWindow	*&mainwindow() { static MainWindow *mw(0); return mw; }
  SceneGLWidget		*glwidget() { return _m_glwidget; }
  ClockEditor		*clockEditor() { return _m_clockEditor; }
public slots:
  void	newScene();
  void	openScene();
  void	saveScene();
  void	exportParticles();
  void	quit();
  void	fixScene();
  void	render();
  void	setKey() { Command::run("setkey"); }
  void	remKey() { Command::run("remkey"); }
  void	editSmokeParticles() { Command::run("edit -path :SmokeParticles0"); }
  void	editSmokeGPUParticles()
  { Command::run("edit -path :SmokeGPUParticles0"); }
  void	editVortexSet() { Command::run("edit -path :VortexSet0"); }
  void	editSky() { Command::run("edit -path :Sky0"); }
  void	editFrontLayer() { Command::run("edit -path :ImageLayer0"); }
  void	editBackLayer() { Command::run("edit -path :ImageLayer1"); }
  void	bindToControl(){ Command::run("bindvortexgenerator"); }
  void	attachControls(){ Command::run("attachcontrol"); }
  void	parent() { Command::run("parent"); }
  void	newGuide();
  void	newCurrentGenerator();
  void	newSmokeGenerator();
  void	newSmokeGPUGenerator();
  void	newNoiseGenerator() { Command::run("new -type Noise"); }
  void	newTurbulence() { Command::run("new -type Turbulence"); }
  void	newControlGenerator() { Command::run("controlgenerator"); }
  void	newCurrentControl();
  void	newRingCheckpoint();
  void	newWind() { Command::run("new -type Wind -path :"); }
  void	newCamera() { Command::run("new -type Camera -path :"); }
  void	newAimCamera() { Command::run("newcamera"); }
  void	setWindowTitle(const QString &s);
  void	createObstacle() { Command::run("new -type Obstacle"); }
  void	bakeObstacle() { Command::run("bakeobstacle"); }
  void	freeze() { _m_isFrozen = true; }
  void	warmup() { _m_isFrozen = false; }
  bool	isFrozen() const { return _m_isFrozen; }
  void	slotEasyGizmo();
};

// ************************************************************************* //
/// DESCRIPTION: App startup screen.
// ************************************************************************* //
class FrontPanel: public QWidget
{
  Q_OBJECT
  QImage	_m_image;
protected:
  void	timerEvent(QTimerEvent *event);
public:
  FrontPanel(QWidget *parent = 0, Qt::WFlags f = 0);
  virtual void	paintEvent(QPaintEvent *e);
};

// ************************************************************************* //
inline void	FrontPanel::timerEvent(QTimerEvent *)//event
{
  close();
}

// ************************************************************************* //
inline FrontPanel::FrontPanel(QWidget		*parent,
			      Qt::WFlags	f):
  QWidget(parent, f), _m_image(gbblogo_xpm)
{
  setMinimumSize(_m_image.width(), _m_image.height());
  setMaximumSize(_m_image.width(), _m_image.height());
  move((QApplication::desktop()->width() - _m_image.width()) / 2,
       (QApplication::desktop()->height() - _m_image.height()) / 2);
  setWindowTitle("GraphicsBlueBox");
  startTimer(2000);
}

// ************************************************************************* //
#endif//__QTMAINWIDGET_HH__
