// ************************************************************************* //
/// AUTHOR Alexis Angelidis
/// STARTED Sat Sep 10 09:40:24 NZST 2005
// ************************************************************************* //
//#ifndef _WIN32
//#include <GL/glx.h>
//#else //_WIN32
//#include <GL/glext.h>
//#ifdef _WIN32
//#include "glext.h"
//#endif//_WIN32
//#endif//_WIN32
#define GLX_GLXEXT_PROTOTYPES
#include <iostream>
#include <QPaintEngine>
#include <QMenuBar>
#include <QGridLayout>
#include <QRadioButton>
#include <QPainter>
#include <QFileDialog>
#include <QApplication>
#include "qtRenderEditor.h"
#include "dagnode.h"
#include "glView.h"
#include "dataclock.h"
#include "plug.h"
#include "gbb.h"
#include <GL/gl.h>
#include <GL/glu.h>

#ifdef _WIN32
#include "glext.h"
#else//LINUX
#include <GL/glx.h>
#endif//_WIN32

#include "parcel7.dat"
#include <fstream>
#include <QImageReader>
#include <QByteArray>

// ************************************************************************* //
DagNode	*RenderEditor::_m_renderCamera(0);

// ************************************************************************* //
RenderEditor::RenderEditor(const QGLContext	*shareContext, 
			   QWidget		*parent): QMainWindow(parent)
{
  setWindowTitle("RenderEditor");
 
  //QGLContext	*context = 0;

  _m_renderwidget = new RenderWidget(shareContext, this);

  //QGLContext ( const QGLFormat & format, QPaintDevice * device )

  setCentralWidget(_m_renderwidget);

  QMenu	*file = menuBar()->addMenu(tr("&File"));

  file->addAction("&Save", _m_renderwidget, SLOT(saveImage()),
		  Qt::CTRL + Qt::Key_S);
  file->addAction("&Close", this, SLOT(hide()), Qt::CTRL + Qt::Key_Q);

  QMenu	*edit = menuBar()->addMenu(tr("&Edit"));

  edit->addAction("&Preferences", _m_renderwidget,
		  SLOT(editPreferences()), Qt::CTRL + Qt::Key_A);

  QMenu	*render = menuBar()->addMenu(tr("&Render"));

  render->addAction("&Single Frame", _m_renderwidget,
		    SLOT(renderFrame()), Qt::CTRL + Qt::Key_R);
  render->addAction("&All Frames", _m_renderwidget,
		    SLOT(renderAnimation()));

  setBaseSize(320, 240);
  setMaximumSize(320, 240);
  hide();
}

// ************************************************************************* //
/// DESCRIPTION: Returns the camera used for rendering.
// ************************************************************************* //
DagNode	&RenderEditor::getRenderCamera()
{
  std::list<DagNode*>	cameras;
  std::list<DagNode*>::iterator	cit;
  DagNode	*rcam(0);
  
  DagNode::root().getNodesByType("Camera", cameras);
  if (cameras.empty())
    {
      std::cerr << "Error: no camera defined." << std::endl;
      exit(-1);
    }
  rcam = cameras.front();
  for (cit = cameras.begin(), ++cit; cit != cameras.end(); ++cit)
    if ((*cit)->readAsBool("rendering"))
      return **cit;
  return *rcam;
}

// ************************************************************************* //
RenderWidget::RenderWidget(const QGLContext	*shareContext, 
			   QWidget		*parent):
  QGLWidget(parent), _m_image(QSize(320, 240), QImage::Format_ARGB32),
  _m_renderFrame(false)
{
  _m_context = new QGLContext(shareContext->format(), this);

  _m_context->create(shareContext);
  if (_m_context->isSharing())
    std::cout << "Sharing context" << std::endl;
  _m_image.fill(0);
  _m_renderpreferences = new RenderPreferences();
}

// ************************************************************************* //
void	RenderWidget::resizeGL(int	w,
							   int	h)
{
  _m_image = _m_image.scaled(w, h, Qt::IgnoreAspectRatio,
			     Qt::SmoothTransformation);
}

// ************************************************************************* //
void	RenderWidget::initializeGL()
{
  const GLfloat	specular[] = {1.0, 1.0, 1.0, 1.0};
  const GLfloat	shininess[] = {50.0};

  glClearColor(0.647, 0.647, 0.647, 0.0);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
  glEnable(GL_CULL_FACE);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_NORMALIZE);

  glShadeModel(GL_SMOOTH);
  glEnable(GL_COLOR_MATERIAL);
  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
  glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);
  glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);

  const GLfloat	lDiffuse0[] = {0.1f, 0.1f, 0.1f, 1.0f};
  const GLfloat	lDiffuse1[] = {0.6f, 1.0f, 0.6f, 1.0f};
  const GLfloat	lDiffuse2[] = {1.0f, 0.6f, 0.6f, 1.0f};
  const GLfloat	lDiffuse3[] = {1.0f, 1.0f, 1.0f, 1.0f};

  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_LIGHT1);
  glEnable(GL_LIGHT2);
  glEnable(GL_LIGHT3);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, lDiffuse0);
  glLightfv(GL_LIGHT1, GL_DIFFUSE, lDiffuse1);
  glLightfv(GL_LIGHT2, GL_DIFFUSE, lDiffuse2);
  glLightfv(GL_LIGHT2, GL_DIFFUSE, lDiffuse3);
  glPointSize(4.0);
}

// ************************************************************************* //
void	RenderWidget::makeCurrent()
{
  //QGLWidget::makeCurrent();
  
  _m_context->makeCurrent();
  std::cout << "???" << std::endl;
}

// ************************************************************************* //
namespace {

// Handle to the framebuffer object
GLuint	fbo = 0;
// Handle to the texture used as color buffer of the fbo
GLuint	fbo_texture = 0;

// ************************************************************************* //
#define GL_FRAMEBUFFER_EXT		0x8D40
#define GL_COLOR_ATTACHMENT0_EXT	0x8CE0

// Function types ********************************************************** //
typedef void (APIENTRY * PFNGLGENRENDERBUFFERSEXTPROC)
  (GLsizei n, GLuint *renderbuffers);
typedef void (APIENTRY * PFNGLBINDFRAMEBUFFEREXTPROC)
  (GLenum target, GLuint framebuffer);
typedef GLboolean (APIENTRY * PFNGLISFRAMEBUFFEREXTPROC)
  (GLuint framebuffer);
typedef void (APIENTRY * PFNGLFRAMEBUFFERTEXTURE2DEXTPROC)
  (GLenum target, GLenum attachment, GLenum textarget, GLuint texture,
   GLint level);
typedef void (APIENTRY * PFNGLGENFRAMEBUFFERSEXTPROC)
  (GLsizei n, GLuint *framebuffers);
typedef void (APIENTRY * PFNGLDELETEFRAMEBUFFERSEXTPROC)
  (GLsizei n, const GLuint *framebuffers);

// Pointers to functions *************************************************** //
PFNGLGENRENDERBUFFERSEXTPROC	glGenRenderbuffersEXT;
PFNGLBINDFRAMEBUFFEREXTPROC	glBindFramebufferEXT;
PFNGLISFRAMEBUFFEREXTPROC	glIsFramebufferEXT;
PFNGLFRAMEBUFFERTEXTURE2DEXTPROC	glFramebufferTexture2DEXT;
PFNGLGENFRAMEBUFFERSEXTPROC	glGenFramebuffersEXT;
PFNGLDELETEFRAMEBUFFERSEXTPROC	glDeleteFramebuffersEXT;
PFNGLACTIVETEXTUREARBPROC	glActiveTextureARB;

// ************************************************************************* //
void	*getProcAddress(const char *name)
{
#ifdef _WIN32
  return (void*)wglGetProcAddress((LPCSTR)name);
#else// LINUX
  return (void*)glXGetProcAddressARB((const GLubyte*)name);
#endif
}

// ************************************************************************* //
/// DESCRIPTION:
///  Return false if Fbo-s can't be initialized.
///  Thanks to Christian Lessig <lessig@dgp.toronto.edu> for this function.
// ************************************************************************* //
bool	initFbo(const unsigned w, 
		const unsigned h)
{
  static bool		isFboInit(false);
  static bool		isFboAvailable(false);
  static unsigned	width(0), height(0);
  GLuint	old_fbo_texture = 0;

  if (w != width || h != height)
    isFboInit = false;
  width = w;
  height = h;
  if (!isFboInit)
    {
      old_fbo_texture = fbo_texture;
      glDeleteTextures(1, &fbo_texture);
      fbo_texture = 0;
      /*
	if ((glDeleteFramebuffersEXT = (PFNGLDELETEFRAMEBUFFERSEXTPROC)
	getProcAddress("glDeleteFramebuffersEXT")))
	glDeleteFramebuffersEXT(1, &fbo);
	fbo = 0;
      */
      isFboInit = true;
      if (!(glGenFramebuffersEXT = (PFNGLGENFRAMEBUFFERSEXTPROC)
	    getProcAddress("glGenFramebuffersEXT")) ||
	  !(glBindFramebufferEXT = (PFNGLBINDFRAMEBUFFEREXTPROC)
	    getProcAddress("glBindFramebufferEXT")) ||
	  !(glGenRenderbuffersEXT = (PFNGLGENRENDERBUFFERSEXTPROC)
	    getProcAddress("glGenRenderbuffersEXT")) ||
	  !(glIsFramebufferEXT = (PFNGLISFRAMEBUFFEREXTPROC)
	    getProcAddress("glIsFramebufferEXT")) ||
	  !(glFramebufferTexture2DEXT = (PFNGLFRAMEBUFFERTEXTURE2DEXTPROC)
	    getProcAddress("glFramebufferTexture2DEXT")))
	return (isFboAvailable = false);
#ifdef _WIN32
      if (!(glActiveTextureARB = (PFNGLACTIVETEXTUREARBPROC)
	    getProcAddress("glActiveTextureARB")))
	return (isFboAvailable = false);
#endif//_WIN32
      glGenFramebuffersEXT(1, &fbo);
      glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fbo);
      if (!glIsFramebufferEXT(fbo))
	{
	  std::cerr << "'glIsFramebufferEXT' failed." << std::endl;
	  return (isFboAvailable = false);
	}

      glGenTextures(1, &fbo_texture);
#ifdef _WIN32
      glActiveTextureARB(GL_TEXTURE0 + fbo_texture);
#else// LINUX
      glActiveTexture(GL_TEXTURE0 + fbo_texture);
#endif//_WIN32

      glBindTexture(GL_TEXTURE_RECTANGLE_NV, fbo_texture);
      glTexImage2D(GL_TEXTURE_RECTANGLE_NV,
		   0, GL_RGBA, w, h, 
		   0, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, NULL);

      // Set the filtering parameter (it's a must) ************************* //
      glTexParameteri(GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_WRAP_S,
		      GL_CLAMP_TO_EDGE);
      glTexParameteri(GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_WRAP_T,
		      GL_CLAMP_TO_EDGE);
      glTexParameteri(GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_MIN_FILTER,
		      GL_NEAREST);
      glTexParameteri(GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_MAG_FILTER,
		      GL_NEAREST);

      if (old_fbo_texture)
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT,
				  GL_TEXTURE_RECTANGLE_NV, 0, 0);

      // Attach the texture as color attachment to the fbo ***************** //
      glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT,
				GL_TEXTURE_RECTANGLE_NV, fbo_texture, 0);

      // Turn off everything for now *************************************** //
      glBindTexture(GL_TEXTURE_RECTANGLE_NV, 0);
#ifdef _WIN32
      glActiveTextureARB(GL_TEXTURE0);
#else// LINUX
      glActiveTexture(GL_TEXTURE0);
#endif//_WIN32
      glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
      isFboAvailable = true;
    }
  return isFboAvailable;
}

// namespace *************************************************************** //
}

// ************************************************************************* //
/// DESCRIPTION: To call when program exits.
// ************************************************************************* //
void	deleteFbo()
{
	glDeleteTextures(1, &fbo_texture);
	if ((glDeleteFramebuffersEXT = (PFNGLDELETEFRAMEBUFFERSEXTPROC)
		getProcAddress("glDeleteFramebuffersEXT")))
		glDeleteFramebuffersEXT(1, &fbo);
}

// ************************************************************************* //
/// DESCRIPTION: 
/// Note: When "nosky" is enabled, in order to get no color behind smoke
///  the screen is cleared with smoke color... doesn't work if two generators
///  of different colors. Dodgy with self-shadowing effect.
///  Ideally, when previous alpha is 0, color should be ignored. But reading 
///  the previous value is way too expensive.
// ************************************************************************* //
void	RenderWidget::paintGL()
{
  const Mat2i	view(0, rect().width(), 0, rect().height());
  bool	isFboAvailable = initFbo(_m_image.width(), _m_image.height());

  if (_m_renderFrame)
    {
	  if (isFboAvailable)
	  {
		// Be sure that the texture we are rendering to is not bound
		glBindTexture(GL_TEXTURE_RECTANGLE_NV, 0); 
		// Enable the fbo
		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fbo);
		// Setup target for rendering
		glDrawBuffer(GL_COLOR_ATTACHMENT0_EXT);
	  }
	  else
	  {
		  std::ofstream	logfbo("logfbo.txt");

		  logfbo << "To get logs on Fbo, delete this file before running GBB."
			     << std::endl;
		  logfbo << "Fbo are not available: plase install more recent drivers."
				 << std::endl;
	  }

	  static const GLfloat	lPosition0[] = {-0.5f, 0.5f, -1.0f, 0.0f};
      static const GLfloat	lPosition1[] = {1.2f, 0.0f, 1.0f, 0.0f};
      static const GLfloat	lPosition2[] = {-1.2f, 0.0f, 1.0f, 0.0f};
      static const GLfloat	lPosition3[] = {0.0f, 0.0f, 1.0f, 0.0f};
      const bool	renderAll = DagNode::root().readAsBool("renderAll");
      DagNode	&camera = RenderEditor::getRenderCamera();
      DagNode	*imageLayer = DagNode::getNodeByPath(":ImageLayer0");
      DagNode	*smokesp = DagNode::getNodeByPath(":SmokeParticles0");
      DagNode	*smokeGPUset = DagNode::getNodeByPath(":SmokeGPUParticles0");
      DagNode	*sky = DagNode::getNodeByPath(":Sky0");

      // Clear screen ****************************************************** //
      glClearColor(0, 0, 0, 0);
      if (sky->readAsBool("noSky") && smokesp != 0)
	{
	  std::list<DagNode *>	smokeg;

	  DagNode::root().getNodesByType("SmokeGenerator", smokeg);
	  if (!smokeg.empty())
	    {
	      const Rgb3d	col = smokeg.front()->readAsRgb3d("color");

	      glClearColor(col.r, col.g, col.b, 0);
	    }
	}
      glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
      
      // Draw scene ******************************************************** //
      const Mat4d	&model = camera.readAsMat4d("model");

      view.glLoad();

      glPushAttrib(GL_TRANSFORM_BIT);
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      gluPerspective(50.0, camera.readAsInt("width") /
		     double(camera.readAsInt("height")), 0.1, 1000.0);
      glPopAttrib();// GL_TRANSFORM_BIT

      glPushAttrib(GL_TRANSFORM_BIT);
      glMatrixMode(GL_MODELVIEW);
      glLoadIdentity();
      glLightfv(GL_LIGHT0, GL_POSITION, lPosition0);
      glLightfv(GL_LIGHT1, GL_POSITION, lPosition1);
      glLightfv(GL_LIGHT2, GL_POSITION, lPosition2);
      glLightfv(GL_LIGHT3, GL_POSITION, lPosition3);
      model.glLoad(GL_MODELVIEW);
      glPopAttrib();// GL_TRANSFORM_BIT

      Vec3d	dir, up;
      Frustum	fs;

      dir.glGetViewdir();
      up.glGetUp();
      fs.getWorldFrustum();

      camera.writeAsVec3d("up", up);
      camera.writeAsVec3d("dir", dir);
      camera.writeAsCustom("frustum", &fs);
      camera.writeAsMat2i("view", view);
      camera.writeAsBool("isVisible", false);

      DagNode::disconnect(*sky->getPlug("camera")->getInputNode(), "signal",
			  *sky, "camera");
      DagNode::connect(camera, "signal", *sky, "camera");

      DagNode::disconnect(*imageLayer->getPlug("camera")->getInputNode(),
			  "signal", *imageLayer, "camera");
      DagNode::connect(camera, "signal", *imageLayer, "camera");

      smokesp->writeAsPnt3d("eye", camera.readAsPnt3d("eye"));
      smokeGPUset->writeAsPnt3d("eye", camera.readAsPnt3d("eye"));

      if (renderAll)
		  DagNode::root().writeAsInt("renderIn", Gbb::Modeling);
      else
		  DagNode::root().writeAsInt("renderIn", Gbb::Rendering);

      DagNode::root().writeAsSignal("renderIn");
      DagNode::root().readAsSignal("renderOut");
      camera.writeAsBool("isVisible", true);
      glFlush();

	  if (isFboAvailable)
		  glReadBuffer(GL_COLOR_ATTACHMENT0_EXT);
	  else
		  glReadBuffer(GL_BACK);
      glReadPixels(0, 0, _m_image.width(), _m_image.height(), GL_BGRA, 
		   GL_UNSIGNED_BYTE, _m_image.bits());
      if (!sky->readAsBool("noSky"))
		  for (int i(3), n(4 * _m_image.width() * _m_image.height()); i < n;
			  i += 4)
			  _m_image.bits()[i] = 255;
      _m_renderFrame = false;
    }
	if (!_m_renderFrame)
	  if (isFboAvailable)
		{
			// Render to default framebuffer ******************************* //
			glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
			glDrawBuffer(GL_BACK);

			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

			//glActiveTexture(GL_TEXTURE0 + fbo_texture);
			glEnable(GL_TEXTURE_RECTANGLE_NV);
			glBindTexture(GL_TEXTURE_RECTANGLE_NV, fbo_texture);

			//glReadBuffer(GL_BACK);
			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

			glMatrixMode(GL_PROJECTION);
			glPushMatrix();
			glLoadIdentity();
			glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);
			glMatrixMode(GL_MODELVIEW);
			glPushMatrix();
			glLoadIdentity();

			// Draw a quad, if it's red something is wrong ***************** //
			glColor3d(1.0, 0.0, 0.0);

			glBegin(GL_QUADS);
			glTexCoord2d(0.0, 0.0);
			glVertex3d(-1.0, -1.0, 0.0);
			glTexCoord2d(_m_image.width(), 0.0);
			glVertex3d(1.0, -1.0, 0.0);
			glTexCoord2d(_m_image.width(), _m_image.height());
			glVertex3d(1.0, 1.0, 0.0);
			glTexCoord2d(0.0, _m_image.height());
			glVertex3d(-1.0, 1.0, 0.0);
			glEnd();//GL_QUADS
			glPopMatrix();
			glMatrixMode(GL_PROJECTION);
			glPopMatrix();
			glFlush();

			glDisable(GL_TEXTURE_RECTANGLE_NV);
			glBindTexture(GL_TEXTURE_RECTANGLE_NV, 0);
		}
		else
		{
			glDrawBuffer(GL_BACK);
			glClearColor(0, 0, 0, 0);
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			glDrawPixels(_m_image.width(), _m_image.height(), GL_BGRA,
				GL_UNSIGNED_BYTE, _m_image.bits());
		}
	glFlush();
}

// ************************************************************************* //
void	RenderEditor::keyPressEvent(QKeyEvent *e)
{
  if (e->key() == Qt::Key_Escape)
    _m_renderwidget->cancel();
}

// ************************************************************************* //
void	RenderWidget::paintEvent(QPaintEvent *e)
{
  DagNode	&camera = RenderEditor::getRenderCamera();

  setMinimumSize(camera.readAsInt("width"), camera.readAsInt("height"));
  raise();
  QGLWidget::paintEvent(e);
  if (_m_renderFrame)
    //{
      /*
#ifdef _WIN32
	QPixmap pixmap;

	pixmap = renderPixmap(camera.readAsInt("width"),
	camera.readAsInt("height"), true);
	_m_image = pixmap.toImage().mirrored(false, true);
      //_m_image = grabFrameBuffer().mirrored(false, true);
#else
      _m_image = grabFrameBuffer().mirrored(false, true);
      QImage	alphaChannel = _m_image.alphaChannel();
      DagNode	*sky = DagNode::getNodeByPath(":Sky0");

      if (!sky->readAsBool("noSky"))
	alphaChannel.fill(255);
      _m_image.setAlphaChannel(alphaChannel);
#endif//_WIN32
      */
      _m_renderFrame = false;
  //}
}

// ************************************************************************* //
void	RenderWidget::saveImage()
{
  QString s = QFileDialog::getSaveFileName(this, "save file dialog", ".",
					   "Image (*.jpg)");

  if (s.isEmpty())
    return;
  if (!s.contains(".jpg"))
    s = s + ".jpg";
  _m_image.mirrored(false, true).save(s, "JPG", 100);
}

// ************************************************************************* //
void	RenderWidget::renderFrame()
{
  _m_renderFrame = true;
  repaint();
  updateGL();
}

// ************************************************************************* //
/// DESCRIPTION: 
/// NOTE: Shading is done once per simulation step, by pulling update.
///  So the shading time is stored in the scene, and added to the rendering 
///  time and removed from the physical simulation time.
// ************************************************************************* //
void	RenderWidget::renderAnimation()
{
  DagNode	*clockNode = DagNode::getNodeByPath(":Clock0");
  const int	startf = clockNode->readAsInt("sf");
  const int	endf = clockNode->readAsInt("ef");
  Clock		clock(Clock::CPUTime);
  double		dt, dtFrame, sumdtSim(0.0), sumdtRen(0.0);
  double		shadingTime;
  //DagNode	*smokesp = DagNode::getNodeByPath(":SmokeParticles0");

  std::cout << "rendering frame " << startf << " to " << endf << std::endl;
  std::cout << "Time is CPU:" << std::endl;
  _m_wasCanceled = false;
  for (int f(startf); f < endf; ++f)
    {
      qApp->processEvents();
      if (wasCanceled())
	break;

      const QString fileName = QString("anim") + 
	QString("%1%2%3%4").arg(f / 1000).arg((f % 1000) / 100).
	arg((f % 100) / 10).arg(f % 10) + QString(".jpg");

      std::cout << fileName.toAscii().data() << std::flush;
      setWindowTitle(QString("RenderEditor: ") + fileName);
      dtFrame = 0.0, clock.tic();
      clockNode->writeAsInt("f", f);
      DagNode::root().readAsSignal("update");
      shadingTime = DagNode::root().readAsDouble("shadingTime");
      _m_renderFrame = true;
      dt = clock.tic();
      dtFrame += dt;
      sumdtSim += dt - shadingTime;
      repaint();
      dt = clock.tic();
      dtFrame += dt;
      sumdtRen += dt + shadingTime;
      std::cout << " " << dtFrame << "s" << std::endl;
      _m_image.mirrored(false, true).save(fileName, "jpg", 100);
      setFocus();
    }
  if (wasCanceled())
    std::cout << "Canceled" << std::endl;
  else
    {
      std::cout << "Average = " << (sumdtSim + sumdtRen) / (endf - startf)
		<< "s/frame = "
		<< (endf - startf) / (sumdtSim + sumdtRen)
		<< "fps" << std::endl;
      std::cout << rint(100 * sumdtSim / (sumdtSim + sumdtRen)) << "% physics"
		<< std::endl;
      std::cout << rint(100 * sumdtRen / (sumdtSim + sumdtRen))
		<< "% rendering" << std::endl;
    }
}

// ************************************************************************* //
/// DESCRIPTION: Widget to select the rendering camera.
// ************************************************************************* //
void	RenderWidget::editPreferences()
{
  if (_m_renderpreferences)
    {
      _m_renderpreferences->show();
      _m_renderpreferences->raise();
      _m_renderpreferences->setFocus();
      _m_renderpreferences->activateWindow();
      std::list<DagNode*>	cameras;
      std::list<DagNode*>::iterator	cit;

      DagNode::root().getNodesByType("Camera", cameras);

      QLayout	*lyt = _m_renderpreferences->centralWidget()->layout();
      int	i;
      std::list<QWidget*>	&children(_m_renderpreferences->children);

      while (!children.empty())
	{
	  lyt->removeWidget(children.back());
	  delete children.back();
	  children.pop_back();
	}
      for (cit = cameras.begin(), i = 0; cit != cameras.end(); ++cit, ++i)
	{
	  QRadioButton	*rbi = new QRadioButton
	    (QString("Camera %1").arg(i),
	     _m_renderpreferences->centralWidget());
	  CameraSelector *cameraselector = new CameraSelector(**cit);

	  QWidget::connect(rbi, SIGNAL(toggled(bool)),
			   cameraselector, SLOT(select(bool)));
	  
	  if ((*cit)->readAsBool("rendering"))
	    rbi->setDown(true);
	  children.push_back(rbi);
	  lyt->addWidget(rbi);//, Qt::AlignCenter | Qt::AlignVCenter
	}
    }
}

// ************************************************************************* //
RenderPreferences::RenderPreferences(QWidget *)//parent
{
  setWindowTitle("RenderPreferences");

  // Menu ****************************************************************** //

  QMenu	*file = menuBar()->addMenu(tr("&File"));

  file->addAction("&Close", this, SLOT(close()), Qt::CTRL + Qt::Key_Q);

  // Central *************************************************************** //
  QWidget	*w = new QWidget(this);

  setCentralWidget(w);
  new QVBoxLayout(w);
}

// ************************************************************************* //
CameraSelector::CameraSelector(DagNode &camera): _m_camera(&camera)
{
}

// ************************************************************************* //
void	CameraSelector::select(bool checked)
{
  _m_camera->writeAsBool("rendering", checked);
  std::cout << _m_camera->path() << ".rendering = "
	    << _m_camera->readAsBool("rendering") << std::endl;
}
