// ************************************************************************* //
/// AUTHOR: Alexis Angelidis 
/// STARTED: Mon Sep  6 16:23:16 NZST 2004
// ************************************************************************* //
#ifndef __QTSPINBOXPNT3D_HH__
#define __QTSPINBOXPNT3D_HH__

// ************************************************************************* //
#include <qwidget.h>
#include <qlayout.h>
#include <qspinbox.h>
#include "gfxtypes.h"
#include <QDoubleSpinBox>

// ************************************************************************* //
class SpinBoxPnt3d: public QWidget
{
  Q_OBJECT
  QDoubleSpinBox	*_m_s0, *_m_s1, *_m_s2;
public:
  SpinBoxPnt3d(double minValue, double maxValue, double step = 1,
	       QWidget *parent = 0);
  void	setRange(const double minValue, const double maxValue);
public slots:
  void	setValue(Pnt3d value);
  void	slotValueChanged(Pnt3d value);
  void	slotValueChanged0(double v0);
  void	slotValueChanged1(double v1);
  void	slotValueChanged2(double v2);
signals:
  void	valueChanged(Pnt3d value);
};

// ************************************************************************* //
inline void	SpinBoxPnt3d::setRange(const double minValue,
				       const double maxValue)
{
  _m_s0->setRange(minValue, maxValue);
  _m_s1->setRange(minValue, maxValue);
  _m_s2->setRange(minValue, maxValue);
}

// ************************************************************************* //
inline SpinBoxPnt3d::SpinBoxPnt3d(double	minValue,
				  double	maxValue,
				  double	,//step,
				  QWidget	*parent): QWidget(parent)
{
  QHBoxLayout	*grid = new QHBoxLayout(this);
  QFontMetrics	fm(parent->font());
  QRect		qs(fm.boundingRect("10000.0000"));

  _m_s0 = new QDoubleSpinBox(this);
  _m_s1 = new QDoubleSpinBox(this);
  _m_s2 = new QDoubleSpinBox(this);

  _m_s0->setRange(minValue, maxValue);
  _m_s0->setFrame(false);
  _m_s0->setDecimals(6);
  _m_s0->setBaseSize(qs.width() + 8, qs.height() + 8);
  _m_s0->setMinimumSize(qs.width() + 8, qs.height() + 8);
  _m_s0->setMaximumSize(qs.width() + 8, qs.height() + 8);

  _m_s1->setRange(minValue, maxValue);
  _m_s1->setFrame(false);
  _m_s1->setDecimals(6);
  _m_s1->setBaseSize(qs.width() + 8, qs.height() + 8);
  _m_s1->setMinimumSize(qs.width() + 8, qs.height() + 8);
  _m_s1->setMaximumSize(qs.width() + 8, qs.height() + 8);

  _m_s2->setRange(minValue, maxValue);
  _m_s2->setFrame(false);
  _m_s2->setDecimals(6);
  _m_s2->setBaseSize(qs.width() + 8, qs.height() + 8);
  _m_s2->setMinimumSize(qs.width() + 8, qs.height() + 8);
  _m_s2->setMaximumSize(qs.width() + 8, qs.height() + 8);

  grid->addWidget(_m_s0);
  grid->addWidget(_m_s1);
  grid->addWidget(_m_s2);
  grid->setMargin(1);
  grid->setSpacing(0);

  connect(_m_s0, SIGNAL(valueChanged(double)),
	  this, SLOT(slotValueChanged0(double)));
  connect(_m_s1, SIGNAL(valueChanged(double)),
	  this, SLOT(slotValueChanged1(double)));
  connect(_m_s2, SIGNAL(valueChanged(double)),
	  this, SLOT(slotValueChanged2(double)));
}

// ************************************************************************* //
inline void	SpinBoxPnt3d::setValue(Pnt3d value)
{
  _m_s0->setValue(value.x);
  _m_s1->setValue(value.y);
  _m_s2->setValue(value.z);
}

// ************************************************************************* //
inline void	SpinBoxPnt3d::slotValueChanged(Pnt3d value)
{
  std::cout << "in SpinBoxPnt3d::slotValueChanged = " << value << std::endl;
  emit valueChanged(value);
}

// ************************************************************************* //
inline void	SpinBoxPnt3d::slotValueChanged0(double v0)
{
  emit valueChanged(Pnt3d(v0, _m_s1->value(), _m_s2->value()));
}

// ************************************************************************* //
inline void	SpinBoxPnt3d::slotValueChanged1(double v1)
{
  emit valueChanged(Pnt3d(_m_s0->value(), v1, _m_s2->value()));
}

// ************************************************************************* //
inline void	SpinBoxPnt3d::slotValueChanged2(double v2)
{
  emit valueChanged(Pnt3d(_m_s0->value(), _m_s1->value(), v2));
}

// ************************************************************************* //
#endif//__QTSPINBOXPNT3D_HH__
